<?php
/* vim: set expandtab tabstop=2 shiftwidth=2 softtabstop=2: */

/**
 * Projectname: Aseco
 * !Modified by AssemblerManiac to fit the RASP plugin.
 * Also made some bug fixes and speed optimizations!
 *
 * PHP version 5.0
 *
 * LICENSE: This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * @author     Florian Schnell <floschnell at gmail.com>
 * @copyright  2006 - 2007
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL
 * @version    CVS: 1.2.0
 * @link       http://www.tmbase.de
 * @since      File available since Release 0.5.0
 */


// Include required classes ...
require_once('includes/basic.inc.php'); // contains standard functions
require_once('includes/xmlparser.inc.php'); // provides a xml parser
require_once('includes/GbxRemote.inc.php'); // needed for dedicated server communication

// load types ...
require_once('types/challenge.class.php');
require_once('types/chatcommand.class.php');
require_once('types/gameinfo.class.php');
require_once('types/player.class.php');
require_once('types/playerlist.class.php');
require_once('types/record.class.php');
require_once('types/recordlist.class.php');
require_once('types/plugin.class.php');
require_once('types/rpccall.class.php');
require_once('types/server.class.php');
require_once('types/event.class.php');
require_once('types/dependence.class.php');
require_once('types/manialinkpage.class.php');
require_once('types/manialinkframework.class.php');

/**
 * Error function
 * Report errors on a regular way.
 *
 * @param int $errno
 * @param string $errstr
 * @param string $errfile
 * @param int $errline
 */
error_reporting(E_ALL);
set_error_handler('displayError');
function displayError($errno, $errstr, $errfile, $errline) {
  switch ($errno) {
	case E_USER_ERROR:
	  $message = "[ASECO Error] $errstr on line $errline in file $errfile\r\n";
	  doLog($message);
		echo $message;
    die();
		break;
  case E_ERROR:
    $message = "[PHP Error] $errstr on line $errline in file $errfile\r\n";
	  doLog($message);
		echo $message;
    break;
	case E_WARNING:
		$message = "[PHP Warning] $errstr on line $errline in file $errfile\r\n";
	  doLog($message);
		echo $message;
		break;
	case E_USER_WARNING:
		$message = "[ASECO Warning] $errstr\r\n";
	  doLog($message);
		echo $message;
		break;
	default:
    // do nothing...
    // only treat known errors ...
	}
}

// Current project version.
define('ASECO_VERSION', '2.1.4 - tm-slowpokes.mod 1.1');
define('CRLF', "\r\n");
define('CR', "\r");
define('LF', "\n");

/**
 * Here Aseco actually starts.
 * Advanced coders can use this class
 * by including the document.
 * Make sure you cut out the lines below the class
 * and create your own instance.
 */
class Aseco {

  /**
   * Public fields.
   * Can be used by pugins.
   */
  public $debug;
  public $xml_parser;
  public $server;
  public $chat_commands;
  public $plugins;
  public $client;
  public $records;
  public $re_recs_count = 5;

  /**
   * Protected fields.
   * Need to be accessed by methods.
   */
  protected $login;
  protected $pass;
  protected $admin_list;
  protected $admin_groups;
  protected $script_timeout;
  protected $command;
  protected $events;
  protected $rpc_calls;
  protected $rpc_responses;
  protected $chat_colors;
  protected $settings;
  protected $silent;
  protected $startuptime;
  protected $chat_messages;


  /**
   * Initializes the server.
   *
   * @param bool $debug
   * @param string $config_file
   * @param bool $silent
   */
  function Aseco($debug, $config_file, $silent = false) {
    $this->console_text('# initialize ASECO ############################################################');

    // send php & mysql version info
    $this->console_text('[Aseco] PHP Version is '.phpversion());

    // initialize ...
    $this->plugins = array(); // plugin's storage ...
    $this->chat_commands = array();
    $this->debug = $debug;
    $this->client = new IXR_ClientMulticall_Gbx;
    $this->xml_parser = new Examsly();
    $this->server = new Server('127.0.0.1', 5000, 'SuperAdmin', 'SuperAdmin');
    $this->server->challenge = new Challenge();
    $this->server->players = new PlayerList();
    $this->admin_list = array();
    $this->silent = $silent;
    $this->records = null; // is set on sync ...
    $this->startuptime = time();

    // load new settings, if available ...
    $this->console_text('[Aseco] Load settings [{1}]', $config_file);
    $this->loadSettings($config_file);

	// initial ASECO info
	$this->console_text('###############################################################################');
    $this->console_text('#');
    $this->console_text('# Aseco v{1} running on {2}:{3}',
		ASECO_VERSION,
		$this->server->ip,
		$this->server->port);
    $this->console_text('#');
    $this->console_text('# Authors: Florian Schnell and Brad Mettee (RASP)');
    $this->console_text('#');
    $this->console_text('###############################################################################');

    // load plugins and register chat commands ...
    $this->loadPlugins();

	if ( file_exists('settings/admingroups.xml' ) )
		{
		$this->admin_groups = $this->xml_parser->parseXml('settings/admingroups.xml');
		$this->checkAuthItems();
		}

    // register the shutdown function ...
    register_shutdown_function(array(&$this, 'onShutdown'));
  }

	function checkAuthItems()
		{
		$added = false;
		$ctr = 0;
		foreach ($this->chat_commands as $cmd)
			{
			foreach ($cmd as $registered_command)
				{
				$item2check = strtoupper($registered_command->admingroup);
				// do we need admin access of any kind for this command?
	//			echo "{$registered_command->name} -> $item2check\r\n";
				if ( $item2check != '' )
					{
					if ( !isset($this->admin_groups['SETTINGS']['AUTHITEMS'][0][$item2check][0]) )
						{
						$this->admin_groups['SETTINGS']['AUTHITEMS'][0][$item2check][0] = 'false';
						$added = true;
						$ctr++;
						}
					}
				}
			}
		if ( $added )
			{
			$newxml = $this->xml_parser->parseArray($this->admin_groups);
			$handle = fopen('settings/admingroups.xml', 'w');
			fwrite($handle, $newxml);
			fclose($handle);
			$this->console_text('Added ' . $ctr . ' items to the authitem list, admingroups.xml updated');
			}
		else
			{
			$this->console_text('No authitems added');
			}

		}  //  checkAuthItems

  /**
   * Returns an object of base class Plugin.
   *
   * @param string $plugin_name
   * @return Plugin
   */
  function getPlugin($plugin_name) {
    return $this->plugins[$plugin_name];
  }


  /**
   * Unregisters all callbacks that are
   * connected to the command in a specific plugin.
   *
   * @param string $plugin_name
   * @param string $command_name
   */
  function unregisterChatCommand($plugin_name, $command_name) {
    for ($i = 0; $i < count($this->chat_commands[$command_name]); $i++) {
      if ($this->chat_commands[$plugin_name][$i]->plugin->getName() == $plugin_name) {
        unset($this->chat_commands[$plugin_name][$i]);
      }
    }
  }


  /**
   * Loads files in the plugins directory that
   * are listed in the plugins.xml.
   *
   */
  function loadPlugins() {

    // load and parse the plugins file ...
    if ($plugins = $this->xml_parser->parseXml('settings/plugins.xml')) {

      // take each plugin tag ...
      foreach ($plugins['ASECO_PLUGINS']['PLUGIN'] as $plugin) {

        // reset plugin variables ...
        unset($_PLUGIN);

        // and include the value between ...
        require_once('plugins/'.$plugin);

        // showup message

        $this->console_text('[Load Plugin] [{1}] [Name:{2}] [Ver:{3}]',
                            $plugin,
                            $_PLUGIN->getName(),
                            $_PLUGIN->getVersion());

        // load only plugins that were configured right ...
        if (!isset($_PLUGIN) || get_parent_class($_PLUGIN) != 'Plugin') {
			trigger_error('$_PLUGIN was not set or was set in a wrong way in the file "'.$plugin.'".'.LF
                        .'This is probably an old version of the plugin, update it or remove it from '
                        .'the plugins.xml.',
                        E_USER_ERROR);
        }

        // set plugin's silent state to aseco's silent state ...
        $_PLUGIN->setSilent($this->silent);

        // register the plugin's events ...
        foreach($_PLUGIN->getEvents() as $event) {
          $this->registerEvent($event);
        }

        // register plugin's chat commands ...
        foreach($_PLUGIN->getChatCommands() as $chat_command) {
          if($this->silent) break;
          $this->addChatCommand($chat_command);
        }

        // register the plugin ...
        $this->registerPlugin($_PLUGIN);
      }
    }

    // check if all plugins are working right ...
    $this->checkDependencies();
  }


  /**
   * Registers a new plugin.
   *
   * @param Plugin $in_plugin
   */
  function registerPlugin($in_plugin) {

    // add the new plugin into the list ...
    $in_plugin->Aseco = $this;
    $this->plugins[$in_plugin->getName()] = $in_plugin;
  }


  /**
   * Checks the included plugins for their
   * dependencies.
   *
   */
  function checkDependencies() {

    // go through the list of plugins if there are any ...
    foreach ($this->plugins as $plugin) {

      // now check the dependencies of each of them ...
      foreach($plugin->getDependencies() as $dependence) {

        // check if dependence exists ...
        if (!isset($this->plugins[$dependence->name])) {
          trigger_error('Plugin "'.$plugin->getName().'" requires the "'
          .$dependence->name.'" plugin to run stable!'.LF
          .'Disclude the pugin or install the required one to continue.' , E_USER_ERROR);
        }

        // check if dependence has min version ...
        if ($this->plugins[$dependence->name]->getVersion() < $dependence->min_ver
        && isset($dependence->min_ver)) {
          trigger_error('Plugin "'.$plugin->getName().'" requires a more recent version of the "'.
          $dependence->name.'" plugin!'.LF
          .'Current version:'.$this->plugins[$dependence->name]->getVersion().LF
          .'Expected version:'.$dependence->min_ver.')', E_USER_ERROR);
        }

        // check if dependence is lower than max version ...
        if ($this->plugins[$dependence->name]->getVersion() > $dependence->max_ver
        && isset($dependence->max_ver)) {
          trigger_error('Plugin "'.$plugin->getName().'" requires an older version of the "'.
          $dependence->name.'" plugin!'.LF
          .'Current version:'.$this->plugins[$dependence->name]->getVersion().LF
          .'Expected version:'.$dependence->max_ver, E_USER_ERROR);
        }
      }
    }
  }


  /**
   * Runs the server.
   *
   */
  function run() {

    // connect to Trackmania Dedicated Server ...
    if (!$this->connect()) {

      // kill program with an error ...
      trigger_error("Connection could not be established!", E_USER_ERROR);
    }

    // send status message ...
    $this->console_text('[Aseco] Connection established successfully!');

    // throw new event ...
    $this->releaseEvent ('onStartup', array());

    // synchronize information with server ...
    $this->console_text('[Aseco] Synchronizing with dedicated server ...');
    $this->serverSync();

    // make a visual header ...
    $this->sendHeader();

    // get current game infos if server loaded a track yet ...
    if ($this->server->status == 100) {
      $this->console_text('[Aseco] Waiting for the server to start a challenge');
    } else {
      $this->beginRace(false);
    }

	$xmlrpc_error_count = 0;

    // main loop ...
    while (true) {

      // get callbacks from the server ...
      $this->executeCallbacks();

      // check on xmlrpc errors ...
      // $this->client->error->message = str_replace($ignore_messages, '', $this->client->error->message);
      if ($this->client->error->code != 0 && $this->client->error->message != '') {
        $this->console('XMLRPC Error ['.$this->client->error->code.'] - '.
                       $this->client->error->message);
		$xmlrpc_error_count++;
		$this->client->error->message = "";
		$this->client->error->code = 0;
      } else {
		$xmlrpc_error_count = 0;
	  }


      // sends calls to the server ...
      $this->executeCalls();

      // update manialink frameset if there's a need ...
      $this->updateFramework();

      // release a mainloop event each time aseco does its loop ...
      $this->releaseEvent('onMainLoop', null);

      // make sure the script does not timeout ...
      set_time_limit($this->settings['script_timeout']);

	  if ( $xmlrpc_error_count > 10 )
	  	{
		break;
	  	}
    }

    // close the client connection ...
    $this->client->Terminate();
  }


  /**
   * Authenticates Aseco at the server.
   *
   * @return bool
   */
  function connect() {

    // only if logins are set ...
    if ($this->server->ip && $this->server->port && $this->server->login && $this->server->pass) {

      $this->console_text('[Aseco] Try to connect to server on {1}:{2}',
	      $this->server->ip,
    	  $this->server->port);

      // connect to the server ...
      if (!$this->client->InitWithIp($this->server->ip, $this->server->port)) {
        trigger_error("[".$this->client->getErrorCode()."] ".$this->client->getErrorMessage());
        return false;
      }

      $this->console_text('[Aseco] Authenticated with username \'{1}\' and password \'{2}\'',
      $this->server->login, $this->server->pass);

      // logon the server ...
      $this->client->query('Authenticate', $this->server->login, $this->server->pass);
	  $result = $this->client->getResponse();
	  if ( isset($result['faultCode']) )
	  	{
		$this->console_text('[' . $result['faultCode'] . '] ' . $result['faultString']);
		return false;
	  	}

	  $this->client->query('SendHideManialinkPage');
	  $result = $this->client->getResponse();
	  if ( isset($result['faultCode']) )
	  	{
		$this->console_text('[' . $result['faultCode'] . '] ' . $result['faultString']);
		return false;
	  	}

      // enable callback system ...
//      $this->client->addCall('EnableCallbacks', array(true));
      $this->client->query('EnableCallbacks', true);			// 08-4-23 BTM - no more waiting, send when you get it

      // start query ...
//      if (!$this->client->multiquery()){
//        trigger_error('['.$this->client->getErrorCode().'] '.$this->client->getErrorMessage());
//        return false;
//      }
      $this->client->getResponse();

      // connection established ...
      return true;
    } else {

      // connection failed ...
      return false;
    }
  }


  /**
   * Initializes the player list.
   * Reads a list of the players who are on the server already.
   *
   */
  function serverSync() {

    // get current players on the server ...
    $this->client->query('GetPlayerList', 100, 0);
    $response['playerlist'] = $this->client->getResponse();

    // get game version the server runs on ...
    $this->client->query('GetVersion');
    $response['version'] = $this->client->getResponse();

/* this won't run on the older TM servers, so let's use the Forever method -cav */
    $this->client->query('GetCurrentGameInfo', 1);
    $response['gameinfo'] = $this->client->getResponse();

    $this->client->query('GetStatus');
    $response['status'] = $this->client->getResponse();

    // get game ...
    $this->server->setGame($response['version']['Name']);

    // get mode ...
    $this->server->gameinfo = new Gameinfo($response['gameinfo']);

    // create record list - now we know what game mode the server is running ...
    $this->records = new RecordList($this->settings['recs_count'], $this->server->gameinfo);

    // update player list ...
    if (!empty($response['playerlist'])) {
      foreach ($response['playerlist'] as $player) {

        // fake it into thinking it's a connecting player, it gets team & ladder info this way ...
        $this->playerConnect(array($player['Login'], ''));
      }
    }

    // get status ...
    $this->server->status = $response['getstatus']['Code'];

    // get trackdir ...
	$this->client->query('GetTracksDirectory');
	$this->server->trackdir = $this->client->getResponse();

    // throw new synchronisation event ...
    $this->releaseEvent('onSync', null);
  }


  /**
   * Load settings and apply them on the current
   * run.
   *
   * @param string $config_file
   * @return bool
   */
  function loadSettings($config_file) {

    if ($settings = $this->xml_parser->parseXml($config_file)) {

//print_r($settings);
      // read the xml structure into an array ...
      $aseco = $settings['SETTINGS']['ASECO'][0];

      // read settings and apply them ...
      $this->masterserver->login = $aseco['LOGIN'][0];
      $this->masterserver->pass = $aseco['PASSWORD'][0];
      $this->masterserver->ip = $aseco['SERVERIP'][0];
      $this->masterserver->port = $aseco['SERVERPORT'][0];
      $this->chat_colors = $aseco['COLORS'][0];
      $this->chat_messages = $aseco['MESSAGES'][0];
      $this->admin_list = $aseco['ADMINS'][0];

      // set script timeout ...
      $this->settings['script_timeout'] = $aseco['SCRIPT_TIMEOUT'][0];

      // display welcome message as window ?
      if (strtoupper($aseco['WELCOME_MSG_WINDOW'][0]) == 'TRUE') {
        $this->settings['welcome_msg_window'] = true;
      } else {
        $this->settings['welcome_msg_window'] = false;
      }

      // display welcome message at all ?
      if (strtoupper($aseco['WELCOME_MSG_ENABLED'][0]) == 'TRUE') {
        $this->settings['welcome_msg_enabled'] = true;
      } else {
        $this->settings['welcome_msg_enabled'] = false;
      }

      // display endrace recs in chat?
      if (strtoupper($aseco['ENDRACE_CHATRECS'][0]) == 'TRUE') {
        $this->settings['endrace_chatrecs'] = true;
      } else {
        $this->settings['endrace_chatrecs'] = false;
      }

      // read how many records to manage ...
      $this->settings['recs_count'] = $aseco['RECS_COUNT'][0];

      // read the xml structure into an array ...
      $tmserver = $settings['SETTINGS']['TMSERVER'][0];

      // read settings and apply them ...
      $this->server->login = $tmserver['LOGIN'][0];
      $this->server->pass = $tmserver['PASSWORD'][0];
      $this->server->port = $tmserver['PORT'][0];
      $this->server->ip = $tmserver['IP'][0];

      // settings loaded ...
      return true;

    } else {

      // Could not parse XML file ...
      trigger_error('You\'ve got an XML error in your Aseco config file!');
      return false;
    }
  }


  /**
   * Sends program header to console ...
   * ... and ingame chat!
   *
   */
  function sendHeader() {
    $this->console_text('###############################################################################');
	$this->console_text('# Game Info: {1} - {2}',
		$this->server->getGame(),
		$this->server->gameinfo->getMode());
    $this->console_text('###############################################################################');

    // show debug message ...
    $this->console_debug('Running Aseco in debugging mode ...');

    // format the text of the message ...
    $startup_msg = formatText($this->getChatMessage('STARTUP'),
    	ASECO_VERSION,
    	$this->server->ip,
    	$this->server->port);

    // replace colors ...
    $startup_msg = $this->formatColors($startup_msg);

    // send the message ...
    $this->sendServerMessage($startup_msg);
  }


  /**
   * Gets callbacks from the TM Dedicated Server
   * and reacts on them.
   */
  function executeCallbacks() {

    // receive callbacks with a timeout of 20000 microsecs ...
    $this->client->readCB(2000);

    // now get the responses out of the "buffer" ...
    $calls = $this->client->getCBResponses();

    if (!empty($calls)) {
      while ($call = array_shift($calls)) {
        switch ($call[0]) {

        case 'TrackMania.PlayerConnect':
          // event is released in the function ...
          $before = memory_get_usage();
          $this->playerConnect($call[1]);
          $after = memory_get_usage();

          // view debug information ...
          $this->console_debug("PlayerConnect ate ".($after - $before)." bytes");
          break;

        case 'TrackMania.PlayerDisconnect':
          // event is released in the function ...
          $before = memory_get_usage();
          $this->playerDisconnect($call[1]);
          $after = memory_get_usage();

          // view debug information ...
          $this->console_debug("PlayerDisconnect ate ".($after - $before)." bytes");
          break;

/* no longer exists
        case 'TrackMania.PlayerServerMessageAnswer':
          $this->playerServerMessageAnswer($call[1]);
          break;
*/

        case 'TrackMania.PlayerFinish':
          $before = memory_get_usage();
          $this->playerFinish($call[1]);
          $after = memory_get_usage();

          // view debug information ...
          $this->console_debug("PlayerFinish ate ".($after - $before)." bytes");
          break;

        case 'TrackMania.PlayerChat':
          $call[1]['author'] = $this->server->players->getPlayer($call[1][1]);
          $this->playerChat($call[1]);
          $this->releaseEvent('onChat', $call[1]);
          break;

        case 'TrackMania.BeginRace':
          $before = memory_get_usage();
          $this->beginRace($call[1]);
          $after = memory_get_usage();

          // view debug information ...
          $this->console_debug("BeginRace ate ".($after - $before)." bytes");
          break;

        case 'TrackMania.EndRace':
          $before = memory_get_usage();
          $this->endRace($call[1]);
          $after = memory_get_usage();

          // view debug information ...
          $this->console_debug("EndRace ate ".($after - $before)." bytes");
          break;

        case 'TrackMania.PlayerCheckpoint':
          $this->releaseEvent('onCheckpoint', $call[1]);
          break;

        case 'TrackMania.StatusChanged':
          if ($call[1][0] == 3) { // get mode ...
            $mode = $this->server->gameinfo->getMode();
 /* this won't run on the older TM servers, so let's use the Forever method -cav */
            $response = $this->doCall('GetCurrentGameInfo', 1);
            $this->server->gameinfo = new Gameinfo($response);
            if ($this->server->gameinfo->getMode() != $mode) {
              $this->console('gamemode changed ['.$mode.'] >> ['.$this->server->gameinfo->getMode().']');
              $this->releaseEvent('onGameModeChange', $this->server->gameinfo->getMode());
  			}
			$this->onStatusChange3();
          }
          $this->releaseEvent('onStatusChangeTo'.$call[1][0], $call[1]);
          break;

        case 'TrackMania.PlayerManialinkPageAnswer':
          $player = $this->server->players->getPlayer($call[1][1]);
		  $this->releaseEvent('onPlayerManialinkPageAnswer', $call[1]);
		  $player->framework->releaseWindowEvent($call[1][2], $player);
          break;

		case 'TrackMania.ChallengeListModified':
		  $this->releaseEvent('onChallengeListModifed', $call[1]);
		  break;

        case 'TrackMania.PlayerInfoChanged':
		  $changeinfo = $call[1][0];
          $player = $this->server->players->getPlayer($changeinfo['Login']);
		  $changeinfo['player'] = $player;

		  if ( $player->spectatorstatus != $changeinfo['SpectatorStatus'] )
		  	{
		  	$changeinfo['changedSpectator'] = 1;
		  	}
		  if ( $player->teamid != $changeinfo['TeamId'] )
		  	{
		  	$changeinfo['changedTeamId'] = 1;
		  	}
		  $player->spectatorstatus = $changeinfo['SpectatorStatus'];
		  $player->teamid = $changeinfo['TeamId'];

          $this->releaseEvent('onPlayerInfoChanged', $changeinfo);
          break;

		case 'TrackMania.BeginRound':
          $this->releaseEvent('onBeginRound', null);
          break;

		case 'TrackMania.EndRound':
          $this->releaseEvent('onEndRound', null);
          break;

		case 'TrackMania.BillUpdated':
			$BillInfo['BillId'] = $call[1][0];
			$BillInfo['State'] = $call[1][1];
			$BillInfo['StateName'] = $call[1][2];
			$BillInfo['TransactionId'] = $call[1][3];
			$this->releaseEvent('onBillUpdated', $BillInfo);
			break;

		case 'TrackMania.PlayerIncoherence':
			$player = $this->server->players->getPlayer($call[1][1]);
			$this->releaseEvent('onPlayerIncoherence', $player);
			break;

        default:
          // do nothing ...
        }
      }
      return $calls;
    } else {
      return false;
    }
  }


  /**
   * Adds calls to a multiquery.
   * It's possible to set a callback function which
   * will be executed on incoming response.
   * You can also set an ID to read response later on.
   *
   * @param string $call
   * @param array $params
   * @param int $id
   * @param string $callback_func
   */
  function addCall($call, $params = array(), $id = 0, $callback_func = false) {

    // adds call and registers a callback if needed ...
    $index = $this->client->addCall($call, $params);
    $rpc_call = new RPCCall($id, $index, $callback_func, array($call, $params));
    $this->rpc_calls[] = $rpc_call;

    // if there are too many requests in queue
    if (count($this->rpc_calls) > 20)
      $this->executeCalls(); // execute them and clear the list ...
  }


  /**
   * Executes one simple query.
   * This stops the script until there's
   * a response from the server.
   *
   * @param string $method
   * @param mixed $params
   * @return response
   */
  function doCall() {
    $args = func_get_args();
    $callback_func = array(&$this->client, 'query');
    call_user_func_array($callback_func, $args);
    return $this->client->getResponse();
  }


  /**
   * Executes a multicall and gets responses.
   * Saves responses in array with IDs as keys.
   */
  function executeCalls() {

    // clear responses ...
    unset($this->rpc_responses);
    $this->rpc_responses = array();

    // stop if there are no rpc calls in query ...
    if (empty($this->client->calls)) {
      return;
    }

    // sends multiquery to the server and gets the response ...
    if ($this->client->multiquery()) {

      // get new response from server ...
      $responses = $this->client->getResponse();

      // handle server responses ...
      foreach ($this->rpc_calls as $call) {

        // display error message if needed ...
        $err = false;
        if (isset($responses[$call->index]['faultString'])) {
          $this->rpcErrorResponse($responses[$call->index]);
          print_r($call->call);
          $err = true;
        }

        // if an id was set, then save the response under the specified id ...
        if ($call->id) {
          $this->rpc_responses[$call->id] = $responses[$call->index][0];
        }

        // if a callback function has been set, then execute it ...
        if ($call->callback && !$err) {
          if (function_exists($call->callback)) {

            // callback the function with the response as parameter ...
            call_user_func($call->callback, $responses[$call->index][0]);

          // if a function with the name of the callback wasn't found, then
          // try to execute a method with its name ...
          } elseif (method_exists($this, $call->callback)) {

            // callback the method with the response as parameter ...
            call_user_func(array($this, $call->callback), $responses[$call->index][0]);
          }
        }
      }
    }

    // clear calls ...
    unset($this->rpc_calls);
    $this->rpc_calls = array();
  }


  /**
   * Documents RPC Errors.
   *
   * @param array $response
   */
  function rpcErrorResponse($response) {
    $this->console_text('[RPC Error '.$response['faultCode'].'] '.$response['faultString']);
  }


  /**
   * Registers functions which are called on specific events!
   *
   * @param Event $in_event
   */
  function registerEvent($in_event) {

    // registers a new event ...
    $this->events[$in_event->type][] = $in_event;
  }


  /**
   * Executes the functions which were registered for
   * specified events.
   *
   * @param string $event_type
   * @param mixed $func_param
   */
  function releaseEvent($event_type, $func_param) {

    // executes registered event functions ...
    // if there are any events for that type ...
    if (isset($this->events[$event_type])) {

      // for each registered function of this type ...
      foreach($this->events[$event_type] as $event) {

        // if function for the specified player connect event can be found ...
        if (method_exists($event->plugin, $event->callback)) {

          $params = array();
          $params[] = $func_param;

          // get mem usage before ...
          $before = memory_get_usage();

          // ... execute it with storage as last parameter ...
          call_user_func_array(array(&$event->plugin, $event->callback), $params);

          // get mem usage after callback ...
          $after = memory_get_usage();

          // display if something has changed ...
          if ($before != $after) $this->console_debug('['.$event->plugin->getName().'] -> ['.$event_type.'] ate '.($after-$before).' bytes!');
        }
      }
    }
  }


  /**
   * Registers functions which are called on specific events!
   *
   * @param ChatCommand $chat_command
   */
  function addChatCommand($chat_command) {
    $this->chat_commands[$chat_command->name][] = $chat_command;
  }


  /**
   * When a new race is started we have to get information
   * about the new track and so on.
   *
   * @param array $race
   */
  function beginRace($race) {

    // show some debug information ...
    $this->displayMemoryUsage();

    // request information about the new challenge ...
    // ... and callback to function newChallenge() ...
	if (!$race)
		{
		$this->addCall('GetCurrentChallengeInfo', array(), '', 'newChallenge');
		}
	else
		{
		$this->newChallenge($race[0]);
		}
  }


  /**
   * Reacts on new challenges.
   * gets record to current challenge etc.
   *
   * @param array $challenge
   */
  function newChallenge($challenge) {

    // reset player votings ...
    $this->server->players->resetVotings();

    // clear all records ...
    $this->records->clear();

    // create new challenge object ...
    $challenge_item = new Challenge($challenge);

    // display console message ...
    $this->console('map changed [{1}] >> [{2}]',
    	stripFormatting($this->server->challenge->name),
	    stripFormatting($challenge_item->name));

    // update the field which contains current challenge ...
    $this->server->challenge = $challenge_item;

    // release a "new challenge" event ...
    $this->releaseEvent('onNewChallenge', $challenge_item);

    // display record to this challenge ..
    $this->sendCurrentRecord();

    unset($challenge_item);
  }


  /**
   * End of current race ...
   * write records to database and/or display
   * final statistics.
   *
   * @param array $race
   */
  function endRace($race)
  	{
	if ( $this->settings['endrace_chatrecs'] )
		{
    	// no records were driven ...
    	if ($this->records->count() == 0) {
    	  // set a no new record message ...
    	  $message = $this->formatColors($this->chat_messages['RANKING_NONE'][0]);

    	// records (were) driven ...
    	} else {
    	  // display which new records were set up this round ...
    	  // replaces aseco color strings ...
    	  $message = $this->formatColors($this->chat_messages['RANKING'][0]);
    	  $records = '';

    	  // display some records at the end of the round ...
    	  for ($i = 0; $i < $this->re_recs_count; $i++) {

    	    $cur_record = $this->records->getRecord($i);

    	    // as long as score is not 0 ...
    	    if ($cur_record->score == 0) break;

    	    // format record score for displaying ...
    	    if ($this->server->gameinfo->getMode() == 'Stunts')
    	      $record_score = $cur_record->score;
    	    else
    	      $record_score = formatTime($cur_record->score);

    	    // if the record is set then display it ...
    	    $record_msg = formatText($this->chat_messages['RANKING_RECORD'][0].' ',
		        $i+1,
    		    stripFormatting($cur_record->player->nickname),
    	    	$record_score);

    	    // replace colors ...
    	    $record_msg = $this->formatColors($record_msg);

    	    // add record to the message ...
    	    $records .= $record_msg;
    	  }
    	}

    	// send the ranking message to the server ...
    	$this->sendServerMessage($message);

		// 07-5-1 AManiac - trim off trailing comma if one exists
		if ( $records && substr($records, -2) == ', ' )
			{
			$records = substr($records, 0, strlen($records) - 2);
			}

    	// send the rankings to the server if there are any ...
    	if ($records) $this->sendServerMessage($records);
		}
	else
		{
		$content = "<line><cell bgcolor='888F' width='.5'><text textcolor='0E0' halign='center'>Records on " . htmlspecialchars($race[1]['Name']) . '</text></cell></line>';
    	if ($this->records->count() == 0)
			{
			$content .= "<line><cell width='.5'><text textcolor='8af' halign='center'>No Records</text></cell></line>" . CRLF;
			}
		else
			{
			$numcell = "<cell width='.02' bgcolor='000F'><text textcolor='DDD' halign='center'>";
			$timecell = "<cell width='.1' bgcolor='000F'><text textcolor='8af' halign='center'>";
			$namecell = "<cell width='0.38' bgcolor='000F'><text textcolor='8af' halign='left'>";

			for ($i = 0; $i < $this->re_recs_count; $i++)
				{
				$cur_record = $this->records->getRecord($i);

				// as long as score is not 0 ...
				if ($cur_record->score == 0) break;

				// format record score for displaying ...
				if ($this->server->gameinfo->getMode() == 'Stunts')
  					$record_score = $cur_record->score;
				else
					{
  					$record_score = formatTime($cur_record->score);
					if ( substr($record_score, 0, 1) == '0' )
						{
						$record_score = substr($record_score, 1);
						}
					}

				$content .= "<line height='.03'>" . $numcell . ($i + 1) . '.</text></cell>';
				$content .= $timecell . $record_score . '</text></cell>';
				$content .= $namecell . htmlspecialchars($cur_record->player->nickname) . '</text></cell></line>' . CRLF;
  				}
			}

		foreach($this->server->players->player_list as $pl)
			{
			if ( !isset($pl->data['w_showendrecs'] ) )
				{
    			$window = $this->createWindowToPlayerFramework('', 1.0, .745, $pl, true);
				$window->setContent($content);
				$window->setBackground('000F');
				$window->setBorderSize(0);
				$window->setTextsize(1);
				$pl->data['w_showendrecs'] = $window;
				}
			else
				{
				$window = $pl->data['w_showendrecs'];
				$window->setContent($content);
				}
			$pl->framework->minimizeAllWindows();
			$pl->framework->setShowMinimizedWindows(false);
			$window->show();

			if ( isset($pl->data['Statusbar_window']) )
				{
				$window = $pl->data['Statusbar_window'];
				$window->hide();
				}
			}
		}

    // release new event ...
    $this->releaseEvent('onEndRace', $race);

    // call endRaceRanking with the current standings ...
    $this->endRaceRanking($race[0]);
  }

	// hide the end race top 5 recs after intermission is over

	function onStatusChange3()
		{
		if ( $this->settings['endrace_chatrecs'] == false )
			{
			foreach($this->server->players->player_list as $pl)
				{
				$pl->framework->setShowMinimizedWindows(true);
				if ( isset($pl->data['w_showendrecs'] ) )
					{
					$window = $pl->data['w_showendrecs'];
					$window->hide();
					}

				if ( isset($pl->data['Statusbar_window']) )
					{
					$window = $pl->data['Statusbar_window'];
					$window->show();
					}
				}

			}
		}  //  onStatusChange4

  /**
   * Check out who won the current run
   * and increment his wins by one.
   *
   * @param array $ranking
   */
  function endRaceRanking($ranking) {
    if ($player = $this->server->players->getPlayer($ranking[0]['Login'])) {
      if ($ranking[0]['Rank'] == 1 && $ranking[0]['BestTime'] > 0 && count($ranking) > 1) {

        // increase the player's wins ...
        $player->wins++;

        // release a new event ...
        $this->releaseEvent('onPlayerWins', $player);
      }
    }
  }


  /**
   * Handles connections of new players.
   *
   * @param array $player
   */
  function playerConnect($player) {

    $this->console_debug('player\'s callback login:'.$player[0]);

    // request information about the new player ...
    // ... and callback to function newPlayer() ...
/* Needed more info than GetPlayerInfo provided -cav */
    $this->addCall('GetDetailedPlayerInfo', array($player[0]), '', 'newPlayer');
  }


  /**
   * Callback function of playerConnect().
   * Receives information about the new player.
   *
   * @param array $player
   */
  function newPlayer($player) {

    // cancel if we got a player without a login ...
    if ($player['Login'] == '') return;

    // creates player object ...
    $player_item = new Player($player);

    // adds a new player to the intern player list ...
    $this->server->players->addPlayer($player_item);

    // display console message ...
    $this->console('<< player {1} joined the game [{2}]',
    	$player_item->id,
    	$player_item->login);

    // send chat message to player ...
    // format message parameters ...
    $message = formatText($this->getChatMessage('WELCOME'),
    	stripFormatting($player_item->nickname),
    	ASECO_VERSION);

    // replace colors ...
    $message = $this->formatColors($message);

    // send the message ...
    $message = str_replace('{br}', LF, $message);
    if ($this->settings['welcome_msg_window'] && $this->settings['welcome_msg_enabled']) {
      $window = $player_item->framework->createWindow('Welcome to Aseco', 0.9, 0.35);
      $window->setContent('<line><cell><text>'.htmlspecialchars($message).'</text></cell></line>');
      unset($window);
    } elseif($this->settings['welcome_msg_enabled']) {
      $this->sendServerMessageToLogin($player_item->login, str_replace(LF.LF, LF, $message));
    }

    // send a message with the current record ...
    $this->sendCurrentRecord($player_item->id);

    // release a new player connect event ...
    $this->releaseEvent('onPlayerConnect', $player_item);
  }


  /**
   * Handles disconnections of players.
   *
   * @param array $player
   */
  function playerDisconnect($player) {

    // prevent aseco from using invalid player objects ...
    if(!isset($player) || $player[0] == '') return;

    // delete player and put him into the player item ...
    $player_item = $this->server->players->getPlayer($player[0]);

    // display message ...
    $this->console('>> player {1} left the game [{2}]',
    $player_item->id,
    $player_item->login);

    $this->console_debug('player\'s callback login:'.$player_item->login);

    // release a new player disconnect event ...
    $this->releaseEvent('onPlayerDisconnect', $player_item);

    // delete reference to the player ...
    unset($player_item);

    // delete player object ...
    $this->server->players->removePlayer($player[0]);
  }


  /**
   * Handles clicks on server messageboxes.
   *
   * @param array $answer
   */
  function playerServerMessageAnswer($answer) {
    if ($answer[2]) {

      // throw new event ...
      $this->releaseEvent('onPlayerServerMessageAnswer', $answer);
    }
  }


  /**
   * Handles record updates, called by playerFinish function.
   * Releases an onPlayerFinish event.
   *
   * @param Record object $finish_item
   */
  function RecordKeeping($finish_item) {

    // get game mode ...
    $mode = $this->server->gameinfo->getMode();

    // insert record into the list ...
    $result = $this->records->addRecord($finish_item);

    // format record score for displaying ...
    if ($mode == 'Stunts')
      $finish_time = $finish_item->score;
    else
      $finish_time = formatTime($finish_item->score);

    // check result of record inserting ...
    switch ($result[0]) {

      case 0: // player improved his record ...
          // do a player improved his record message ...
          $message = formatText($this->getChatMessage('RECORD_NEW'),
                                $result[1]+1,
                                $finish_time);
          // replace colors ...
          $message = $this->formatColors($message);
          // send the message ...
          $this->sendServerMessageToLogin($finish_item->player->login, $message);
        break;

      case 1: // player improved his record and moved up in the ladder ...
          // do a player improved his WR rank message ...
          // replace parameters ...
          $message = formatText($this->getChatMessage('RECORD_NEW_RANK'),
                                stripFormatting($finish_item->player->nickname),
                                $finish_time,
                                $result[1]+1);
          // replace colors ...
          $message = $this->formatColors($message);
          // send the message ...
          $this->sendServerMessage($message);
        break;

      case 2: // new record has been set up ...
          // do a player drove first record message ...
          // replace parameters ...
          $message = formatText($this->getChatMessage('RECORD_FIRST'),
                                stripFormatting($finish_item->player->nickname),
                                $finish_time,
                                $result[1]+1);
          // replace colors ...
          $message = $this->formatColors($message);
          // send the message ...
          $this->sendServerMessage($message);
        break;

      default: // no record, or an error occured ...
          // do nothing ...
        break;
    }

    // display new record message if result is bigger or equal to 0 ...
    if ($result[0] >= 0) {

      // display record message in console
      $this->console('player {1} finished with {2} and took the {3}. WR place!',
                     $finish_item->player->id,
                     $finish_time,
                     $result[1]+1);

      // release an "on record" event ...
      $this->releaseEvent('onRecord', $finish_item);
    }
  }


  /**
   * Player reaches finish.
   *
   * @param array $finish
   */
  function playerFinish($finish) {

    // only if we have some information about the track ...
    if ($this->server->challenge->name == '') {
    	return;
    }

	if ( $finish[2] == 0 )
		{
		return;
		}

    // build a record object with the current finish information ...
    $finish_item = new Record(true);
    $finish_item->score = $finish[2];
    $finish_item->challenge = $this->server->challenge;
    $player_item = new RecPlayer($this->server->players->getPlayer($finish[1]));
    $finish_item->player = $player_item;
    $finish_item->date = strftime('%Y-%m-%d %H:%M:%S');

    // check for records and release event ...
	$this->RecordKeeping($finish_item);

    // throw new event - player finished ...
    $this->releaseEvent('onPlayerFinish', $finish_item);
  }


  /**
   * Receives chat messages and reacts on them.
   * Reactions are done by the chat plugins.
   *
   * @param array $chat
   */
	function playerChat($chat) {

    // check for chat command ...
		$command = $chat[2];
		if (substr($command, 0, 1) == "/") {

			// remove the '/' prefix ...
			$command = substr($command, 1);

			// split strings at spaces and add them into an array ...
			$params = explode(' ', $command, 2);
			$cmd = $params[0];

			// put arguments in array ...
			$chat_command['author'] = $chat['author'];
			$chat_command['params'] = $params[1];

			// if this is an admin command ...
			if ($params[0] == 'admin') {
				// get the params of the admin command ...
				$admin_params = explode(' ', $params[1], 2);
				$cmd .= ' ' . $admin_params[0];
				$chat_command['params'] = $admin_params[1];
				$params[1] = $admin_params[1];
			}

			// if there are commands registered onto this string ...
			if (isset($this->chat_commands[$cmd])) {

				// show message in console ...
				$this->console('player {1} used chat command \'/{2} {3}\'',
								$chat_command['author']->login,
								$cmd,
								$params[1]);

				// call each registered command ...
				foreach ($this->chat_commands[$cmd] as $registered_command) {

					// do we need admin access of any kind for this command?
					if ( $registered_command->admingroup != '' )
						{
						// author is not an admin ...
						if (!$this->isAdmin($chat_command['author']->login, $registered_command->admingroup))
							{
							$message = '$f00You have to be in the admin list to do that!';
							$this->sendServerMessageToLogin($chat_command['author']->login, $message);
							return;
							}

						}

					// check if callback function exists ...
					if (method_exists($registered_command->plugin,
						$registered_command->callback)) {

						$func_params = array();
						$func_params[] = $chat_command;

						// call the function which belongs to the command ...
						call_user_func_array(array(&$registered_command->plugin,
							$registered_command->callback),
							$func_params);
          }
        }
      }
    }
  }


  /**
   * Gets the specified chat messages out of the
   * settings file.
   *
   * @param string $name
   * @return string
   */
  function getChatMessage($name) {
    return $this->chat_messages[$name][0];
  }


  /**
   * Sets a new text to be displayed.
   *
   * @param string $name
   * @param string $text
   */
  function setChatMessage($name, $text) {
    $this->chat_messages[$name][0] = $text;
  }


  /**
   * Checks if the given player ID is in admin list.
   *
   * @return bool
   */
  function isAdmin($login, $item2check = 'adminsuper')
  	{
	if ( count($this->admin_groups) )
		{
		$login = strtoupper($login);
		$item2check = strtoupper($item2check);
		$group = strtoupper($this->admin_groups['SETTINGS']['TMLOGINS'][0][$login][0]);

		$authorized = $this->admin_groups['SETTINGS']['GROUPS'][0][$group][0][$item2check][0];

		if ( !isset($authorized) )	// if it's not set in a particular group, fallback to the authitems list, it might default to true
			{
			$authorized = $this->admin_groups['SETTINGS']['AUTHITEMS'][0][$item2check][0];
			}

		return $authorized == 'true';
		}
	else
		{
    	for ($i = 0; $i < count($this->admin_list['TMLOGIN']); $i++)
			{
    	  	if ($this->admin_list['TMLOGIN'][$i] == $login)
				{
	    	    return true;
				}
			}
		}
    return false;
	}


  /**
   * Adds a new Admin to the list.
   *
   * @param string $admin
   */
  function addAdmin($login) {
    $this->admin_list['TMLOGIN'][] = $login;
  }


  /**
   * Formats aseco color codes in a string,
   * for example "{#server} hello" will end up as
   * "$0ff hello".
   * It depends on what you've set in the config file.
   *
   * @param string $text
   * @return string
   */
  function formatColors ($text) {

    // replace colors ...
    for ($i = 0; $i < sizeof($this->chat_colors); $i++) {
      $key = key($this->chat_colors);
      $value = current($this->chat_colors);
      $text = str_replace('{#'.strtolower($key).'}', $value[0], $text);
      next($this->chat_colors);
    }

    // jump back to start of the array ...
    if (is_array($this->chat_colors)) {
      reset($this->chat_colors);
    }

    return $text;
  }


  /**
   * Outputs a formatted string without datetime.
   *
   */
  function console_text() {
    $args = func_get_args();
    $message = call_user_func_array('formatText', $args).CRLF;
    echo $message;
    doLog($message);
    flush();
  }


  /**
   * Outputs a string to console with datetime prefix.
   *
   */
  function console() {
    $args = func_get_args();
    $message = '['.date('m/d,H:i:s').'] '.call_user_func_array('formatText', $args).CRLF;
    echo $message;
    doLog($message);
    flush();
  }


  /**
   * Shows debug message in the console,
   * if debug mode is set to true.
   *
   */
  function console_debug() {
    if (!$this->debug) return;
    $args = func_get_args();
    $message = '[Debug] '.call_user_func_array('formatText', $args).CRLF;
    echo $message;
    flush();
    return $message;
  }


  /**
   * Creates a window on the
   * specified player's framework.
   *
   * @param string $title
   * @param float $pos_x
   * @param float $pos_y
   * @param Player $player
   * @return ManialinkPage
   */
  function createWindowToPlayerFramework($title, $pos_x, $pos_y, $player, $is_static = false) {
    if ($this->silent) return null;
    return $player->framework->createWindow($title, $pos_x, $pos_y, $this, $is_static);
  }


  /**
   * Sends a message without a name tag to
   * the player with the specified login.
   *
   * @param string $login
   * @param string $message
   */
  function sendServerMessageToLogin($login, $message) {
    if ($this->silent | strlen($message) == 0 | !isset($message)) return;
//    $this->addCall('ChatSendServerMessageToLogin', array($message, $login));
    $this->client->query('ChatSendServerMessageToLogin', $message, $login);		// 08-4-23 BTM - no more waiting, send when you get it
  }


  /**
   * Sends a message without a name tag to
   * the player with the specified id.
   *
   * @param string $login
   * @param string $message
   */
  function sendServerMessageToId($id, $message) {
    if ($this->silent | strlen($message) == 0 | !isset($message)) return;
//    $this->addCall('ChatSendServerMessageToId', array($message, $id));
    $this->client->query('ChatSendServerMessageToId', $message, $id);			// 08-4-23 BTM - no more waiting, send when you get it
  }


  /**
   * Sends a message without a namge tag to
   * all players on the server.
   *
   * @param string $message
   */
  function sendServerMessage($message) {
    if ($this->silent | strlen($message) == 0 | !isset($message)) return;
//    $this->addCall('ChatSendServerMessage', array($message));
    $this->client->query('ChatSendServerMessage', $message);					// 08-4-23 BTM - no more waiting, send when you get it
  }


  /**
   * Sends chat messages of the current record
   * to a specified player or the whole server.
   *
   * @param int $player_target
   */
  function sendCurrentRecord($player_target = null) {

    // send a message with the current record ...
    $cur_record = $this->records->getRecord(0);
    if ($cur_record->score > 0 && $this->server->challenge->name != 'none') {

      // format record time depending on what gamemode the server is running ...
      if ($this->server->gameinfo->getMode() == 'Stunts')
        $record_score = $cur_record->score;
      else
        $record_score = formatTime($cur_record->score);

      // replace parameters ...
      $message = formatText($this->getChatMessage('RECORD_CURRENT'),
      	stripFormatting($this->server->challenge->name),
      	$record_score,
      	stripFormatting($cur_record->player->nickname));

      // replace colors ...
      $message = $this->formatColors($message);

      if (isset($player_target)) {

        // send the message ...
        $this->sendServerMessageToId($player_target, $message);
      } else {

        // display console message of current record ...
        $this->console('current record on {1} is {2} and held by {3}',
        	stripFormatting($this->server->challenge->name),
        	$record_score,
        	stripFormatting($cur_record->player->nickname));

        // send the message ...
        $this->sendServerMessage($message);
      }

    } elseif ($this->server->challenge->name != 'none') {

      // send chat message of no record to all players ...
      // replace parameters ...
      $message = formatText($this->getChatMessage('RECORD_NONE'),
      stripFormatting($this->server->challenge->name));

      // replace colors ...
      $message = $this->formatColors($message);

      if (isset($player_target)) {

        // send the message ...
        $this->sendServerMessageToId($player_target, $message);
      } else {

        // display console message of current record ...
        $this->console('currently no record on {1}',
        stripFormatting($this->server->challenge->name));

        // send the message ...
        $this->sendServerMessage($message);
      }
    }
  }


  /**
   * Updates the framework for clients that have old xml code
   *
   */
  function updateFramework() {

    // only continue if aseco is not in silent mode ...
    if ($this->silent) return;

    // check if the framework changed for anyone ...
    foreach($this->server->players->player_list as $player) {

      // close open popups ...
      $player->framework->closePopups();

      // if framework isn't up to date anymore ...
      if(!$player->framework->isValid()) {

        // get the new xml code ...
        $framework_xml = $player->framework->getXml();

        // send it to the server ...
//        $this->addCall('SendDisplayManialinkPageToLogin', array($player->login, $framework_xml, 0, false));
        $this->client->query('SendDisplayManialinkPageToLogin', $player->login, $framework_xml, 0, false);		// 08-4-23 BTM - force data out immediately, should stop the "Login unknown" errors
		$result = $this->client->getResponse();
		if ( isset($result['faultCode']) )
			{
			$this->console_text('[Aseco] UpdateFramework - SendDisplayManialinkPageToLogin Failed - [' . $result['faultCode'] . '] ' . $result['faultString'] . CRLF . "\tlogin/xml written to xml-failure.txt");
			$handle = fopen('logs/xml-failure.txt', 'a');
			fwrite($handle, 'xmlrpc error [' . $result['faultCode'] . '] ' . $result['faultString'] . CRLF);
			fwrite($handle, 'Player Login=' . $player->login . CRLF);
			fwrite($handle, '---- XML Begins' . CRLF . $framework_xml . CRLF . '---- XML Ends' . CRLF);
			fclose($handle);
			}
		// this really should be part of the else if there was a failure, but that may result in this continuing to fail on every call
		// so mark it done anyway

        // and mark framework as updated ...
        $player->framework->validate();
      }
    }
  }


  /**
   * Script died, now rescue data!
   * Will only work for script related aborts, not user intervention.
   *
   */
  function onShutdown() {

  	// server shutdown ...
  	$this->console('Beginning to shutdown Aseco due to an error ...');

    // fake a player disconnect event ...
    $this->server->players->resetPlayers();
    while($player = $this->server->players->nextPlayer()) {
      $this->playerDisconnect(array($player->login));
    }

    // release the internal shutdown event ...
    $this->releaseEvent('onServerCrash', null);

    // close server connection ...
    $this->client->Terminate();
  }

  /**
   * Provides information about the
   * current memory usage and helps to detect
   * memory leaks.
   *
   */
  function displayMemoryUsage() {
    global $last_mem_usage, $first_mem_usage;
    if (!isset($first_mem_usage)) $first_mem_usage = memory_get_usage();
    $cur_mem_usage = memory_get_usage();
    $this->console_debug("#-----------------------------------");
    $this->console_debug("| Current Memory Usage: ".$cur_mem_usage);
    $this->console_debug("| Memory Loss:          ".($cur_mem_usage - $last_mem_usage));
    $this->console_debug("| Total Memory Loss:    ".($cur_mem_usage - $first_mem_usage));
    $this->console_debug("#-----------------------------------");
    $last_mem_usage = memory_get_usage();
  }


  /**
   * Returns the time how long Aseco is running yet.
   *
   * @return int
   */
  function getUptime() {
    return time() - $this->startuptime;
  }
}

$cfg = 'settings/config.xml';

if ( $argc >= 2 )
	{
	$filename = $argv[1];
	if (substr($filename, -4, 4) == '.xml' )
		if (file_exists($filename))
			$cfg = $filename;
	}


// create an instance of Aseco and run it ...
$aseco = new Aseco(false, $cfg);
$aseco->run();
?>
