<?php
/**
 * Writes a message to the logfile
 * modified by AM
 */
function doLog($text) {

  // handle to log file
  global $logfile;

  // make directory for year-month (07-May) ...
  $dir = './logs/'.date("y-M");

  // make log entry for each day (YY-MM-DD_log.txt) ...
  $file = date("y-m-d")."_log.txt";

  // directory doesn't exist yet, make it ...
  if (!file_exists($dir)) mkdir($dir);

  // file doesn't exist yet and logfile is in use, close logfile, unset the var ...
  if (!file_exists($dir.'/'.$file) && $logfile) {
	fclose($logfile);
	$logfile = 0;
  }

  // logfile var not set, open file & assign it ...
  if (!isset($logfile) || $logfile == 0) {
    $logfile = fopen($dir.'/'.$file, "a+");
  }
  fwrite($logfile, $text);
}



/**
 * Puts an element onto a specific position into an array.
 * Keeps original size.
 */
function insertArrayElement(&$array, $value, $pos) {

  // get current size ...
  $size = count($array);

  // if position is in array range ...
  if ($pos < 0 || $pos > $size) {
    return false;
  }

  // shift values below ...
  for ($i = $size-1; $i >= $pos; $i--) {
    $array[$i+1] = $array[$i];
  }

  // now put in the new element.
  $array[$pos] = $value;
  return true;
}


/**
 * Moves an element from one position to the other.
 * All items between are shifted down.
 */
function moveArrayElement(&$array, $from, $to) {

  // get current size ...
  $size = count($array);

  // destination and source have to be among the array borders!
  if ($from < 0 && $from > $size && $to < 0 && $to > $size) {
    return false;
  }

  // backup the element we have to move ...
  $moving_element = $array[$from];

  if ($from > $to) {

    // shift values between downwards ...
    for ($i = $from-1; $i >= $to; $i--) {
      $array[$i+1] = $array[$i];
    }
  } else {

    // not needed yet ...
    return false;
  }

  // now put in the element which was to move ...
  $array[$to] = $moving_element;
  return true;
}


/**
 * Formats a string from the format ssssms
 * into the format mmm:ss:ms.
 * modified by AM
 */
function formatTime($MwTime, $msec = true) {
  if ($MwTime == -1) {
		return "???";
  } else {
    $minutes = floor($MwTime/(1000*60));
    $seconds = floor(($MwTime-$minutes*60*1000)/1000);
    if ($msec) {
      $mseconds = ($MwTime / 10) % 100;
      return sprintf("%02d:%02d.%02d", $minutes, $seconds, $mseconds);
    } else {
      return sprintf("%02d:%02d", $minutes, $seconds);
    }
  }
}

/**
 * Formats a string from the format ssssmss
 * into the format hh:mm:ss.ms
 * modified by AM
 */
function formatTimeH($MwTime, $msec = true) {
  if ($MwTime == -1) {
		return "???";
  } else {
    $mseconds = ($MwTime / 10) % 100;
	$MwTime = ($MwTime - $MwTime % 1000) / 1000;
	$hours = floor($MwTime / 3600);
	$MwTime = $MwTime - ($hours * 3600);
    $minutes = floor($MwTime / 60);
	$MwTime = $MwTime - ($minutes * 60);
    $seconds = floor($MwTime);
    if ($msec) {
      return sprintf("%02d:%02d:%02d.%02d", $hours, $minutes, $seconds, $mseconds);
    } else {
      return sprintf("%02d:%02d:%02d", $hours, $minutes, $seconds);
    }
  }
}

/*
 Summary: Strips all display formatting from an input string, suitable for
          display within the game (escaped '$$' pairs are preserved).
 Param  : $input- The input string to strip formatting from.
 Returns: The content portion of $input without formatting.
 Author : Bilge/Assembler Maniac
*/
function stripFormatting($input) {
    return
        //Replace all occurances of a null character back with a pair of dollar
        //signs again
        str_replace("\0", '$$',
            //Replace links (introduced with the advent of TMU)
            preg_replace(
                '/
                #Strip TMU H and L links by stripping everything between each
                #square bracket pair until another $H or $L sequence is found.
                #This allows a $H to close a $L and vice versa, as does the game
                \\$[hl](.*?)(?:\\[.*?\\](.*?))*(?:\\$[hl]|$)
                /ix',
                //Keep the second and third capturing groups if present
                '$1$2',
                //Replace various patterns beginning with an unescaped dollar
                preg_replace(
                    '/
                    #Match a single dollar sign and by any of the following:
                    \\$
                    (?:
                        #Strip colour codes by matching any hexadecimal
                        #character and any other two characters following it
                        [0-9a-f]..
                        #Strip any single character that does not match any of
                        #the above rules and is not an H or L link
                        |[^hl]
                        #Strip the dollar sign if it is at the end of the string
                        |$
                    )
                    #Ignore alphabet case and ignore whitespace in pattern
                    /ix',
                    //Replace any matches with nothing (i.e. strip matches)
                    '',
                    //Replace all occurances of dollar sign pairs with a null
                    //character
                    str_replace('$$', "\0", $input)
                )
            )
        )
    ;
}

/*The following code is a workaround for php's unpack function
which does not have the capability of unpacking double precision
floats that were packed in the opposite byte order of the current
machine.

function big_endian_unpack ($format, $data) {
   $ar = unpack ($format, $data);
   $vals = array_values ($ar);
   $f = explode ('/', $format);
   $i = 0;
   foreach ($f as $f_k => $f_v) {
   $repeater = intval (substr ($f_v, 1));
   if ($repeater == 0) $repeater = 1;
   if ($f_v{1} == '*')
   {
       $repeater = count ($ar) - $i;
   }
   if ($f_v{0} != 'd') { $i += $repeater; continue; }
   $j = $i + $repeater;
   for ($a = $i; $a < $j; ++$a)
   {
       $p = pack ('d',$vals[$i]);
       $p = strrev ($p);
       list ($vals[$i]) = array_values (unpack ('d1d', $p));
       ++$i;
   }
   }
   $a = 0;
   foreach ($ar as $ar_k => $ar_v) {
   $ar[$ar_k] = $vals[$a];
   ++$a;
   }
   return $ar;
}

list ($endiantest) = array_values (unpack ('L1L', pack ('V',1)));
if ($endiantest != 1) define ('BIG_ENDIAN_MACHINE',1);
if (defined ('BIG_ENDIAN_MACHINE')) $unpack_workaround = 'big_endian_unpack';
else $unpack_workaround = 'unpack';
*/

/**
 * Formats a text.
 * replaces parameters in the text which are marked with {n}
 */
function formatText ($text) {

  // get all function's parameters ...
  $args = func_get_args();

  // first parameter is the text to format ...
  $text = array_shift($args);

  // further parameters will be replaced in the text ...
  $parameters = $args;

  // replace all parameters in the text ...
  $i = 0;
  foreach($parameters as $parameter) {
    $i++;
    $text = str_replace('{'.$i.'}', $parameter, $text);
  }

  // and return the modified one.
  return $text;
}

/**
 * Make String for SQL use that single quoted & got special chars replaced
 */
function quotedString($str_in) {
  return '\'' . mysql_real_escape_string($str_in) . '\'';
}

/**
 * Shortens strings that are too long
 * to display as whole.
 *
 * @param string $string
 * @param int $max_length
 * @param string $count_string
 * @return string
 */
function cutString($string, $max_length, $count_string = null) {
  if ($count_string == null) $count_string = $string;
  if (strlen($count_string) > $max_length) {
    return substr($string, 0, $max_length-3).'...';
  } else {
    return $string;
  }
}


// function posted from http://us2.php.net/manual/en/function.memory-get-usage.php#64156
// Warning: This will NOT work for Windows 2000 without some adjustments, see below for where to get more info
if( !function_exists('memory_get_usage') ) {
	function memory_get_usage() {
		//If its Windows
		//Tested on Win XP Pro SP2. Should work on Win 2003 Server too
		//Doesn't work for 2000
		//If you need it to work for 2000 look at http://us2.php.net/manual/en/function.memory-get-usage.php#54642
		if ( substr( PHP_OS, 0, 3 ) == 'WIN' ) {
			$output = array();
			exec( 'tasklist /FI "PID eq ' . getmypid() . '" /FO LIST', $output );
			return preg_replace( '/[\D]/', '', $output[5] ) * 1024;
		} else {
			//We now assume the OS is UNIX
			//Tested on Mac OS X 10.4.6 and Linux Red Hat Enterprise 4
			//This should work on most UNIX systems
			$pid = getmypid();
			// original, slower way using grep - don't use unless 2nd one doesn't work
			// exec("ps -eo%mem,rss,pid | grep $pid", $output);
			// $output = explode("  ", $output[0]);
			//rss is given in 1024 byte units
			//return $output[1] * 1024;

			// faster way to do the find memory directly PID w/out grep
			exec("ps -o rss -p $pid", $output);
			//rss is given in 1024 byte units
			return $output[1] * 1024;
		}
	}
}
?>
