<?php
/**
 * Exchanges information and data with
 * the Aseco Server.
 * 
 * @author Florian Schnell
 */
class DataExchanger {
  private $_host;
  private $_script_path;
  private $duration;
  private $avg_duration;
  private $connection;
  private $requests;
  
  function DataExchanger($host, $script_path) {
    $this->_host = $host;
    $this->_script_path = $script_path;
    $this->connection = fsockopen($this->_host, 80, $errno, $errstr);
    $this->requests = 0;
  }
  
  function sendRequest() {
    
    // build request string ...
    $params = func_get_args();
    $name = array_shift($params);
    $request_string = 'action='.$name;
    for($i = 0; $i < count($params); $i++) {
      $param = $params[$i];
      $request_string .= '&param'.$i.'='.urlencode(utf8_decode($param));
    }
    
    // get start time ...
    $start_time = microtime(true);
    
    // send request and get response ...
    $response = $this->doChunkedRequest($this->_host, $this->_script_path, $request_string);
    
    // get time the request dured ...
    $this->duration = microtime(true) - $start_time;
    if ($this->avg_duration > 0)
      $this->avg_duration = ($this->avg_duration + $this->duration)/2;
    else
      $this->avg_duration = $this->duration;
    
    // return a formatted response from the server ...
    return $this->formatResponse($response);
  }
  
  function keepAlive() {
    
    $this->reopenConnection();
    
    // send request ...
    $request = "HEAD /aseco/publicdb/aseco_server.php HTTP/1.1\r\n";
    $request .= "Host: {$this->_host}\r\n\r\n";
    fwrite($this->connection, $request);
    
    // split header from content ...
    do {
      $responseHeader.= fread($this->connection, 1);
    } while (!preg_match('/\\r\\n\\r\\n$/', $responseHeader));
    
  }
  
  function doChunkedRequest($host, $url, $request_string) {
  
    // connect ...
    if (!$this->connection) return false;
    
    $this->reopenConnection();
    
    // send request ...
    $request = "POST $url HTTP/1.1\r\n";
    $request .= "Host: $host\r\n";
    $request .= "Referer: aseco.php\r\n";
    $request .= "Content-type: application/x-www-form-urlencoded\r\n";
    $request .= "Content-length: ". strlen($request_string) ."\r\n\r\n";
    $request .= $request_string;
    fwrite($this->connection, $request);
  
    $responseHeader = '';
    $responseContent = '';

    // split header from content ...
    do {
      $responseHeader.= fread($this->connection, 1);
    } while (!preg_match('/\\r\\n\\r\\n$/', $responseHeader));
    
    // read content on the normal way ...
    if (!strstr($responseHeader, "Transfer-Encoding: chunked")) {
      while (!feof($this->connection)) {
        $responseContent.= fgets($this->connection, 128);
      }
    // read chunked content ...
    } else {
      
      // read each chunk ...
      while ($chunk_length = hexdec(fgets($this->connection))) {
        $responseContentChunk = '';
        $read_length = 0;
        while ($read_length < $chunk_length) {
          $responseContentChunk .= fread($this->connection, $chunk_length - $read_length);
          $read_length = strlen($responseContentChunk);
        }
        
        // and put them together ...
        $responseContent.= $responseContentChunk;
        fgets($this->connection);
      }
    }
    
    return chop($responseContent);
  }
  
  function reopenConnection() {
    $this->requests++;
    if ($this->requests % 90 == 0) {
      fclose($this->connection);
      $this->connection = fsockopen($this->_host, 80, $errno, $errstr);
    }
  }
  
  function formatResponse($body) {
    
    // parse response body ...
    $result = array();
    $rows = split("\r\n", $body);
    foreach ($rows as $row) {
      $columns = split(chr(9), $row);
      $params = array();
      foreach ($columns as $column) {
        $params[] = utf8_encode(str_replace('<br>', "\r\n", $column));
      }
      $result[] = $params;
    }
    
    // check for errors ...
    if ($result[0][0] == "Error") {
      $result['error'] = true;
      $result['errno'] = $result[1][0];
      $result['errstr'] = $result[2][0];
    } else {
      $result['error'] = false;
    }
    
    return $result;
  }
  
  function getLastRequestTime() {
    return $this->duration;
  }
  
  function getAvgRequestTime() {
    return $this->avg_duration;
  }
}
?>