<?php
/**
 * @author      Florian Schnell
 *
 * Builds an easy structured array out of
 * a xml file.
 * Element names will be the keys and the data the values.
 */

class Examsly {
  var $data;
  var $struct;
  var $parser;
  var $stack;
  var $deep;
  var $attribs;

  function __construct()
  	{
	$this->attribs = false;
  	}  //  __construct

  /*
   * Parses an XML structure into an array
   *	Each tag has 2 parts in an array, Data and Attributes
   */

  function parseXmlAttribs($source, $isfile = true)
  	{
	$this->attribs = true;
	return $this->parseXml($source, $isfile);
  	}  //  parseXMLAttribs

  /**
   * Parses a XML structure into an array.
   */

  function parseXml($source, $isfile = true) {

    // clear last results ...
    $this->stack = array();
    $this->struct = array();

    // create the parser ...
    $this->parser = xml_parser_create();
    xml_set_object($this->parser, $this);
    xml_set_element_handler($this->parser, "openTag", "closeTag");
    xml_set_character_data_handler($this->parser, "tagData");

    // load the xml file ...
    if ($isfile) {
      $this->data = file_get_contents($source);
    } else {
      $this->data = $source;
    }

    // parse xml file ...
    $parsed = xml_parse($this->parser, $this->data);

    // display errors ...
    if (!$parsed) {
      $code = xml_get_error_code($this->parser);
      $err = xml_error_string($code);
      $line = xml_get_current_line_number($this->parser);
      trigger_error("[XML Error $code] $err on line $line", E_USER_WARNING);
      return false;
    }

	$this->attribs = false;
    return $this->struct;
  }

  function openTag($parser, $name, $attributes) {
    $this->stack[] = $name;
    $this->struct[$name] = "";
	if ( $this->attribs )
		{
		$this->struct[$name . '-Attrib'] = $attributes;
		}
  }

  function tagData($parser, $data) {
    if (trim($data)) {
      $index = $this->stack[count($this->stack)-1];
      $this->struct[$index] .= utf8_decode(urldecode($data));
    }
  }

  function closeTag($parser, $name) {
    if (count($this->stack) > 1) {
      $from = array_pop($this->stack);
      $to = $this->stack[count($this->stack)-1];
	  if ( $this->attribs )
		{
      	$this->struct[$to][$from][] = array('Data'=>$this->struct[$from], 'Attributes'=>$this->struct[$from . '-Attrib']);
      	unset($this->struct[$from]);
      	unset($this->struct[$from . '-Attrib']);
		}
	  else
		{
      	$this->struct[$to][$from][] = $this->struct[$from];
      	unset($this->struct[$from]);
		}
    }
	else
		{
		if ( $this->attribs )
			{
			$this->struct[$name] = array('Data'=>$this->struct[$name], 'Attributes'=>$this->struct[$name . '-Attrib']);
			unset($this->struct[$name . '-Attrib']);
			}
		}

  }

  /**
   * Parses an array into an XML structure.
   */

	function parseArray($array)
		{
		$xmlstring = '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>';
    	$this->deep = 0;
		$topkey = key($array);
		$xmlstring .= "\r\n" . '<' . $topkey . '>';
    	$xmlstring .= $this->parseArrayElements($array);
		$xmlstring .= "\r\n" . '</' . $topkey . '>';
    	return $xmlstring;
  		}

	function parseArrayElements($array, $opt_tag = "")
		{
		$xml = '';

		// read each element of the array ...
		for($i = 0; $i<sizeof($array); $i++)
			{
		  // check if array is associative ...
			if(is_numeric(key($array)))
				{
				$this->deep++;
				$xml .= "\r\n";
				for($j = 0; $j<$this->deep; $j++)
					$xml .= "\t";

				if ( !is_array(current($array)) && strlen(htmlentities(utf8_encode(current($array)))) == 0 )
					{
					$xml .= '<' . $opt_tag . ' />';
					}
				else
					{
					$xml .= '<'.$opt_tag.'>';
				    if(is_array(current($array)))
						{
						$xml .= $this->parseArrayElements(current($array), key($array));
						$xml .= "\r\n";
						for($j = 0; $j<$this->deep; $j++)
							$xml .= "\t";
				        }
					else
						{
						$xml .= htmlentities(utf8_encode(current($array)));
				        }
				    $xml .= '</' . $opt_tag . '>';
				    if(is_array(current($array))) $xml .= "\r\n";
					}
				$this->deep--;
  				}
			else
				{
		    	if(is_array(current($array)))
					{
		      		$xml .= $this->parseArrayElements(current($array), key($array));
		    		}
				else
					{
					$xml .= "\r\n" . str_repeat("\t", $this->deep);
			  		if ( strlen(htmlentities(utf8_encode(current($array)))) == 0)
						{
						$xml .= '<' . key($array) . ' />';
						}
			  		else
						{
						$xml .= '<' . key($array) . '>';
		        		$xml .= htmlentities(utf8_encode(current($array)));
		        		$xml .= '</' . key($array) . '>';
						}
		    		}
		  		}
		  	next($array);
			}
		return $xml;
		}
	}
?>
