<?php
class LocalDb extends Plugin {
  private $settings;
  public $records;
  public $challenge;
  public $players;

  function startup($param) {

    // init fields ...
    $this->settings = array();
    $this->players = array();
    $this->challenge = array();

    // load local database settings ...
    $this->loadSettings();

    // display new status message ...
    $this->console_text("Try to connect to MySQL server");

    // connect to the mysql database ...
    $this->connect();

    // display server version ...
    $this->console_text("MySQL Server Version is " . mysql_get_server_info());
  }

  function loadSettings() {

    // show status message ...
    $this->Aseco->console_text("[LocalDb] Load settings file ...");

    // create new xml parser object and parse the settingsfile ...
    $xml_parser = new Examsly();
    $settings = $xml_parser->parseXml('settings/localdb.xml');
    $settings = $settings['SETTINGS'];

    // read mysql server settings ...
    $this->settings['mysql']['host'] = $settings['MYSQL_SERVER'][0];
    $this->settings['mysql']['login'] = $settings['MYSQL_LOGIN'][0];
    $this->settings['mysql']['password'] = $settings['MYSQL_PASSWORD'][0];
    $this->settings['mysql']['database'] = $settings['MYSQL_DATABASE'][0];

    // read message settings ...
    $this->settings['messages'] = $settings['MESSAGES'][0];

    // display records in game?
    if (strtoupper($settings['DISPLAY'][0]) == "FALSE")
      $this->setSilent(true);

    // read honors ...
    $this->settings['honors'] = $settings['HONORS'];
  }

  function connect() {

    // connect ...
    if (!mysql_connect($this->settings['mysql']['host'],
    $this->settings['mysql']['login'],
    $this->settings['mysql']['password'])) {
      trigger_error("[LocalDb] Could not authenticate at MySQL server!", E_USER_ERROR);
    }

    // select database ...
    if (!mysql_select_db($this->settings['mysql']['database'])) {
      trigger_error("[LocalDb] Could not find MySQL database!", E_USER_ERROR);
    }

    // configure ...
    $result = mysql_query('SET CHARSET "utf8"');
    $result = mysql_query('SET NAMES "utf8"');
  }

  function newChallenge($challenge) {

    // get new records etc from database ...
    $this->dbGetChallenge($challenge);

    if ($this->challenge['Votes'] > 0)
      $avg_score = $this->challenge['Score']/$this->challenge['Votes'];
    else
      $avg_score = 0;
    //$this->console('challenge\'s score is {1}', $avg_score);
  }

  function playerConnect(&$player) {

    // try to update the player ...
    if ($this->dbUpdatePlayer($player)) {

      // if he was updated successfully, then get his stats ...
      $playerstats = $this->dbGetPlayerStats($player);
      $this->players[$player->login] = $playerstats;

    } else { // could not update player - is not in db yet!

      // insert player and don't get any stats yet ...
      if (!$this->dbInsertPlayer($player)) {
        $this->trigger_warning('could not insert player into database!');
        $this->Aseco->console(mysql_error());
      }
    }
  }

  function playerDisconnect(&$player) {

    // ignore players with empty logins ...
    if ($player->login == "") return;

    // remove player from list ...
    unset($this->players[$player->login]);

    if (!$this->dbUpdatePlayerStats($player)) {
      $this->trigger_warning('Could not update leaving player in database!');
    }
  }

  function raceEnd() {

    echo '[LocalDB] Transferring records to local database ...';
    doLog('[LocalDB] Transferring records to local database ...'.CRLF);

    // Insert records into the database ...
    for ($i = 0; $i < $this->Aseco->records->count(); $i++) {
      $record = $this->Aseco->records->getRecord($i);
      if ($record->isnew) // only insert new set up records ...
        $this->dbInsertRecord($record);
    }

    echo ' Done!'.CRLF;
    doLog(' Done!'.CRLF);
  }

  function playerWins($player) {

    // calculate total wins ...
    $wins = $this->players[$player->login]['Wins'] + $player->getWins();

    // display console message ...
    $this->console('{1} won for the {2}. time!', $player->login, $wins);

    // replace parameters ...
    $message = formatText('{#server}>> Congratulations, you won your {#highlite}{1}{#server}. race!',
    $wins);

    // replace colors ...
    $message = $this->Aseco->formatColors($message);

    // send the message ...
    $this->sendServerMessageToLogin($player->login, $message);

    // check if this is a win that should be honored ...
    foreach($this->settings['honors'] as $honor) {
      if ($honor['WINS'][0] == $wins) {

        // build the message ...
        $message = formatText($honor['MESSAGE'][0], stripFormatting($player->nickname));

        // replace colors ...
        $message = $this->Aseco->formatColors($message);

        // send the message ...
        $this->sendServerMessage($message);
      }
    }

    // update player wins in db ...
    $this->dbUpdatePlayerWins($player);
  }

  /**************************************
  * Database functions
  **************************************/

  function dbUpdatePlayerWins($player) {

    $sql = 'UPDATE players
            SET UpdatedAt=NOW(), Wins=Wins+1
            WHERE Login='.quotedString($player->login).
            ' AND Game='.quotedString($this->Aseco->server->getGame());

    $result = $this->db_query($sql);

    if ( mysql_affected_rows() != 1 ) {
      $this->trigger_warning('Could not update winning player in database! ('
                    .mysql_error().")\r\nsql=$sql");
    }
  }

  function dbInsertRecord($record) {

    if (!($playerid = $this->dbGetPlayerId($record->player))) {
      $this->trigger_warning("Player was not found in the database!");
    }

    $query = "INSERT INTO records
              (ChallengeId, PlayerId, Score, Date)
              VALUES
              (".$this->challenge['Id'].", {$playerid}, {$record->score}, NOW())";

    // commit ...
    $result = $this->db_query($query, false);

    // could not be inserted?
    // so player had a record already ...
    if (mysql_affected_rows() != 1) {

      $query = "UPDATE records
                SET Score={$record->score}, Date=NOW()
                WHERE ChallengeId=".$this->challenge['Id']." AND PlayerId={$playerid}";

      // commit ...
      $result = $this->db_query($query);

      // could not be updated?
      // then there's something going wrong!
      if (mysql_affected_rows() != 1) {
        $this->trigger_warning("Could not insert record into database!");
      }
    }
  }

  function dbUpdatePlayerStats($player) {

    // update the player in the database ...
    $query = "UPDATE players
              SET UpdatedAt=NOW(),
              TimePlayed=TimePlayed+".$player->getTimeOnline()."
              WHERE Login='{$player->login}' AND Game='".$this->Aseco->server->getGame()."'";

    // commit ...
    $result = $this->db_query($query);

    return (mysql_affected_rows() == 1);
  }

  function dbGetChallengeId($uid) {

    // ... get its Id now ...
    $query = "SELECT c.Id
              FROM challenges c
              WHERE c.Uid='{$uid}'";

    $result = $this->db_query($query);

    if (mysql_num_rows($result) == 1) {

      // challenge was found, return its id now ...
      $row = mysql_fetch_row($result);
      $retval = $row[0];

    } else {

      // challenge Id could not be found ...
      $this->trigger_warning('Could not get challenge\'s Id!');
      $retval = -1;
    }

    mysql_free_result($result);
    return $retval;
  }


  function dbGetPlayerId($player) {

    // build query ...
    $query = "SELECT Id
              FROM players
              WHERE Login='{$player->login}' AND Game='{$this->Aseco->server->getGame()}'";

    // commit ...
    $result = $this->db_query($query);

    // return result ...
    if (mysql_num_rows($result) > 0) {
      $row = mysql_fetch_row($result);
      $rtnval = $row[0];
    } else {
      $rtnval = 0;
    }

    mysql_free_result($result);
    return $rtnval;
  }

  function dbUpdatePlayer($player) {

    // try to update the player ...
    $query = "UPDATE players SET
              NickName=".quotedString($player->nickname).",
              Nation=".quotedString($player->nation).",
              UpdatedAt=NOW()
              WHERE Login=".quotedString($player->login)." AND Game=".quotedString($this->Aseco->server->getGame());

	// commit ...
    $result = $this->db_query($query);

    // was updated?
    return (mysql_affected_rows() > 0);
  }

  function dbInsertPlayer($player) {

    // insert player into database ...
    $query = "INSERT INTO players
              (Login, Game, NickName, Nation, UpdatedAt)
              VALUES
              (".quotedString($player->login).
              ", ".quotedString($this->Aseco->server->getGame()).
              ", ".quotedString($player->nickname).
              ", ".quotedString($player->nation).
              ", NOW())";

    // commit ...
    $result = $this->db_query($query, false);

    // was inserted?
    return (mysql_affected_rows() > 0);
  }

  function dbGetPlayerStats($player) {

    // get stats of the player ...
    $query = "SELECT Wins AS wins, TimePlayed AS timeplayed
    FROM players
    WHERE Login='{$player->login}' AND Game='".$this->Aseco->server->getGame()."'";

    // commit ...
    $result = $this->db_query($query);

    // save player stats ...
    $dbplayer = mysql_fetch_object($result);
    $stats = array();
    $stats['Wins'] = $dbplayer->wins;
    $stats['TimePlayed'] = $dbplayer->timeplayed;
    $stats['HasVoted'] = false;

    return $stats;
  }

  function dbGetChallenge($challenge) {

	// 08-8-5 AM - added test for game mode, if it stunts, order by scores is Higher=Better, otherwise lower=better
    if ( $this->Aseco->server->gameinfo->getMode() == 'Stunts' )
		{
		$orderby = 'r.Score DESC, r.Date ASC';
		}
	else
		{
		$orderby = 'r.Score ASC, r.Date ASC';
		}

    $query = 'SELECT c.Id AS ChallengeId, r.Score, p.NickName, p.Login, r.Date, SUM(v.Score) AS VotingSum, COUNT(v.Score) AS VotingCount
            FROM challenges c
            LEFT JOIN records r ON (r.ChallengeId=c.Id)
            LEFT JOIN votes v ON (v.ChallengeId=c.Id)
            LEFT JOIN players p ON (r.PlayerId=p.Id)
            WHERE c.Uid='.quotedString($challenge->uid).'
            GROUP BY r.Id
            ORDER BY ' . $orderby . '
            LIMIT ' . $this->Aseco->records->getLimit();

    // commit ...
    $result = $this->db_query($query);

    // challenge found in database ...
    if (mysql_num_rows($result) > 0) {

      // get each record ...
      while ($record = mysql_fetch_array($result)) {

        // create record object ...
        $record_item = new Record();
        $record_item->score = $record['Score'];

        // create a player object to put it into the record object ...
	    $player_item = new RecPlayer();
        $player_item->nickname = $record['NickName'];
        $player_item->login = $record['Login'];
        $record_item->player = $player_item;

        // add the track information to the record object ...
        $record_item->challenge = $challenge;

        // add the created record to the list ...
        $this->Aseco->records->addRecord($record_item);
        unset($record_item);

        $this->challenge['Id'] = $record['ChallengeId'];
        $this->challenge['Score'] = $record['VotingSum'];
        $this->challenge['Votes'] = $record['VotingCount'];
      }

      // free mysql result ...
      mysql_free_result($result);

    } else { // challenge could not be found in database ...

      // then create it ...
      $query = "INSERT INTO challenges
                (Uid, Name, Author, Environment)
                VALUES
                (\"{$challenge->uid}\", \"{$challenge->name}\", '{$challenge->author}', '{$challenge->environment}')";

      // commit ...
      $result = $this->db_query($query);

      // reset scores and votings ...
      $this->challenge['Score'] = 0;
      $this->challenge['Votes'] = 0;

      // challenge was inserted successfully ...
      if (mysql_affected_rows() == 1) {

        // get challenge's id ...
        $this->challenge['Id'] = $this->dbGetChallengeId($challenge->uid);
      } else {

        // challenge could not be inserted ...
        $this->trigger_warning('Could not insert challenge into database!');
      }
    }
  }

  function dbInsertVote($vote) {

    $score = $vote['params'];
    $player = $vote['author'];

    if (!($playerid = $this->dbGetPlayerId($player))) {
      $this->trigger_warning('Player was not found in the database!');
    }

    // insert vote into database ...
    $query = "INSERT INTO votes
              (Score, PlayerId, ChallengeId)
              VALUES
              ({$score}, {$playerid}, {$this->challenge['Id']})";

    // commit ...
    $result = $this->db_query($query, false);

    return (mysql_affected_rows() == 1);
  }

  /**
   * Tries to execute a mysql query.
   * If the response is wrong, then examine if the
   * connection has timed out.
   * Reconnect and resend the query if it has.
   *
   */
  function db_query($query, $err_report = true) {
    if (!($result = mysql_query($query))) {
      if ($err_report) $this->console_text('[MySQL]['.mysql_errno().'] '.mysql_error());
      if(!mysql_ping()) {
        mysql_free_result($result);
        mysql_close();
        $this->connect();
        return mysql_query($query);
      }
    }
    return $result;
  }
}

$_PLUGIN = new LocalDb();
$_PLUGIN->setAuthor('Florian Schnell');
$_PLUGIN->setVersion(1.1);
$_PLUGIN->addEvent('onStartup', 'startup');
$_PLUGIN->addEvent('onNewChallenge', 'newChallenge');
$_PLUGIN->addEvent('onPlayerConnect', 'playerConnect');
$_PLUGIN->addEvent('onPlayerDisconnect', 'playerDisconnect');
$_PLUGIN->addEvent('onEndRace', 'raceEnd');
$_PLUGIN->addEvent('onPlayerWins', 'playerWins');
$_PLUGIN->setDescription('Builds the database interface for other plugins. Stores information in the database and manages records driven on the server in a local database. Use the localdb.xml to configure this plugin.');
?>
