<?php
class PubDb extends Plugin
	{
	private $mysqli_conn;
	private $_server;
	private $_login;
	private $_password;
	private $_challengeid;
	private $_online;
    private $last_keep_alive;
	private $keep_alives;

	function dbConnect()
		{
		if ( isset($this->mysqli_conn) )
			{
			$this->mysqli_conn->close();
			unset($this->mysqli_conn);
			}
		// everything I've tried to suppress a PHP Warning message if connect fails, has failed, we're currently stuck with extra crap in the console if connect fails
		$this->mysqli_conn = new mysqli($this->_server, $this->_login, $this->_password, 'aseco_public');

		if ( mysqli_connect_errno() )
			{
			$this->console_text('Failed to connect, not currently available - ' . mysqli_connect_error() . '(' . mysqli_connect_errno() . ')');
			$this->_online = false;
			unset($this->mysqli_conn);
			return false;
			}
		else
			{
			$this->_online = true;
			$this->mysqli_conn->set_charset('utf8');
			}

    	// get server name ...
    	$servername = $this->Aseco->doCall('GetServerName');
		$result = $this->mysqli_conn->query("call setServerName(" . $this->quotedString($servername) . ")");
		if ( $result )
			{
			$row = $result->fetch_row();		// $row[0] = 1 means success, =0 means failed
			$result->free();
			}
		else
			{
			$this->console_text('[pubdb] mysql error - ' . $this->mysqli_conn->error);
			}

		// free up phantom results, this will eventually be patched in the server, but for now it's necessary
		while ( $this->mysqli_conn->more_results() )
			{
			if ( $this->mysqli_conn->next_result() )
				{
				$result = $this->mysqli_conn->use_result();
				}
			}
		return $this->_online;
		}  //  dbConnect

  /**
   * Gets news from the server and displays
   * them. Also validates the user's account.
   */
	function startup()
  	{
    // load settings ...
    $this->loadSettings();

    $this->console_text('Connecting to Public Database Server ...');
	// connect to remote database using _login / _pass, save connection in object var

	if ( !$this->dbConnect() )
		{
		return;
		}

	$this->console_text('Account validation complete, access permitted!');

    // get news from server and display them...
  	$query = "SELECT Date, Subject, Text FROM news ORDER BY Date DESC LIMIT 3";
	$result = $this->mysqli_conn->query($query);
	if ( $result )
		{
		$this->console_text('#- Server News -----------------------------------------------------');
		$this->console_text('|');
		while ( $row = $result->fetch_row() )
			{
			$po_news = explode("\r\n", $row[2]);
			$this->console_text('| ' . $row[0] . ' - ' . $row[1] . ':');
			foreach ($po_news as $news_line)
				{
				$lines = str_split($news_line, 66);
				foreach ($lines as $line)
					{
					$this->console_text('| '.$line);
					}
				}
			$this->console_text('|');
			}
		$this->console_text('#-------------------------------------------------------------------');
		$result->free();		// free the result set
		}
	else
		{
		// there's no results, check for errors
		echo "error: " . $this->mysqli_conn->error . CRLF;
		}

	}

	function pingCheck()
		{
		if ( !$this->_online )
			{
			$this->dbConnect();		// we weren't online for some reason, try again
			}

		if ( $this->_online )	// yes, now test for online again, it looks like it should be in the else, but the online status may have changed in dbConnect
			{
			$this->_online = $this->mysqli_conn->ping(); // result is true/false for whether or not server is connected
			if ( !$this->_online )
				{
				$merrno = $this->mysqli_conn->errno;
				$merror = $this->mysqli_conn->error;
				$this->mysqli_conn->close();
				unset($this->mysqli_conn);
				$this->console_text('Failed to ping, not currently available - ' . $merror . '(' . $merrno . ')');
				}
			}

		if ( $this->_online )	// it's still online, is processing enabled?
			{
			$result = $this->mysqli_conn->query("select online from settings where id=1");
			if ( $result )
				{
				$row = $result->fetch_row();
				$result->free();
				if ( $row[0] != 1 )
					{
					$this->console_text("Server is offline");
					return 0;			// processing temporarily disabled, mysql server is alive, but not handling requests
					}
				}

			}

		return $this->_online;

		}  //  pingCheck


  /**
   * Updates the client's online status
   * every minute.
   */
	function keepAlive()
  		{
		if ((time() - $this->last_keep_alive) < 60) return;

		$this->last_keep_alive = time();

		if ($this->pingCheck())		// is it time to do a keepalive and is the connection still valid?
			{
			$playerlist = '';

			$this->Aseco->server->players->resetPlayers();
			while ($player = $this->Aseco->server->players->nextPlayer())
				{
				$playerlist .= $player->login.',';
				}
			if (strlen($playerlist) > 0)
				$playerlist = substr($playerlist, 0, strlen($playerlist) - 1);

			$result = $this->mysqli_conn->query('call keepAlive(' . $this->quotedString($playerlist) . ')');
			if ( $result )
				{
				$row = $result->fetch_row();
				if ($row[0] == 1)
					$this->console_debug('sent a keep alive signal!');
				$result->free();
				}
			else
				{
				$this->console_text('MySqlI failure - ' . $this->mysqli_conn->error);
				}

			$this->keep_alives = 1;

			// free up phantom results, this will eventually be patched in the server, but for now it's necessary
			while ( $this->mysqli_conn->more_results() )
				{
				if ( $this->mysqli_conn->next_result() )
					{
					$result = $this->mysqli_conn->use_result();
					}
				}
			}
		}

	function newChallenge($challenge)
		{
		if ( !$this->pingCheck() ) return;

		$query = 'call getChallengeInfo(' . $this->quotedString($challenge->uid) . ', ' .
                                             $this->Aseco->records->getLimit() . ', ' .
                                             $this->quotedString($challenge->name) . ', ' .
                                             $this->quotedString($challenge->author) . ', ' .
                                             $this->quotedString($challenge->environment) . ', ' .
                                             $this->quotedString($challenge->mood) . ')';

		$result = $this->mysqli_conn->query($query);
		if ( !$result )
			{
			$this->console_text('MySqlI failure - ' . $this->mysqli_conn->error);
			return;
			}

		$this->_challengeid = -1;

		while ( $result->num_rows > 0 && $record = $result->fetch_row() )
			{
			if ( $this->_challengeid == -1 )
				{
				$this->_challengeid = $record[0];
				}

			if ( $record[1] )		// if this value is null (not set), there are no records, so it will ext loop gracefully
				{
				// create record object ...
				$record_item = new Record();
				$record_item->score = $record[1];

				// create a player object to put it into the record object ...
				$player_item = new RecPlayer();
				$player_item->login = $record[2];
				$player_item->nickname = $record[3];
				$record_item->player = $player_item;

				// add the track information to the record object ...
				$record_item->challenge = $challenge;

				// add the created record to the list ...
				$this->Aseco->records->addRecord($record_item);
				}
			}
		$result->free();

		// free up phantom results, this will eventually be patched in the server, but for now it's necessary
		while ( $this->mysqli_conn->more_results() )
			{
			if ( $this->mysqli_conn->next_result() )
				{
				$result = $this->mysqli_conn->use_result();
				}
			}
		}

	function raceEnd()
	  	{
		if ( !$this->pingCheck() ) return;

	    echo '[PubDb] Transferring records to server ...' . CRLF;
	    doLog('[PubDb] Transferring records to server ...' . CRLF);

	    // send records to the database ...
	    for ($i = 0; $i < 5; $i++)
			{
			$record = $this->Aseco->records->getRecord($i);
			if ($record->isnew)
				{
				$this->console_debug('sending record ...');
				$query = 'call insertRecord(' . $this->_challengeid . ', ' .
               	                              $this->quotedString($record->player->login) . ', ' .
                       	                      $record->score . ')';
				$result = $this->mysqli_conn->query($query);
				if ( $this->mysqli_conn->errno )
					{
					echo "mysqli error: " . $this->mysqli_conn->error . CRLF;
					echo "Query=" . $query . CRLF;
					}

				if ( $result )
					{
					$row = $result->fetch_row();
	    			if ($row[0] == 0)
						{
						$this->trigger_warning('Could not send record to server - ' . $row[1]);
						}

					$result->free();
					}

				// free up phantom results, this will eventually be patched in the server, but for now it's necessary
				while ( $this->mysqli_conn->more_results() )
					{
					if ( $this->mysqli_conn->next_result() )
						{
						$result = $this->mysqli_conn->use_result();
						}
					}
				}
			}
		doLog(' Done!'.CRLF);
		}

	function playerConnect($player)
		{
		if ( !$this->pingCheck() ) return;

		$ladderscore = $player->ladderscore;
		if ( !$ladderscore )
			{
			$ladderscore = 0;
			}
		$query = 'call insertPlayer(' .
                                $this->quotedString($player->login) . ', ' .
                                $this->quotedString($player->nickname) . ', ' .
                                $ladderscore . ')';
    	$result = $this->mysqli_conn->query($query);

		if ( $this->mysqli_conn->errno )
			{
			echo "mysqli error: " . $this->mysqli_conn->error . CRLF;
			echo "Query=" . $query . CRLF;
			}

		if ( $result )
			{
			$row = $result->fetch_row();
			$result->free();

			// free up phantom results, this will eventually be patched in the server, but for now it's necessary
			while ( $this->mysqli_conn->more_results() )
				{
				if ( $this->mysqli_conn->next_result() )
					{
					$result = $this->mysqli_conn->use_result();
					}
				}

			if ( $row[0] == 0 )
				{
				$this->trigger_warning($row[1]);
				}
			}
  		}

	function playerDisconnect($player)
  		{
		if ( !$this->pingCheck() ) return;

		$result = $this->mysqli_conn->query('call updatePlayerStatus(' . $this->quotedString($player->login) . ')');
		$errstg = $this->mysqli_conn->error;
		$row = $result->fetch_row();
		$result->free();

		// free up phantom results, this will eventually be patched in the server, but for now it's necessary
		while ( $this->mysqli_conn->more_results() )
			{
			if ( $this->mysqli_conn->next_result() )
				{
				$result = $this->mysqli_conn->use_result();
				}
			}

		if ( $row[0] == 0 )
			{
			$this->trigger_warning($row[1] . ' - ' . $errstg);
			}
  		}

	function getCPTimes($challenge_uid)		// returns results of get, either a single col with times, '', or error (2 cols)
		{
		if ( !$this->pingCheck() ) return;
		$result = $this->mysqli_conn->query('call getCPTimes(' . $this->quotedString($challenge_uid) . ')');
		$errstg = $this->mysqli_conn->error;
		$row = $result->fetch_row();
		$result->free();

		// free up phantom results, this will eventually be patched in the server, but for now it's necessary
		while ( $this->mysqli_conn->more_results() )
			{
			if ( $this->mysqli_conn->next_result() )
				{
				$result = $this->mysqli_conn->use_result();
				}
			}

		if ( $row[0] == 0 )
			{
			$this->trigger_warning($row[1] . ' - ' . $errstg);
			}

		return $row;
		}  //  getCPTimes($challenge)

	function putCPTimes($challenge_uid, $cptimes)
		{
		if ( !$this->pingCheck() ) return;
		$result = $this->mysqli_conn->query('call putCPTimes(' . $this->quotedString($challenge_uid) . ',' . $this->quotedString($cptimes) . ')');
		$errstg = $this->mysqli_conn->error;
		$row = $result->fetch_row();
		$result->free();

		// free up phantom results, this will eventually be patched in the server, but for now it's necessary
		while ( $this->mysqli_conn->more_results() )
			{
			if ( $this->mysqli_conn->next_result() )
				{
				$result = $this->mysqli_conn->use_result();
				}
			}

		if ( $row[0] == 0 )
			{
			$this->trigger_warning($row[1] . ' - ' . $errstg);
			}

		return $row;
		}  //  putCPTimes($challenge_uid, $cptimes)

  /**
   * Loads settings from an extern xml file.
   */
	function loadSettings()
  		{

		// show status message ...
		$this->console_text("Load settings file ...");

		// create new xml parser object and parse the settingsfile ...
		$xml_parser = new Examsly();
		$settings = $xml_parser->parseXml('settings/pubdb.xml');
		$settings = $settings['SETTINGS'];

		// read mysql server settings ...
		$this->_server = $settings['SERVER_HOST'][0];
		$this->_login = $settings['SERVER_LOGIN'][0];
		$this->_password = $settings['SERVER_PASSWORD'][0];

		// display records in game?
		if (strtoupper($settings['DISPLAY'][0]) == "FALSE")
		  $this->setSilent(true);
		}

	/**
	 * Make String for SQL use that single quoted & got special chars replaced
	 */
	function quotedString($str_in)
		{
		return '\'' . $this->mysqli_conn->real_escape_string($str_in) . '\'';
		}

}

$_PLUGIN = new PubDb();
$_PLUGIN->setAuthor('Assembler Maniac');
$_PLUGIN->setVersion(1.0);
$_PLUGIN->addEvent('onStartup', 'startup');
$_PLUGIN->addEvent('onMainLoop', 'keepAlive');
$_PLUGIN->addEvent('onNewChallenge', 'newChallenge');
$_PLUGIN->addEvent('onEndRace', 'raceEnd');
$_PLUGIN->addEvent('onPlayerConnect', 'playerConnect');
$_PLUGIN->addEvent('onPlayerDisconnect', 'playerDisconnect');
$_PLUGIN->setDescription('Stores records in a public database. This way records can be driven world wide and server independent!');
?>
