<?php
class Statusbar extends Plugin {

  private $_window;
  private $_buttons;
  private $_uptime;
  private $_minimized;

  function showToPlayer($player) {
    if ($this->existsPlayerData($player, 'window')) {
      $window = $this->getPlayerData($player, 'window');
    } else {
      $this->_minimized[$player->login] = true;
      $this->_uptime = $this->Aseco->getUptime();
      $window = $this->createWindowToPlayerFramework('Statusbar', 1, 0, $player, true);
      $window->getFramework()->addEvent('onMinimize', $this, 'refresh');
      $window->getFramework()->addEvent('onMaximize', $this, 'refresh');
      $window->addCallback('hide', 'hideStatusbar');
      $window->addCallback('show', 'showStatusbar');
      $window->addCallback('button', 'pressButton');
      $window->setTextsize(1);
      $this->storePlayerData($player, 'window', $window);

      // show the welcome window again if there's any ...
      $window->getFramework()->openRecentMinimizedWindow();
    }

	  if ($this->_minimized[$player->login]) {
	    $window->setBackground('fffb');
	    
	    // create the bar ...
	    $xml = '<quad halign="left" pos=".3 0" size=".55 .05" style="Bgs1InRace" substyle="BgButtonBig" />
	    <label textsize="1" textcolor="000" pos="-.08 -.015 -.11" halign="left" valign="top" text="Aseco v'.ASECO_VERSION.'" />';
	    
	    // create the button ...
	    $xml .= '<quad size=".06 .03" pos="0 -.01 -.1" halign="left" valign="top" style="BgsPlayerCard" substyle="BgActivePlayerCard" action="{name:show,param:0}" />
	    <label textsize="1" textcolor="000" pos="-.017 -.015 -.11" halign="left" valign="top" text="&gt;&gt;" />';
	    
	    $window->setContent($xml);
	    
	  } else {
	    
	    // add buttons ...
	    $cur_width = 0.08;
	    for ($i = 0; $i < count($this->_buttons); $i++) {
	    
	      $button = $this->_buttons[$i];
	      
	      // create a button for each command ...
	      $xml .= '<quad size="'.$button['width'].' .03" pos="-'.$cur_width.' -.01 -.1" halign="left" valign="top" style="BgsPlayerCard" substyle="ProgressBar" action="{name:button,param:'.$i.'}" />
	      <label textsize="1" textcolor="fff" pos="-'.($cur_width+0.017).' -.015 -.11" halign="left" valign="top" text="'.$button['title'].'" />';

	      $cur_width += $button['width']+0.01;
	      
	    }

	    $days = floor($this->_uptime/(3600*24));
	    $hours = floor(($this->_uptime%(3600*24))/3600);
	    $mins = floor((($this->_uptime%(3600*24))%3600)/60);
	    
	    // create the button ...
	    $xml .= '<quad size=".06 .03" pos="0 -.01 -.1" halign="left" valign="top" style="BgsPlayerCard" substyle="BgActivePlayerCard" action="{name:hide,param:0}" />
	    <label textsize="1" textcolor="000" pos="-.017 -.015 -.11" halign="left" valign="top" text="&lt;&lt;" />';
	    
	    $bar_width = 0.85;
	    if ($this->Aseco->isAdmin($player->login))  {
	    	$bar_width = 0.88;
	    	$xml .= '<quad size=".07 .07" pos="-'.($cur_width+0.5).' 0.01 -.1" halign="left" valign="top" style="Icons128x128_1" substyle="Padlock" />';
	    }
	    
	    // create the bar ...
	    $xml .= '<quad halign="left" pos=".3 0" size="'.($cur_width+$bar_width).' .05" style="Bgs1InRace" substyle="BgButtonBig" />
	    <label textsize="1" textcolor="000" pos="-'.($cur_width+0.01).' -.015 -.11" halign="left" valign="top" text="Aseco v'.ASECO_VERSION.' running for '.$days.' days '.$hours.' hours and '.$mins.' minutes" />';
	    
	    $window->setContent($xml);
	    $window->setBackground('fffb');
	  }
	  
		if ($player->framework->getHeight() > 0.095)
		  $window->setPosY(0.745-$player->framework->getHeight());
		else
		  $window->setPosY(0.65);
	}

	function pressButton($param, $window, $player) {
	  $button = $this->_buttons[$param];
	  if (!method_exists($button['cb_obj'], $button['cb_func'])) return;
	  call_user_func_array(array(&$button['cb_obj'], $button['cb_func']), array($player));
	}

	function hideStatusbar($param, $window, $player) {
	  $this->_minimized[$player->login] = true;
	  $this->showToPlayer($player);
	}

	function showStatusbar($param, $window, $player) {
	  $this->_minimized[$player->login] = false;
	  $this->showToPlayer($player);
	}

	function removePlayer($player) {
    $this->removePlayerData($player, 'window');
	}

	function refresh($framework) {
	  $this->showToPlayer($framework->getOwner());
	}

	function refreshAllPlayers() {
	  $this->Aseco->server->players->resetPlayers();
	  while($player = $this->Aseco->server->players->nextPlayer()) {
	    $this->showToPlayer($player);
	  }
	}

	function addButton($title, $callback_obj, $callback_func, $width = 0.1) {
	  $button['title'] = $title;
	  $button['cb_func'] = $callback_func;
	  $button['cb_obj'] = $callback_obj;
	  $button['width'] = $width;
	  $this->_buttons[] = $button;
	  $this->refreshAllPlayers();
	}

	function checkForRefresh() {
	  // refresh every minute ...
	  if ($this->Aseco->getUptime() - $this->_uptime > 60) {
	    $this->_uptime = $this->Aseco->getUptime();
	    $this->refreshAllPlayers();
	  }
	}
}

$_PLUGIN = new Statusbar();
$_PLUGIN->setAuthor('Florian Schnell');
$_PLUGIN->setVersion(1.0);
$_PLUGIN->setDescription('Displays Aseco runtime information.');
$_PLUGIN->addEvent('onPlayerConnect', 'showToPlayer');
$_PLUGIN->addEvent('onPlayerDisconnect', 'removePlayer');
$_PLUGIN->addEvent('onMainLoop', 'checkForRefresh');
?>
