<?php
// Changes Timelimit for TimeAttack dynamically depending on the next tracks authortime
// Loosely based on version by ck|cyrus and mods by Mistral

class RaspAutotime extends Plugin
	{
	private $multiplier = 3.6;	// the multiplier for authortime (e.g. 7 x authortime = new timelimit). Set to 0 to disable func
	// all values are in seconds
	private $authadjust = -60;	// This track's authortime will be adjusted by this much
	private $calcadjust = 330;	// Add this to new computed time
	private $mintime = 180;		// minimum timelimit
	private $maxtime = 420;		// maximum timelimit
	private $defaulttime = 390; // default time in case something goes wrong trying to load the author time
	private $displaymode = 'none';	// show new time on chat/popup/none

	function loadSettings()
		{
		// parse the raspautotime.xml file here
		$settings = $this->Aseco->xml_parser->parseXML('settings/rasp.xml');
		$settings = $settings['SETTINGS']['AUTOTIME'][0];
		$this->multiplier = $settings['MULTIPLIER'][0];
		$this->authadjust = $settings['AUTHADJUST'][0];
		$this->calcadjust = $settings['CALCADJUST'][0];
		$this->mintime = $settings['MINTIME'][0];
		$this->maxtime = $settings['MAXTIME'][0];
		$this->defaulttime = $settings['DEFAULTTIME'][0] * 1000;
		$this->displaymode = strtolower($settings['DISPLAYMODE'][0]);
		}  //  loadSettings

	function calcAutoTime($authortime, &$automsg)
		{
		if ( $authortime == 0 || !isset($authortime) )
			{
			return $this->defaulttime;
			}

		$newtime = $authortime / 1000;	// convert time from milliseconds to seconds
		$newtime = (($newtime + $this->authadjust) * $this->multiplier) + $this->calcadjust;

		//mintime , maxtime check
		if($newtime < $this->mintime) $newtime = $this->mintime;
		if($newtime > $this->maxtime) $newtime = $this->maxtime;

		$newtime *= 1000;
		$automsg = 'time=' . formatTime($newtime, true);
		if($newtime<=0 || empty($newtime) || !is_numeric($newtime))
			{
			$failtime = $newtime;
			$newtime = $this->defaulttime;
			$automsg = 'calculate time failed (' . $failtime . '), using default time of : ' . formatTime($newtime, true);
			}
		return $newtime;
		}  //  CalcAutoTime

	function onStatusChange2(&$call)
		{
		if ( $this->multiplier > 0.0 )
			{
			// check to see if server is in timeattack mode
			$this->Aseco->client->query('GetNextGameInfo');
			$GameInfo = $this->Aseco->client->getResponse();
			// if we're not in timeattack mode, leave
			if($GameInfo['GameMode'] != 1)		// it is not timeattack, do nothing
				{
				return;
				}

			$failpoint = '';
			$failed = false;
			$this->Aseco->client->query("GetCurrentChallengeIndex");
			$cur_index = $this->Aseco->client->getResponse();
			// do GetChallengeList and send it the current index, this way you avoid looping through data to find the right one later
			$qresp = $this->Aseco->client->query("GetChallengeList", 1, $cur_index);
			$challenge = $this->Aseco->client->getResponse();
			$filename = $challenge[0]['FileName'];
			$qresp = $this->Aseco->client->query("GetChallengeInfo", $filename);
			$result = $this->Aseco->client->getResponse();
			if ( isset($result['faultCode'] ) )
				{
				$this->Aseco->console_text('[AutoTime] - \'' . $filename . '\' - unable to get author time');
				$this->Aseco->console_text('[AutoTime] - GetChallengeInfo failed - Error ' . $result['faultCode'] . '-' . $result['faultString']);
				$failed = true;
				$failpoint = 'GetChallengeInfo';
				}

			if ( $failed )
				{
				$newtime = $this->defaulttime;
				$automsg = $failpoint . ' failed, using default time of: ' . formatTime($newtime, true);
				}
			else
				{
				$newtime = $this->calcAutoTime($result['AuthorTime'], $automsg);
				}

			settype($newtime, 'integer');
			$automsg = '[Autotime] Track=' . stripFormatting($result['Name']) . ', ' . $automsg . ', AuthorTime=' . formatTime($result['AuthorTime']);
			$this->Aseco->console_text($automsg, true);
			$this->Aseco->client->query("SetTimeAttackLimit", $newtime);
			$result = $this->Aseco->client->getResponse();
			if (!$result)
				{
				$this->Aseco->console_text('[' . $this->Aseco->client->getErrorCode() . '] ' . $this->Aseco->client->getErrorMessage());
				}
			else
				{
				if ( $this->displaymode == 'chat' )
					{
					$this->sendServerMessage($automsg);
					// create text to put on chat to players
					}
				elseif ( $this->displaymode == 'popup' )
					{
					//use framework to create a popup window with info
					$this->Aseco->console_text("[AutoTime] Popup option chosen, but not yet available", false);
					}
				}
			}
		}  //  onStatusChange
	}
$_PLUGIN = new RaspAutotime();
$_PLUGIN->setAuthor('Assembler Maniac');
$_PLUGIN->setVersion(1.0);
$_PLUGIN->setDescription('Set Track Time based on AuthorTime');
$_PLUGIN->addEvent('onStartup', 'loadSettings');
$_PLUGIN->addEvent('onStatusChangeTo2', 'onStatusChange2');

/*
 onStatusChangeTo#
	1 = Waiting - occurs when server is stopped, waiting for commands but not actively waiting for clients
	2 = Launching - occurs when map is loading, ideal place to set the time limit for the track
	3 = Running - Synchronizing - occurs when clients are getting synced up
	4 = Running - Play - this happens when the timer starts counting down the to begin the race the very first time of a given track
	5 = Running - Finish - occurs when time limit has been reached OR someone forces a restart or next track (the final scores shows up between here & #2)
*/


?>
