<?php

class ChatCommands extends Plugin
	{
	private $messages;

	function startup()
		{
		// load from an xml file
		$this->clearChatCommands();
		$settings = $this->Aseco->xml_parser->parseXML('settings/rasp.xml');
		$this->messages = $settings['SETTINGS']['CHATMSGS'][0];

		// turn messages into directly usable strings instead of the [0] crap
		foreach ($this->messages as $key => $value)
			{
			unset($this->messages[$key]);
			$text = $value[0]['TEXT'][0];
			$help = $value[0]['HELP'][0];
			unset($value[0]['TEXT']);
			unset($value[0]['HELP']);
			$value[0]['TEXT'] = $text;
			$value[0]['HELP'] = $help;
			$this->messages[$key] = $value[0];
			$this->addChatCommand(strtolower($key), 'chatParse', $help);
			}

		$this->addChatCommand('msg', 'chat_msg', 'Sends a private msg to a specified user login');
		$this->addChatCommand('admin chatcmdreload', 'startup', 'Reload RASP chat command list');

        foreach($this->getChatCommands() as $chat_command)
			{
			$this->Aseco->addChatCommand($chat_command);
			}

		}  //  startup

	function chatParse(&$command)		// 0=server playerid, 1=playerlogin, 2=chat text, 3=is this a cmd, 'author' = player object
		{
		if ( $command[3] == 0 ) return;

		$text = $command[2];
		$pos = strpos($text, ' ');
		if ( !$pos )
			{
			$pos = strlen($text);
			}
		$cmd = strtoupper(substr($text, 1, $pos - 1));
		if ( !isset($this->messages[$cmd]) ) return;
		$parm = trim(substr($text, $pos + 1));
		if ( strlen($parm) == 0 )
			{
			$parm = 'All';
			}

		$text = $this->messages[$cmd]['TEXT'];
		$text = str_replace('{NICK}', $command['author']->nickname, $text);
		$text = str_replace('{PARM}', $parm, $text);
		$text = $this->Aseco->formatColors($text);
		$this->Aseco->client->query('ChatSendServerMessage', $text);
		}  //  chatparse

	function chat_msg(&$command)
		{
		$command['params'] = explode(" ", $command['params'], 2);
		$m = '{#error}-pm-$z['.$command['author']->nickname.'$z] {#interact}'.$command['params'][1];
		$msg = $this->Aseco->formatColors($m);
		$this->Aseco->client->query('ChatSendServerMessageToLogin', $msg, $command['author']->login);
		if ( strtolower($command['params'][0]) == 'admin' )
			{
			foreach($this->Aseco->server->players->player_list as $player_item)
				{
				if($this->Aseco->isAdmin($player_item->login))
					{
					$this->Aseco->client->query('ChatSendServerMessageToLogin', $msg, $player_item->login);
					}
				}
			}
		else
			{
			// 9-9-07 AM - thanks for jaypsi for the idea to check other player name before blindly sending them a message
			$plogin = $command['params'][0];
			$other = $this->Aseco->server->players->getPlayer($plogin);
			if ( !isset($other) )
				{
				$msg = $this->Aseco->formatColors('{#error}Login "{#highlite}' . $plogin .
	                                       '{#error}" is invalid! Type {#interact}/players{#error} for a list.');
				$this->Aseco->client->query('ChatSendServerMessageToLogin', $msg, $command['author']->login);
				}
			else
				{
				$this->Aseco->client->query('ChatSendServerMessageToLogin', $msg, $plogin);
				}
			}
		}
	}

$_PLUGIN = new ChatCommands();
$_PLUGIN->setVersion(1.0);
$_PLUGIN->addChatCommand('msg', 'chat_msg', 'Sends a private msg to a specified user login');
$_PLUGIN->addChatCommand('admin chatcmdreload', 'startup', 'Reload RASP chat command list');
$_PLUGIN->addEvent('onChat', 'chatParse');
$_PLUGIN->addEvent('onStartup', 'startup'); // load messages from xml file
?>
