<?php
/* TMX api info
to query track info from tmx
 http://united.tm-exchange.com/apiget.aspx?action=apitrackinfo&uid=JqS9AkBDcnYv86rZVF0GY7sYMog (or use id=tmxid)
returns following cols, tab separated
TrackId, TrackName, UserId, UserLogin, UploadedAt, UpdatedAt, TrackVisible, PrimaryType, Environment, Mood, Style, Routes, Length, Difficulty, ReplayScore, Version, Comments
309498	Ridge Riders	1522	[ATP]scottmc	2007-05-22 18:01:09	2007-05-22 18:01:09	True	Race	Desert	Sunset	Offroad	Single	45s	Intermediate	37176	TM United	Here's a desert track with some tricky offroad parts, a few cool jumps and lots of speed.  Post your replays, let's see who can finish under a minute and who can get closest to the 50 second mark...	<BR>

to query record info from tmx
 http://united.tm-exchange.com/apiget.aspx?action=apitrackrecords&uid=JqS9AkBDcnYv86rZVF0GY7sYMog (or use id=tmxid)
returns following cols, tab separated
ReplayId, UserId, LoginId, EffectiveTime, ReplayAt, TrackAt, IsApproved, Score, ExpiresSeconds, LockSpan
316561	2485	quinxstar	49760	2007-05-27 13:04:26	2007-05-22 18:01:09	True	37176	2420880	2420880

to search for a track by author or track name (searches for partial match within auth or track)
 http://united.tm-exchange.com/apiget.aspx?action=apisearch&author=scottmc
 http://united.tm-exchange.com/apiget.aspx?action=apisearch&track=street
 http://united.tm-exchange.com/apiget.aspx?action=apisearch&env=island
 TrackID, TrackName, UserId, UserLogin, Race/Stunts/Puzzle, Env, Mood, Style, Routes, Length, Difficulty, Version, Awards, User Comments, World Rec Avail, Game (TM United), Version, ignore rest
 345445	Bay Rumble	1522	[ATP]scottmc	Race	Bay	Day	Race	Symmetric	1m	Intermediate	39203	15	12	True	TM United	354488	0	1		0		2007-06-13 17:29:26	2007-06-17 04:13:02


/*
 * Allow players to add tracks to the 'jukebox' so they can play favorites without waiting
 * Each player can only have one track in jukebox at a time
 */

Class JukeBox extends Plugin
	{
	private $tmxadd, $tmxvote;
	private $jb_buffer, $tmxplaying, $tmxvotes, $tmxplayed, $jukebox;

	// loaded from settings
	private $feature_tmxadd;
	private $tmxvoteratio;
	private $buffersize;
	private $tmxdir;
	private $tmx_env;
	private $tracklistfilename;
	private $sepchar;

	private $debug;

	private $messages;

	function loadSettings()
		{
		$settings = $this->Aseco->xml_parser->parseXML('settings/jukebox.xml');
		$settings = $settings['JUKEBOX_SETTINGS'];

		$this->feature_tmxadd = $settings['ALLOWTMXADD'][0] == '1' ? 1 : 0;
		$this->tmxvoteratio = $settings['TMXVOTERATIO'][0];
		$this->buffersize = $settings['BUFFERSIZE'][0];
		$this->tmxdir = $settings['TMXDIR'][0];
		$this->tmx_env = $settings['TMXENVLIST'][0];
		$this->tmx_env = str_ireplace('desert', 'speed', $this->tmx_env);
		$this->tracklistfilename = $settings['TRACKLISTFILENAME'][0];
		$this->messages = $settings['MESSAGES'][0];
		$this->maxtries = $settings['MAXJBTRIES'][0];
		// turn messages into directly usable strings instead of the [0] crap

		foreach ($this->messages as $key => $value)
			{
			unset($this->messages[$key]);
			$this->messages[$key] = $value[0];
			}

		$this->jb_buffer = array();
		$this->jukebox = array();
		$this->tmxadd = array();
		$this->tmxplaying = false;
		$this->tmxvotes = array();
		$this->tmxvote = false;
		$this->tmxplayed  = false;

		$this->debug = false;
		}  //  loadSettings

	function onSync()
		{
		$this->sepchar = substr($this->Aseco->server->trackdir, -1, 1);
		if ( $this->sepchar == '\\' )
			{
			$this->tmxdir = str_replace('/', $this->sepchar, $this->tmxdir);
			}

		if ( !file_exists($this->Aseco->server->trackdir . $this->tmxdir ) )
			{
			$this->Aseco->console_text('[RASP] Error - TMX Directory (' . $this->Aseco->server->trackdir . $this->tmxdir . ') does not exist');
			$this->feature_tmxadd = false;
			}

		if ( !is_writeable($this->Aseco->server->trackdir . $this->tmxdir ) )
			{
			$this->Aseco->console_text('[RASP] Error - TMX Directory (' . $this->Aseco->server->trackdir . $this->tmxdir . ') can not be written to');
			$this->feature_tmxadd = false;
			}

		}  //  onSync

	function newtrack(&$data)
		{
		$this->tmxvotes = array();
		$this->tmxadd = array();

		if (sizeof($this->jb_buffer) > $this->buffersize)		// drop the first track from stack
			{
			$drop_value = array_shift($this->jb_buffer);
			}
		$this->jb_buffer[] = $data->uid;					// add current track to the stack

		$current = array_shift($this->jukebox);
		if ( $current['uid'] != $data->uid && isset($current['uid']) && $current['tries'] < $this->maxtries )		// track playing isn't next from jukebox, put it back so it's still next in jukebox
			{
			$current['tries']++;
			$this->jukebox = array($current['uid']=>$current) + $this->jukebox;
//			array_unshift($this->jukebox, $current);
			return;
			}

		if ( isset($current) && $current['tries'] >= $this->maxtries )
			{
			$this->Aseco->client->query('RemoveChallenge', $current['FileName']);
			$this->Aseco->client->getResponse();
			$this->Aseco->console_text('[RASP] - automatic removal of track from jukebox & tracklist for failure to play ' . $current['FileName']);
			}

		// 9-9-07 AM - jaypsi fix for tracks left in server list when more than one was added before a track change
		// 9-9-07 AM - (2 or more tracks added via tmx left all but the final one played in the track list
		if ($this->tmxplayed != false && $this->tmxplayed != $this->tmxplaying)
			{
			$this->Aseco->client->query('RemoveChallenge', $this->tmxplayed);
			$this->Aseco->client->getResponse();
			}
		$this->tmxplayed = $this->tmxplaying;
		$this->tmxplaying = false;

/*		if ($this->tmxplaying != false)
			{
			$this->tmxplayed = $this->tmxplaying;
			$this->tmxplaying = false;
			}
		else
			{
			if ($this->tmxplayed != false)
				{
				$this->Aseco->client->query('RemoveChallenge', $this->tmxplayed);
				$this->Aseco->client->getResponse();
				$this->tmxplayed = false;
				}
			} */
		}

	function endrace(&$challenge)
		{
		if (sizeof($this->jukebox) > 0)
			{
			$next = array_shift($this->jukebox);
			$this->jukebox = array($current['uid']=>$current) + $this->jukebox;
			//array_unshift($this->jukebox, $next);
			if ($next['tmx'])
				{
				if ( $this->Aseco->debug )
					{
					$this->Aseco->console_text('{RASP} TMX challenge filename is ' . $next['FileName']);
					}
				$this->Aseco->client->query('AddChallenge', $next['FileName']);
				}
			$this->Aseco->client->getResponse();
			$this->Aseco->client->query('ChooseNextChallenge', $next['FileName']);
			$msg = '{RASP Jukebox} Setting Next Challenge to ' . $next['Name'] . ' $z(' . $next['env'] . ')';
			if ( $next['tmx'] )
				{
				$msg = '{RASP Jukebox} Setting Next Challenge to ' . $next['Name'] . ' $z(' . $next['env'] . '), file downloaded from TMX';
				}
			$this->Aseco->console_text($msg);
			$response = $this->Aseco->client->getResponse();
			$message = formatText($this->messages['NEXTTRACK'], $next['Name'], stripFormatting($next['Nick']), $next['env']);
			$message = $this->Aseco->formatColors($message);
			$this->sendServerMessage($message);
			if($next['tmx'])
				{
				$this->tmxplaying = $next['FileName'];
				}
			}
		}


	function jb(&$command)
		{
		$player = $command['author'];
		$login = $player->login;
		if (is_numeric($command['params']) && $command['params'] >= 0)
			{
			if(count($player->data['tracklist']) == 0)
				{
				$message = $this->Aseco->formatColors($this->messages['USELISTFIRST']);
				$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
				return;
				}
			$jid = ltrim($command['params'], '0');
			$jid--;
			foreach ($this->jukebox as $key)
				{
				if ($player->nickname == $key['Nick'])
					{
					$message = $this->Aseco->formatColors($this->messages['PLAYERTRACKINJB']);
					$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
					return;
					}
				}
			if (array_key_exists($jid, $player->data['tracklist'])) 		// find track by given #
				{
				$uid = $player->data['tracklist'][$jid]['uid'];
				if (array_key_exists($uid, $this->jukebox)) 				// find by uid in jukebox
					{
					$message = $this->Aseco->formatColors($this->messages['TRACKINJB']);
					$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
					return;
					}
				elseif (in_array($uid, $this->jb_buffer) && !$this->Aseco->isAdmin($login))
					{
					$message = $this->Aseco->formatColors($this->messages['TRACKRECENT']);
					$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
					return;
					}

				$this->jukebox[$uid]['FileName'] = $player->data['tracklist'][$jid]['filename'];
				$this->jukebox[$uid]['Name'] = $player->data['tracklist'][$jid]['name'];
				$this->jukebox[$uid]['Nick'] = $player->nickname;
				$this->jukebox[$uid]['tmx'] = false;
				$this->jukebox[$uid]['uid'] = $uid;
				$this->jukebox[$uid]['env'] = $player->data['tracklist'][$jid]['env'];
				$this->jukebox[$uid]['tries'] = 0;
				$message = formatText($this->messages['JUKEBOX'], $player->data['tracklist'][$jid]['name'], stripFormatting($player->nickname), $this->jukebox[$uid]['env']);
				$message = $this->Aseco->formatColors($message);
				$this->Aseco->client->query('ChatSendServerMessage', $message);
				return;
				}
			else
				{
				$message = $this->Aseco->formatColors($this->messages['NOTFOUND']);
				$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
				return;
				}
			}
		elseif ($command['params'] == 'list')
			{
			if (sizeof($this->jukebox) > 0)
				{
				$message = $this->messages['CURTRACKS'];
				$i = 1;
				foreach($this->jukebox as $item)
					{
					$message .= '{#highlite}' . $i . '{#emotic}. [{#highlite}' . stripFormatting($item['Name']) . '{#emotic}], ';
					$i++;
				}
				$message = substr($message, 0, strlen($message)-2);
				$message = $this->Aseco->formatColors($message);
				$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
				return;
				}
			else
				{
				$message = $this->Aseco->formatColors($this->messages['NOTRACKS']);
				$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
				return;
				}
			}
		elseif ($command['params'] == '')
			{
			$message = $this->Aseco->formatColors($this->messages['NOARG']);
			$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
			return;
			}
		}

	function chat_admin_cancel(&$command)
		{
		if ( sizeof($this->tmxadd) == 0 )
			{
			$message = $this->Aseco->formatColors($this->messages['NOTMXVOTECANCEL']);
			$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $command['author']->login);
			return;
			}

		$this->tmxadd = array();
		$this->tmxvotes = array();
		$message = $this->Aseco->formatColors($this->messages['TMXVOTECANCELED']) .' '. $command['author']->nickname;
		$this->Aseco->client->query('ChatSendServerMessage', $message);
		}  //  chat_admin_cancel

	function chat_cancel(&$command)
		{
		if ( sizeof($this->tmxadd) > 0 )
			{
			if ( $command['author']->nickname != $this->tmxadd['nick'] )
				{
				$message = $this->Aseco->formatColors('{#server}>{#emotic} This vote isn\'t yours, you can\'t cancel it.');
				$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $command['author']->login);
				return;
				}
			}

		$this->chat_admin_cancel($command);		// re-use existing function
		}  //  chat_cancel

	function chat_y(&$command)
		{
		if (in_array($command['author']->login, $this->tmxvotes))
			{
			$message = $this->Aseco->formatColors($this->messages['ALREADYVOTED']);
			$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $command['author']->login);
			return;
			}
		if (sizeof($this->tmxadd) > 0 && $this->tmxadd['votes'] > 0)
			{
			$votereq = $this->tmxadd['votes'];
			$votereq--;
			if ($votereq > 0)
				{
				$this->tmxadd['votes'] = $votereq;
				$msg = formatText($this->messages['TMXVOTESNEEDED'], $votereq);
				$message = $this->Aseco->formatColors($msg);
				$this->Aseco->client->query('ChatSendServerMessage', $message);
				$this->tmxvotes[] = $command['author']->login;
				return;
				}
			else
				{
				$uid = $this->tmxadd['uid'];
				$this->jukebox[$uid]['FileName'] = $this->tmxadd['filename'];
				$this->jukebox[$uid]['Name'] = stripFormatting($this->tmxadd['name']);
				$this->jukebox[$uid]['Nick'] = $this->tmxadd['nick'];
				$this->jukebox[$uid]['tmx'] = true;
				$this->jukebox[$uid]['uid'] = $uid;
				$this->jukebox[$uid]['env'] = $this->tmxadd['env'];
				$this->jukebox[$uid]['tries'] = 0;
				$msg = formatText($this->messages['TMXVOTEPASS'], stripFormatting($this->tmxadd['name']) , $this->tmxadd['env']);
				$message = $this->Aseco->formatColors($msg);
				$this->Aseco->client->query('ChatSendServerMessage', $message);
				$this->tmxadd = array();
				$this->tmxvotes = array();
				return;
				}
			}
		else
			{
			$message = $this->Aseco->formatColors($this->messages['NOTMXVOTE']);
			$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $command['author']->login);
			}
		}

	function addtrack(&$command)
		{
		// get packmask here, if it's stadium, default to TMNF
		$this->Aseco->client->query('GetServerPackMask');
		$packmask = strtolower($this->Aseco->client->getResponse());
		$united_server = !( $packmask == 'stadium' | $packmask == 'nations' );
		$player = $command['author'];
		$login = $player->login;
		$tracktypes = array();
		$tracktypes['TMU'] = 'united';
        	$tracktypes['TMNF'] = 'tmnforever';
		$tracktypes['TMN'] = 'nations';
		$tracktypes['SM'] = 'sharemania';
		if ( $united_server ) $tracktypes['TMS'] = 'sunrise';
		if ( $united_server ) $tracktypes['TMO'] = 'original';

		$params = explode(' ', trim($command['params']));
		$tmxid = $params[0];

		$typelist = '';
		foreach($tracktypes as $key=>$value) $typelist .= $key . '/';
		$typelist = substr($typelist, 0, -1);

		// if only one parm, tracktype = TMU, else validate tracktype to be TMU/TMN/TMNF/TMS/TMO, error out if it's not
		// parm 1 should always be numeric (tested further down)

		if ($this->feature_tmxadd == true)
			{
			if (sizeof($this->tmxadd) > 0)
				{
				$message = $this->Aseco->formatColors($this->messages['TMXVOTEINPROGRESS']);
				$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
				return;
				}

// between here & next marker is functionally identical to code in the adminAdd() function
			if (is_numeric($tmxid) && $tmxid >= 0)
				{
				$tmxid = ltrim($tmxid, '0');
				if ( count($params) > 1 )
					{
					$tracktype = strtoupper($params[1]);
					if ( !isset($tracktypes[$tracktype]) )
						{
						$message = formatText($this->messages['TMXBADTYPE'], $tracktype, $typelist);
						$message = $this->Aseco->formatColors($message);
						$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
						return;
						}
					}
				else
					{
					if ( $united_server )
						{
						$tracktype = 'TMU';
						}
					else
						{
						$tracktype = 'TMNF';
						}
					}
					if ($tracktype == 'SM')
				$remotefile = 'http://' . $tracktypes[$tracktype] . '.eu/download.php?id='. $tmxid;
				else
				$remotefile = 'http://' . $tracktypes[$tracktype] . '.tm-exchange.com/get.aspx?action=trackgbx&id=' . $tmxid;
				if (!$stream = fopen($remotefile, 'r'))
					{
					$message = $this->Aseco->formatColors($this->messages['TMXNOTFOUND']);
					$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
					return;
					}
				else
					{
					if (!$file = stream_get_contents($stream))
						{
						$message = formatText($this->messages['TMXFAILREAD'], $localfile);
						$message = $this->Aseco->formatColors($message);
						$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
						return;
						}
					fclose($stream);

					if ( stristr($file, 'GBX') == false )
						{
						unset($file);
						$message = $this->Aseco->formatColors($this->messages['TMXNOTFOUND']);
						$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
						return;
						}

					$partialdir = $this->tmxdir . $this->sepchar . $tracktype . $tmxid . '.challenge.gbx';
					$localfile = $this->Aseco->server->trackdir . $partialdir;
					if ( $this->Aseco->debug )
						{
						$this->Aseco->console_text('/add - tmxdir=' . $this->tmxdir);
						$this->Aseco->console_text('/add - path + filename=' . $partialdir);
						$this->Aseco->console_text('/add - aseco->server->trackdir = ' . $this->Aseco->server->trackdir);
						}
					if (file_exists($localfile))
						{
						if (!unlink($localfile))
							{
							$message = formatText($this->messages['TMXFAILUNLINK'], $localfile);
							$message = $this->Aseco->formatColors($message);
							$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
							return;
							}
						}
					if (!$lfile = fopen($localfile, 'w+'))
						{
						$message = formatText($this->messages['TMXFAILCREATE'], $localfile);
						$message = $this->Aseco->formatColors($message);
						$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
						return;
						}
					if (!fwrite($lfile, $file))
						{
						$message = formatText($this->messages['TMXFAILWRITE'], $localfile);
						$message = $this->Aseco->formatColors($message);
						$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
						return;
						}
					fclose($lfile);
					$newtrk = $this->getChallengeData($localfile, True);		// 2nd parm is whether or not to get players & votes required
					if ( $newtrk['votes'] == 500 && $newtrk['name'] == 'Not a GBX file')
						{
						$message = $this->Aseco->formatColors($this->messages['TMXNOTFOUND']);
						$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
						unlink($localfile);
						return;
						}
// Ending marker, virtually duplicate code

// instead of this trackmode testing, use CheckChallengeForCurrentServerParams with $localfile
/*					$trackmode = $newtrk['trackmode'];
					$servermode = $this->Aseco->server->gameinfo->getMode();
					if ( ( $trackmode == 'Stunts' && $servermode != 'Stunts') ||
						 ( $trackmode != 'Stunts' && $servermode == 'Stunts') )
						{
						$msg = formatText($this->messages['TMXMODEMISMATCH'], $servermode, $trackmode);
						$message = $this->Aseco->formatColors($msg);
						$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
						return;
						}
*/
					$this->Aseco->client->query('CheckChallengeForCurrentServerParams', $localfile);
					$result = $this->Aseco->client->getResponse();
	  				if ( isset($result['faultCode']) )
	  					{
	  					$this->console_text('[' . $result['faultCode'] . '] ' . $result['faultString']);
						$this->Aseco->client->query('ChatSendServerMessageToLogin', 'Server query failed, try again', $login);
	  					return false;
	  					}

					if ( $result != '1' )
						{
						$msg = formatText($this->messages['TMXMODEMISMATCH'], $servermode, $trackmode);
						$message = $this->Aseco->formatColors($msg);
						$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
						return;
						}

					if ( isset($this->tmx_env) && strlen($this->tmx_env) > 0)			// it exists and has data, go check it
						{
						if ( stripos($this->tmx_env, $newtrk['environment']) === false )
							{
							$msg = formatText($this->messages['TMXBADENV'], $newtrk['environment']);
							$message = $this->Aseco->formatColors($msg);
							$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
							unlink($localfile);
							return;
							}
						}
					$ctr = 1;

					$this->getAllChallenges($player, '*', '*', false);		// populate players tracklist with all current tracks
					foreach($player->data['tracklist'] as $key)
						{
						if ( $key['uid'] == $newtrk['uid'] )
							{
							$msg = $this->messages['TMXINLIST'];
							$message = $this->Aseco->formatColors($msg);
							$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
							$command['params'] = $ctr;
							$this->jb($command);
							return;
							}
						$ctr++;
						}
					$this->tmxadd['filename'] = $partialdir;
					$this->tmxadd['votes'] = $newtrk['votes'];
					$this->tmxadd['name'] = stripFormatting($newtrk['name']);
					$this->tmxadd['nick'] = stripFormatting($command['author']->nickname);
					$this->tmxadd['uid'] = $newtrk['uid'];
					$this->tmxadd['env'] = $newtrk['environment'];
					$this->tmxvote = true;
					$msg = formatText($this->messages['TMXADDMSG'], $this->tmxadd['nick'], $this->tmxadd['name'], $this->tmxadd['env'], $this->tmxadd['votes']);
					$message = $this->Aseco->formatColors($msg);
					$this->Aseco->client->query('ChatSendServerMessage', $message);
					}
				}
			else
				{
				$message = $this->Aseco->formatColors($this->messages['TMXNEEDID']);
				$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
				}
			}
		else
			{
			$message = $this->Aseco->formatColors($this->messages['TMXNOADD']);
			$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
			}
		}

	function adminadd(&$command)
		{
		$this->Aseco->client->query('GetServerPackMask');
		$packmask = strtolower($this->Aseco->client->getResponse());
		$united_server = !( $packmask == 'stadium' | $packmask == 'nations' );
		$player = $command['author'];
		$login = $player->login;
		$tracktypes = array();
		$tracktypes['TMU'] = 'united';
		$tracktypes['TMNF'] = 'tmnforever';
		$tracktypes['TMN'] = 'nations';
		$tracktypes['SM'] = 'sharemania';
		if ( $united_server ) $tracktypes['TMS'] = 'sunrise';
		if ( $united_server ) $tracktypes['TMO'] = 'original';

		$params = explode(' ', trim($command['params']));
		$tmxid = $params[0];

		$typelist = '';
		foreach($tracktypes as $key=>$value) $typelist .= $key . '/';
		$typelist = substr($typelist, 0, -1);

		// if only one parm, tracktype = TMU, else validate tracktype to be TMU/TMN/TMNF/TMS/TMO, error out if it's not
		// parm 1 should always be numeric (tested further down)
		if ( $tmxid == 'trackref' )
				{
				$this->build_tmx_trackref($this->Aseco);
				$message = $this->Aseco->formatColors("{#server}>{#emotic} You wrote a new trackref.txt");
				$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
				return;
				}

		if (is_numeric($tmxid) && $tmxid >= 0)
			{
			$tmxid = ltrim($tmxid, '0');
			if ( count($params) > 1 )
				{
				$tracktype = strtoupper($params[1]);
				if ( !isset($tracktypes[$tracktype]) )
					{
					$message = formatText($this->messages['TMXBADTYPE'], $tracktype, $typelist);
					$message = $this->Aseco->formatColors($message);
					$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
					return;
					}
				}
			else
				{
				if ( $united_server )
					{
					$tracktype = 'TMU';
					}
				else
					{
					$tracktype = 'TMNF';
					}
				}
				if ($tracktype == 'SM')
			$remotefile = 'http://' . $tracktypes[$tracktype] . '.eu/download.php?id='. $tmxid;
			else
			$remotefile = 'http://' . $tracktypes[$tracktype] . '.tm-exchange.com/get.aspx?action=trackgbx&id=' . $tmxid;
			if (!$stream = fopen($remotefile, 'r'))
				{
				$message = $this->Aseco->formatColors($this->messages['TMXNOTFOUND']);
				$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
				return;
				}
			else
				{
				if (!$file = stream_get_contents($stream))
					{
					$message = formatText($this->messages['TMXFAILREAD'], $localfile);
					$message = $this->Aseco->formatColors($message);
					$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
					return;
					}
				fclose($stream);

				if ( stristr($file, 'GBX') == false )
					{
					unset($file);
					$message = $this->Aseco->formatColors($this->messages['TMXNOTFOUND']);
					$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
					return;
					}

				$partialdir = $this->tmxdir . $this->sepchar . $tracktype . $tmxid . '.challenge.gbx';
				$localfile = $this->Aseco->server->trackdir . $partialdir;
				if ( $this->Aseco->debug )
					{
					$this->Aseco->console_text('/add - tmxdir=' . $this->tmxdir);
					$this->Aseco->console_text('/add - path + filename=' . $partialdir);
					$this->Aseco->console_text('/add - aseco->server->trackdir = ' . $this->Aseco->server->trackdir);
					}
				if (file_exists($localfile))
					{
					if (!unlink($localfile))
						{
						$message = formatText($this->messages['TMXFAILUNLINK'], $localfile);
						$message = $this->Aseco->formatColors($message);
						$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
						return;
						}
					}
				if (!$lfile = fopen($localfile, 'w+'))
					{
					$message = formatText($this->messages['TMXFAILCREATE'], $localfile);
					$message = $this->Aseco->formatColors($message);
					$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
					return;
					}
				if (!fwrite($lfile, $file))
					{
					$message = formatText($this->messages['TMXFAILWRITE'], $localfile);
					$message = $this->Aseco->formatColors($message);
					$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
					return;
					}
				fclose($lfile);
				$newtrk = $this->getChallengeData($localfile, True);		// 2nd parm is whether or not to get players & votes required
				if ( $newtrk['votes'] == 500 && $newtrk['name'] == 'Not a GBX file')
					{
					$message = $this->Aseco->formatColors($this->messages['TMXNOTFOUND']);
					$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
					unlink($localfile);
					return;
					}

// use CheckChallengeForCurrentServerParams to see if it's allowed to be played
// also check the

				$this->getAllChallenges($player, '*', '*', false);		// populate players tracklist with all current tracks
				$ctr = 1;
				foreach($player->data['tracklist'] as $key)
					{
					if ( $key['uid'] == $newtrk['uid'] )
						{
						if ( strpos($localfile, $key['filename']) != false )		// doesn't exist with same name
							{
							$msg = $this->messages['TMXDIFFNAME'];
							$message = $this->Aseco->formatColors($msg);
							$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
							unlink($localfile);
							}

						// file does already exist in tracklist, queue it up
						$msg = $this->messages['TMXINLIST'];
						$message = $this->Aseco->formatColors($msg);
						$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
						$command['params'] = $ctr;
						$this->jb($command);
						return;
						}
					$ctr++;
					}
				if ( getenv('windir') != '' )
					{
					$partialdir = str_replace('/', '\\', $partialdir);
					}

				$resp = $this->Aseco->client->query('AddChallenge', $partialdir);
				$this->Aseco->client->getResponse();
				$track = $this->Aseco->client->query('GetChallengeInfo', $partialdir);
				$response = $this->Aseco->client->getResponse();
				$msg = $this->messages['TMXADMINADD'] . $response['Name'] . ' $z(' . $response['Environnement'] . ')';
				$this->Aseco->client->query('ChatSendServerMessage', $this->Aseco->formatColors($msg));
				}
			}
		}  //  adminadd

	function build_tmx_trackref()
		{
		$td = $this->Aseco->server->trackdir . $this->tmxdir;

		if ( is_dir($td) )
			{
			$dir = opendir($td);
			$fp = fopen($td . '/trackref.txt', 'w');
			while ( ($file = readdir($dir)) !== false )
				{
				if ( substr($file, -4) == '.gbx' )
					{
					$ci = $this->getChallengeData($td . '/' . $file, false);
					$file = str_replace('.challenge.gbx', '', $file);
					fwrite($fp, $file . "\t" . $ci['tracktype'] . "\t" . $ci['author'] . "\t" . stripFormatting($ci['name']) . "\t" . $ci['coppers'] . CRLF);
					}
				}
			fclose($fp);
			closedir($dir);
			}

		}  //  build_tmx_trackref

	//	Preload jb_buffer by ck|cyrus
	function preload_jb_buffer($data)
		{
		$this->loadSettings();
		//Get Tracklist
		$this->Aseco->client->query("GetChallengeList", 1000, 0);
		$trackresponse = $this->Aseco->client->getResponse();
		//Number of Challenges
		$challengeCount = count($trackresponse);
		if ( $challengeCount == 0 ) return;		// 9-15-07 AM - no challenges? don't bother trying to fill the buffer

		//Get Current Trackposition
		$this->Aseco->client->query("GetCurrentChallengeIndex");
		$currentChallengeIndex = $this->Aseco->client->getResponse();
		$i = $currentChallengeIndex;

		for ($j = 0; $j < $this->buffersize; $j++)
			{
			$this->jb_buffer[] = $trackresponse[$i]['UId'];
			$i--;
			if ( $i < 0 )
				{
				$i = $challengeCount - 1;
				}
			}

		$adminplugin = $this->Aseco->getPlugin('Admin');
		if ( isset($adminplugin) )
			{
			$adminplugin->addButton('Replay Track', 'Put current track in Jukebox as next track', 'replaytrackAP', $this);
			}
		}

	function clearjukebox()
		{
		$this->jukebox = array();
		$message = $this->Aseco->formatColors($this->messages['ADMINCLEAR']);
		$this->Aseco->client->query('ChatSendServerMessage', $message);
		}  //  clearjukebox

	function skipjukebox()
		{
		$dummy = array_shift($this->jukebox);
		$message = $this->Aseco->formatColors($this->messages['ADMINSKIP']);
		$this->Aseco->client->query('ChatSendServerMessage', $message);
		}  //  skipjukebox

	function adminerase(&$command)
		{
		$admin = $command['author'];
		$tracklist = $admin->data['tracklist'];
		if(count($tracklist) == 0)
			{
			$message = $this->Aseco->formatColors($this->messages['USELISTFIRST']);
			$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $admin->login);
			return;
			}
		$tid = $command['params'];
		$tid--;
		if (array_key_exists($tid, $tracklist)) 		// find track by given #
			{
			$filename = $this->Aseco->server->trackdir . $tracklist[$tid]['filename'];
			$this->Aseco->client->query('RemoveChallenge', $filename);
			$this->Aseco->client->getResponse();
			if ( is_file($filename) )
				{
				if ( unlink($filename) )
					{
					$msg = $this->messages['ADMINERASE'] . ' ' . stripFormatting($tracklist[$tid]['name']);
					}
				else
					{
					$msg = $this->Aseco->formatColors('{#server}>{#emotic} Delete file ' . $filename . ' failed');
					$this->Aseco->client->query('ChatSendServerMessageToLogin', $msg, $admin->login);
					$msg = '{#server}>> Admin delete track failed: ' . stripFormatting($tracklist[$tid]['name']);
					}
				}
			$this->Aseco->client->query('ChatSendServerMessage', $this->Aseco->formatColors($msg));
			$this->Aseco->console_text('Admin [{1}] erased track ' . $tracklist[$tid]['name'], $admin->login);
			}
		else
			{
			$message = $this->Aseco->formatColors($this->messages['NOTFOUND']);
			$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $admin->login);
			return;
			}
		}  //  adminerase

	function adminremove(&$command)
		{
		$admin = $command['author'];
		$tracklist = $admin->data['tracklist'];
		if(count($tracklist) == 0)
			{
			$message = $this->Aseco->formatColors($this->messages['USELISTFIRST']);
			$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $admin->login);
			return;
			}
		$tid = $command['params'];
		$tid--;
		if (array_key_exists($tid, $tracklist)) 		// find track by given #
			{
			$filename = $this->Aseco->server->trackdir . $tracklist[$tid]['filename'];
			$this->Aseco->client->query('RemoveChallenge', $filename);
			$this->Aseco->client->getResponse();
			$msg = $this->messages['ADMINREMOVE'] . ' ' . stripFormatting($tracklist[$tid]['name']);
			$this->Aseco->client->query('ChatSendServerMessage', $this->Aseco->formatColors($msg));
			$this->Aseco->console_text('Admin [{1}] removed track ' . $tracklist[$tid]['name'], $admin->login);
			}
		else
			{
			$message = $this->Aseco->formatColors($this->messages['NOTFOUND']);
			$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $admin->login);
			return;
			}
		}  //  adminremove

	function listtracks(&$command)
		{
		$player = $command['author'];
		$login = $player->login;

		// split params into array
		if ( strlen(trim($command['params'])) > 0 )
			{
			$params = explode(' ', trim($command['params']));
			}
		else
			{
			$params = array();
			}

		$cmdcount = count($params);
		$env = "*";

		if ( $cmdcount > 0 )
			{
			foreach($params as $parm)
				{
				if ( substr($parm, 0, 4) == 'env:' )
					{
					$command['params'] = trim(str_replace($parm, '', $command['params']));	// if env: was here, remove it so you can search for stuff later
					$env = substr($parm, 4);			// get the text of the environment
					$cmdcount--;
					}
				}
			}

		if ( $cmdcount == 1 && $params[0] == 'help' )
			{
			$lf = "\n";
			$help = $lf . '  /list will show tracks in rotation on the server' . $lf;
			$help .= '    - nofinish, tracks you haven\'t completed' . $lf;
			$help .= '    - xxx, where xxx is part of a track or author name' . $lf;
			$help .= '    - env:bay/coast/speed/island/rally/snow/stadium to list all tracks of that type' . $lf;
			$help .= '    - karma # #, shows all with karma between entered values, can' . $lf;
			$help .= '      (examples: /list karma 100 95 will show all tracks with karma between 100%' . $lf . '      and 95% in descending order)' . $lf;
			$help .= '      (/list karma -100 -95 will show all tracks with bad karma in descending order)' . $lf;

			$this->sendPopupWindowToPlayer($login, $help, "/list Help");
			return;
			}
		elseif ( $cmdcount == 1 && ($params[0] == "nofinish") )
			{
			$this->getChallengesNoFinish($player);
			}
		elseif ( $cmdcount == 3 && ($params[0] == "karma" ) )
			{
			$res = mysql_query('show columns from challenges');
			while ($row = mysql_fetch_row($res)){ $cols[] = $row[0]; }

			if (in_array('GoodKarma', $cols))
				{
				$karmaval1 = intval($params[1]);
				$karmaval2 = intval($params[2]);
				if ( ($karmaval1 <= 0 && $karmaval2 >= 0) ||
					 ($karmaval1 > 0 && $karmaval2 < 0) )
					{
					$this->Aseco->client->query('ChatSendServerMessageToLogin', 'Mismatched values, both have to be + or -, no mixing', $login);
					return;
					}
				$this->getChallengesByKarma($player, $karmaval1, $karmaval2);
				}
			else
				{
				$this->Aseco->client->query('ChatSendServerMessageToLogin', 'Karma list not available, it\'s not installed.', $login);
				return;
				}
			}
		elseif ( $cmdcount == 0 )
			{
			$wildcard = "*";
			$this->getAllChallenges($player, $wildcard, $env);
			}
		elseif ( $cmdcount >=1 && strlen($params[0]) > 0 )
			{
			$wildcard = $command['params'];
			$this->getAllChallenges($player, $wildcard, $env);
			}

		if ( sizeof($player->data['tracklist']) == 0 )
			{
			$this->Aseco->client->query('ChatSendServerMessageToLogin', "No tracks found, try again.", $login);
			return;
			}
		}

	function getAllChallenges(&$player, $wildcard, $env, $showwindow = true)
		{
		$header = "<line><cell width='0.94'><text halign='center'>- Server Tracks -</text></cell></line>
	<line height='.05'>
	 <cell width='0.06' bgcolor='888E'><text halign='right'>Id</text></cell>
	 <cell width='0.51' bgcolor='888E'><text>  Name</text></cell>
	 <cell width='0.12' bgcolor='888E'><text>  Env</text></cell>
	 <cell width='0.25' bgcolor='888E'><text>Author</text></cell>
	</line>
	";

		$detail = "<line height='0.04'>
		<cell width='0.06'><text action='{name:jbtrack,param:{TRACKNUM}}' halign='right'>{TRACKID}</text></cell>
		<cell width='0.51'><text action='{name:jbtrack,param:{TRACKNUM}}'>  {TRACKNAME}</text></cell>
		<cell width='0.12'><text>{TRACKENV}</text></cell>
		<cell width='0.25'><text>{TRACKAUTHOR}</text></cell>
		</line>
		";

		$footer = "<line height='0.04'><cell width='0.94'></cell></line><line height='0.05'><cell bgcolor='FFF5' width='0.94'><text halign='center'>Click on track name to add to JukeBox</text></cell></line>" . CRLF;
		$newlist = array();
		$done = false;
		$i = 0;
		while (!$done)
			{
			$this->Aseco->client->query('GetChallengeList', 300, $i);
			$tlist = array();
			$response = $this->Aseco->client->getResponse();
			if (sizeof($response) > 0)
				{
				if ( isset($response['faultString'] ) )
					{
					$done = true;
					$this->Aseco->client->error->code = 0;
					$this->Aseco->client->error->message = '';
					break;
					}
				foreach ($response as $trow)
					{
					$newlist[] = $trow;
					}
				if ( sizeof($response) < 300 )
					{
					$done = true;
					break;
					}
				else
					{
					$i = $i + 300;
					}
				}
			else
				{
				$done = true;
				break;
				}
			}

		$player->data['tracklist'] = array();
		$content = array();

		$s = '';
		$tid = 1;
		$ctr = 0;
		$msgs = 0;

		foreach ($newlist as $row)
			{
			$trackname = stripFormatting($row['Name']);
			$trackauthor = $row['Author'];
			$trackenv = $row['Environnement'];
			if ( $wildcard == '*' )
				{
				$pos = 0;
				}
			else
				{
				$pos = stripos($trackname, $wildcard);
				if ( $pos === false )
					{
					$pos = stripos($trackauthor, $wildcard);
					}
				}

			// env is a additive compare always, so /list xxx env:bay will give all bay tracks with xxx in track or author
			if ($env != '*' &&
				($wildcard == '*' || ( $wildcard != '*' && !($pos === false))))
				{
				$pos = stripos($trackenv, $env);
				}

			if (!($pos === false))
				{
				$trkarr = array();
				$stgout = $detail;
				$stgout = str_replace('{TRACKNUM}', $tid, $stgout);
				$stgout = str_replace('{TRACKID}', $tid . '. ', $stgout);
				$stgout = str_replace('{TRACKNAME}', htmlspecialchars($trackname) . '  ', $stgout);
				$stgout = str_replace('{TRACKAUTHOR}', htmlspecialchars($trackauthor), $stgout);
				$stgout = str_replace('{TRACKENV}', $trackenv, $stgout);
				$s .= $stgout;
				$tid++;
				$ctr++;
				if ( $ctr == 20 && $showwindow )
					{
					$ctr = 0;
					$s = $header . $s . $footer;
					$content[] = $s;
					$s = '';
					}
				$trkarr['name'] = $trackname;
				$trkarr['filename'] = $row['FileName'];
				$trkarr['uid'] = $row['UId'];
				$trkarr['env'] = $trackenv;
				$player->data['tracklist'][] = $trkarr;
				}
			}

		if ( $s != '' && $showwindow )		// add if last batch exists
			{
			$s = $header . $s . $footer;
			$content[] = $s;
			}

		if ( sizeof($content)>0 && $showwindow )
			{
    		$window = $this->createWindowToPlayerFramework('Jukebox', 0.94, 1.15, $player);
			$window->setContent($content);
//			$window->setWidth(0.94);
			$window->addCallback('jbtrack', 'jbtrack');
			}
		}

	function jbtrack($clickvalue, $window, $player)
		{
		$cmd['params'] = '' . $clickvalue;
		$cmd['author'] = $player;
		$window->close();
		$this->jb($cmd);
		}  //  jbtrack

	function getChallengesByKarma(&$player, $karmaval1, $karmaval2)
		{
		$newlist = array();
		$done = false;
		$i = 0;
		$header = "<line><cell width='1.05'><text halign='center'>- Server Tracks by Karma -</text></cell></line>
		<line height='.04'>
		<cell width='0.06' bgcolor='888E'><text halign='right'>Id</text></cell>
		<cell width='0.52' bgcolor='888E'><text>  Name</text></cell>
		<cell width='0.25' bgcolor='888E'><text>Author</text></cell>
		<cell width='0.11' bgcolor='888E'><text halign='center'>Karma +</text></cell>
		<cell width='0.11' bgcolor='888E'><text halign='center'>Karma -</text></cell>
		</line>
		";

		$detail = "<line><cell width='0.06'><text action='{name:jbtrack,param:{TRACKNUM}}' halign='right'>{TRACKID}</text></cell>
				<cell width='0.52'><text action='{name:jbtrack,param:{TRACKNUM}}'>  {TRACKNAME}</text></cell>
				<cell width='0.25'><text>{TRACKAUTHOR}</text></cell>
				<cell width='0.11'><text halign='right'>{TRACKKARMAG} </text></cell>
				<cell width='0.11'><text halign='right'>{TRACKKARMAB} </text></cell></line>" . CRLF;

		$footer = "<line height='0.02'><cell width='1.05'></cell></line><line height='0.05'><cell bgcolor='FFF5' width='1.05'><text halign='center'>Click on track name to add to JukeBox</text></cell></line><line height='0.02'><cell width='1.05'></cell></line>";

		$content = "";

/*
may not be needed, clause "having" might work fine
alter table challenges add GoodKarma mediumint null default 0, add BadKarma mediumint null default 0;
update challenges set goodkarma = (select sum(score) from rs_karma where rs_karma.challengeid=challenges.id and rs_karma.score > 0);
update challenges set badkarma = (select sum(score) from rs_karma where rs_karma.challengeid=challenges.id and rs_karma.score < 0);
update challenges set goodkarma = 0 where goodkarma is null;
update challenges set badkarma = 0 where badkarma is null;


		$sql = 'select name, uid, (SELECT sum(score) FROM rs_karma WHERE ChallengeId=k.ChallengeId AND score > 0) as goodkarma,
			(SELECT sum(score) FROM rs_karma WHERE ChallengeId=k.ChallengeId AND score < 0) as badkarma
			from challenges c, rs_karma k
			where c.id=k.challengeid group by uid';
*/
		$sql = 'select uid, name,
		goodkarma + abs(badkarma) as totkarma,
		100 / (goodkarma + abs(badkarma)) * goodkarma as `gk`,
		100 / (goodkarma + abs(badkarma)) * badkarma as `bk`
		from challenges having gk is not null and bk is not null ';

		if ( $karmaval1 > $karmaval2 )
			{
			$t = $karmaval2;
			$karmaval2 = $karmaval1;
			$karmaval1 = $t;
			}

		if ( $karmaval1 > 0 )
			{
			$sql .= 'and gk between ' . $karmaval1 . ' and ' . $karmaval2;
			$sql .= ' order by totkarma, gk desc';
			}
		else
			{
			$sql .= 'and bk between ' . $karmaval1 . ' and ' . $karmaval2;
			$sql .= ' order by totkarma, bk asc';
			}

		$sql .= ', challenges.name';

		$result = mysql_query($sql);

		$player->data['tracklist'] = array();
		if ( mysql_num_rows($result) == 0 )
			{
			mysql_free_result($result);
			return;
			}

		while (!$done)
			{
			$this->Aseco->client->query('GetChallengeList', 300, $i);
			$tlist = array();
			$response = $this->Aseco->client->getResponse();
			if (sizeof($response) > 0)
				{
				if ( isset($response['faultString'] ) )
					{
					$done = true;
					$this->Aseco->client->error->code = 0;
					$this->Aseco->client->error->message = '';
					break;
					}
				foreach ($response as $trow)
					{
					$newlist[$trow['UId']] = $trow;
					}
				if ( sizeof($response) < 300 )
					{
					$done = true;
					break;
					}
				else
					{
					$i = $i + 300;
					}
				}
			else
				{
				$done = true;
				break;
				}
			}

		$content = array();
		$s = '';
		$tid = 1;
		$ctr = 0;
		while($dbrow = mysql_fetch_array($result))
			{
			if ( array_key_exists($dbrow[0], $newlist) )		// does the uid exist in the current server track list?
				{
				$row = $newlist[$dbrow[0]];		// indexed by uid
				$trkarr = array();
				$trackname = stripFormatting($row['Name']);
				$stgout = $detail;
				$stgout = str_replace('{TRACKNUM}', $tid, $stgout);
				$stgout = str_replace('{TRACKID}',  $tid . '. ', $stgout);
				$stgout = str_replace('{TRACKNAME}', htmlspecialchars($trackname), $stgout);
				$stgout = str_replace('{TRACKAUTHOR}', htmlspecialchars($row['Author']), $stgout);
				$stgout = str_replace('{TRACKKARMAG}', sprintf('%.02f', $dbrow[3]), $stgout);
				$stgout = str_replace('{TRACKKARMAB}', sprintf('%.02f', $dbrow[4]), $stgout);
				$s .= $stgout;
				$tid++;
				$ctr++;
				if ( $ctr == 20 )
					{
					$ctr = 0;
					$s = $header . $s . $footer;
					$content[] = $s;
					$s = '';
					}
				$trkarr['name'] = $trackname;
				$trkarr['filename'] = $row['FileName'];
				$trkarr['uid'] = $row['UId'];
				$trkarr['env'] = $row['Environnement'];
				$player->data['tracklist'][] = $trkarr;
				}
			}
		if ( $s > '' )		// add if last batch exists
			{
			$s = $header . $s . $footer;
			$content[] = $s;
			}

		mysql_free_result($result);

		if ( sizeof($content)>0 )
			{
    		$window = $this->createWindowToPlayerFramework('Track List by Karma', 1.05, 1.15, $player);
			$window->setContent($content);
//			$window->setWidth(1.05);
			$window->addCallback('jbtrack', 'jbtrack');
			}
		}	// end admin_getChallengesByKarma($karmaval)

	function getChallengesNoFinish(&$player) {
		$newlist = array();
		$done = false;
		$i = 0;
		$sql = 'select uid from challenges where id not in (select distinct challengeID from records,players
			where records.playerID=players.id AND
				players.login=' . quotedString($player->login) . ' AND game=' . quotedString($this->Aseco->server->getGame()) .
				') order by `name`';
		$header = "<line>
	  <cell width='0.94'><text halign='center'>- Server Tracks not Finished -</text></cell></line>
	<line height='.04'>
	<cell width='0.06' bgcolor='888E'><text halign='right'>Id</text></cell>
	<cell width='0.51' bgcolor='888E'><text>  Name</text></cell>
	<cell width='0.12' bgcolor='888E'><text>  Env</text></cell>
	<cell width='0.25' bgcolor='888E'><text>Author</text></cell></line>";

		$detail = "<line>
		<cell width='0.06'><text action='{name:jbtrack,param:{TRACKNUM}}' halign='right'>{TRACKID}</text></cell>
		<cell width='0.51'><text action='{name:jbtrack,param:{TRACKNUM}}'>  {TRACKNAME}</text></cell>
		<cell width='0.12'><text>{TRACKENV}</text></cell>
		<cell width='0.25'><text>{TRACKAUTHOR}</text></cell></line>\r\n";

		$footer = "<line height='0.02'><cell width='0.94'></cell></line><line height='0.05'><cell bgcolor='FFF5' width='0.94'><text halign='center'>Click on track name to add to JukeBox</text></cell></line><line height='0.02'><cell width='0.94'></cell></line>";

		$result = mysql_query($sql);

		if ( mysql_num_rows($result) == 0 )
			{
			mysql_free_result($result);
			return;
			}

		while (!$done)
			{
			$this->Aseco->client->query('GetChallengeList', 300, $i);
			$tlist = array();
			$response = $this->Aseco->client->getResponse();
			if (sizeof($response) > 0)
				{
				if ( isset($response['faultString'] ) )
					{
					$done = true;
					$this->Aseco->client->error->code = 0;
					$this->Aseco->client->error->message = '';
					break;
					}
				foreach ($response as $trow)
					{
					$newlist[$trow['UId']] = $trow;
					}
				if ( sizeof($response) < 300 )
					{
					$done = true;
					break;
					}
				else
					{
					$i = $i + 300;
					}
				}
			else
				{
				$done = true;
				break;
				}
			}

		$player->data['tracklist'] = array();
		$content = array();

		$s = '';
		$tid = 1;
		$ctr = 0;
		$msgs = 0;
		while($dbrow = mysql_fetch_array($result))
			{
			if ( array_key_exists($dbrow[0], $newlist) )		// does the uid exist in the current server track list?
				{
				$row = $newlist[$dbrow[0]];		// indexed by uid
				$trkarr = array();
				$stgout = $detail;
				$trackname = stripFormatting($row['Name']);
				$trackenv = $row['Environnement'];
				$stgout = str_replace('{TRACKNUM}', $tid, $stgout);
				$stgout = str_replace('{TRACKID}', $tid . '. ', $stgout);
				$stgout = str_replace('{TRACKNAME}', htmlspecialchars($trackname) . '  ', $stgout);
				$stgout = str_replace('{TRACKAUTHOR}', htmlspecialchars($row['Author']), $stgout);
				$stgout = str_replace('{TRACKENV}', $trackenv, $stgout);
				$s .= $stgout;
				$tid++;
				$ctr++;
				if ( $ctr == 20 )
					{
					$ctr = 0;
					$msgs++;
					$s = $header . $s . $footer;
					$content[] = $s;
					$s = '';
					}
				$trkarr['name'] = $trackname;
				$trkarr['filename'] = $row['FileName'];
				$trkarr['uid'] = $row['UId'];
				$trkarr['env'] = $trackenv;
				$player->data['tracklist'][] = $trkarr;
				}
			}
		if ( $s > '' )		// add if last batch exists
			{
			$s = $header . $s . $footer;
			$content[] = $s;
			}

		if ( sizeof($content)>0 )
			{
    		$window = $this->createWindowToPlayerFramework('Track List by Karma', 1.05, 1.15, $player);
			$window->setContent($content);
//			$window->setWidth(1.05);
			$window->addCallback('jbtrack', 'jbtrack');
			}

		mysql_free_result($result);
		}	// end admin_getChallengesByKarma($karmaval)

	/*
	 all strings are length followed by data of exact length (no trailing null)

	 0x4A long coppers
	 0x67 string id
	 4 unknown bytes
	 string environment
	 4 unknown bytes
	 string author
	 string trackname


	 */

	function GetGBXString($handle)
		{
		$data = fread($handle, 4);
		$result = unpack('Vlen', $data);
		if ( ($result['len'] > 256) || ($result['len'] < 1))
			{
			return 'read error';
			}
		$data = fread($handle, $result['len']);
		return $data;
		}  //  GetGBXString

	function getChallengeData($filename, $rtnvotes)
		{
		// This function assumes that TM Server and ASECO are running on the same machine and will NOT work
		// if they are on different machines.
		$ret = array();
		if ( !file_exists($filename) )
			{
			$ret['name'] = 'file not found';
			$ret['votes'] = 500;
			return $ret;
			}
		if ( $rtnvotes )
			{
			$players = $this->Aseco->client->query('GetPlayerList', 50,0);
			if (!$players)
				{
				$this->Aseco->console_text('[' . $this->Aseco->client->getErrorCode() . '] ' . $this->Aseco->client->getErrorMessage());
				$response = array();
				$ret['name'] = 'failed to get players';
				$ret['votes'] = 500;
				return $ret;
				}
			else
				{
				$response = $this->Aseco->client->getResponse();
				$nbplrs = sizeof($response);
				$ret['votes'] = floor($nbplrs * $this->tmxvoteratio);
				if ($ret['votes'] < 1)
					{
					$ret['votes']++;
					}
				if ( $this->Aseco->debug )
					{
					$ret['votes'] = 1;
					}
				}
			}

		$handle = fopen($filename, 'rb');
		fseek($handle, 0x00, SEEK_SET);
		$data = fread($handle, 3);
		if ( $data == 'GBX' )
			{
			fseek($handle, 0x11, SEEK_SET);
			$data = fread($handle, 4);
			$r = unpack('Vblocklens', $data);
			$numblocks = $r['blocklens'];		// # of blocks to read, with lengths we need later (only 2 lengths)
			$data = fread($handle, 8 * $numblocks);
			$s = '';
			for ($i = 1; $i <= $numblocks; $i++)
				{
				$s .= 'Vjunk' . $i . '/Vlen' . $i;
				if ( $i < $numblocks )
					{
					$s .= '/';
					}
				}
			$r = unpack($s, $data);

			$timeslen = $r['len1'];			// this is needed to read & use next block of data
			$data = fread($handle, 1);
			$r = unpack('Cnumtimes', $data);
			$numtimes = $r['numtimes'];
			$data = fread($handle, $timeslen-1);
			$s = '';
			for ($i = 1; $i <= $numtimes; $i++)
				{
				$s .= 'Vtime' . $i;
				if ( $i < $numtimes )
					{
					$s .= '/';
					}
				}
			$r = unpack($s, $data);
			$ret['timebronze'] = $r['time2'];
			$ret['timesilver'] = $r['time3'];
			$ret['timegold'] = $r['time4'];
			$ret['timeauthor'] = $r['time5'];
			$ret['coppers'] = $r['time6'];
			$ret['multilap'] = $r['time7'] > 0;		// 08-1-13 AssemblerManiac - added to the resulting fields
			switch($r['time8'])
				{
				case 0:
					$ret['trackmode'] = 'Race';
					break;

				case 1:
					$ret['trackmode'] = 'Platform';
					break;

				case 2:
					$ret['trackmode'] = 'Puzzle';
					break;

				case 5:
					$ret['trackmode'] = 'Stunts';
					break;
				}

			if ( $numtimes > 9 )
				{
				$ret['authorscore'] = $r['time10'];
				}

			fseek($handle, 0x09, SEEK_CUR);		// skip block of 9 unknowns

			$ret['uid'] = $this->getGbxString($handle);
			fseek($handle, 0x04, SEEK_CUR);
			$ret['environment'] = $this->getGbxString($handle);
			fseek($handle, 0x04, SEEK_CUR);
			$ret['author'] = $this->getGbxString($handle);
			$ret['name'] = $this->getGbxString($handle);

			// 08-1-13 AssemblerManiac - added Mood to the resulting fields
			fseek($handle, 0x05, SEEK_CUR);		// more unknowns
			$junk = $this->getGbxString($handle);		// password
			fseek($handle, 0x04, SEEK_CUR);		// more unknowns
			$ret['mood'] = $this->getGbxString($handle);
			}
		else
			{
			$ret['votes'] = 500;
			$ret['name'] = 'Not a GBX file';
			}

		fclose($handle);
		return $ret;
		}  //  GetChallengeData($filename)

	function writetracklist(&$command)
		{
		if ( isset($command['params'][1]) )
			{
			$filename = $command['params'][1];
			}
		elseif ( isset($this->tracklistfilename) && strlen($this->tracklistfilename) > 0 )
			{
			$filename = $this->tracklistfilename;
			}
		else
			{
			$filename = 'rasp-tracklist.txt';
			}

		$localfile = $this->Aseco->server->trackdir . $this->sepchar . 'MatchSettings' . $this->sepchar . $filename;

		if ( file_exists($localfile) )
			{
			$histdir = $this->Aseco->server->trackdir . $this->sepchar . 'MatchSettings' . $this->sepchar . 'History';
			if ( !file_exists($histdir) ) mkdir($histdir);
			$histdir .= $this->sepchar . date('Y-m-d His') . '.txt';
			rename($localfile, $histdir);
			}

		$this->Aseco->client->query('SaveMatchSettings', 'MatchSettings' . $this->sepchar . $filename);
		$this->Aseco->client->getResponse();
		$msg = '{#server}>> ' . $filename . ' written';
		$this->Aseco->client->query('ChatSendServerMessageToLogin', $this->Aseco->formatColors($msg), $command['author']->login);
		}  //  writetracklist

	function chat_nextmap(&$command)
		{
		if (sizeof($this->jukebox) > 0)
			{
			$jbtemp = $this->jukebox;
			$nextmap = array_shift($jbtemp);
			$message = '{#server}> {#interact}The next map is {#highlite}' . stripFormatting($nextmap['Name']) . ' (' . $nextmap['env'] . ')';
			$message = $this->Aseco->formatColors($message);
			$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $command['author']->login);
			}
		else
			{
			$this->Aseco->client->query("GetCurrentChallengeIndex");
			$cur_index = $this->Aseco->client->getResponse();
			$cur_index++;
			// do GetChallengeList and send it the current index, this way you avoid looping through data to find the right one later
			$qresp = $this->Aseco->client->query("GetChallengeList", 1, $cur_index);
			$challenge = $this->Aseco->client->getResponse();
			if ( isset($challenge['faultString'] ) )
				{
				$qresp = $this->Aseco->client->query("GetChallengeList", 1, 0);
				$challenge = $this->Aseco->client->getResponse();
				$this->Aseco->client->error->code = 0;
				$this->Aseco->client->error->message = '';
				}

			$message = '{#server}> {#welcome}The next map is {#highlite}' . stripFormatting($challenge[0]['Name']) . ' (' . $challenge[0]['Environnement'] . ')';
			$message = $this->Aseco->formatColors($message);
			$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $command['author']->login);
			}
		}

	function isTrackInJB()
		{
		return sizeof($this->jukebox) > 0;
		}  //  isTrackInJB

/* alternate function version - uses direct encoding call, but the normal one below this works ok for utf-8 too
	function getTMXSearchResults($pagenum, &$tracktype, &$search, &$env, &$author )
		{
		$data = 'GET /apiget.aspx?action=apisearch&page=' . $pagenum . '&track=' . $search . $env . $author . ' HTTP/1.1' . CRLF;
		$data .= 'Host: ' . $tracktype . '.tm-exchange.com' . CRLF;
		$data .= 'User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.8.1.12) Gecko/20080201 Firefox/2.0.0.12' . CRLF;
		$data .= 'Accept: text/xml,application/xml,text/html;q=0.9,text/plain;q=0.8' . CRLF;
		$data .= 'Accept-Charset: utf-8;q=0.7,*;q=0.7' . CRLF;
		$data .= 'Keep-Alive: 300' . CRLF;
		$data .= 'Connection: keep-alive' . CRLF . CRLF;

		$sock = fsockopen($tracktype . '.tm-exchange.com', 80, $errno, $errstg, 30);
		if ( !$sock )
			{
			return false;
			}

		fwrite($sock, $data);

		$headers = "";
		while ($str = trim(fgets($sock, 4096)))
  			$headers .= "$str\n";

		$i = strpos($headers, 'Content-Length:');
		$s = substr($headers, $i + 16);
		$s = str_replace(CRLF, '', $s);

		settype($s, 'integer');
		$body = fread($sock, $s);
		fclose($sock);

		return $body;

		}  //  getTMXSearchResults
*/

	function getTMXSearchResults($pagenum, &$tracktype, &$search, &$env, &$author )
		{
		$remotefile = 'http://' . $tracktype . '.tm-exchange.com/apiget.aspx?action=apisearch&page=' . $pagenum . '&track=' . $search . $env . $author;
		if (!$stream = fopen($remotefile, 'r'))
			{
			$message = $this->Aseco->formatColors($this->messages['TMXNOTFOUND']);
			$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
			return false;
			}
		else
			{
			$results = stream_get_contents($stream);
			fclose($stream);
			}

		return $results;

		}  //  getTMXSearchResults


	function tmxSearch(&$command)
		{
 		//http://united.tm-exchange.com/apiget.aspx?action=apisearch&author=scottmc
		// /tlist TMU/TMN/TMNF/TMO/TMS "string" (both author and track) /env:whatever
		// get packmask here, if it's stadium, default to TMNF
		$this->Aseco->client->query('GetServerPackMask');
		$packmask = strtolower($this->Aseco->client->getResponse());
		$united_server = !( $packmask == 'stadium' | $packmask == 'nations' );
		$player = $command['author'];
		$login = $player->login;
		$tracktypes = array();
		$tracktypes['TMU'] = 'united';
		$tracktypes['TMNF'] = 'tmnforever';
		$tracktypes['TMN'] = 'nations';
		if ( $united_server ) $tracktypes['TMS'] = 'sunrise';
		if ( $united_server ) $tracktypes['TMO'] = 'original';

		$params = explode(' ', trim($command['params']));
		$pcount = count($params);

		$typelist = '';
		foreach($tracktypes as $key=>$value) $typelist .= $key . '/';
		$typelist = substr($typelist, 0, -1);

		// if only one parm, tracktype = TMU, else validate tracktype to be TMO/TMS/TMN, error out if it's not
		// parm 1 should always be numeric (tested further down)

		if ( $pcount == 1 && $params[0] == 'help' )
			{
			$lf = "\n";
			$help = $lf . '/tlist will show tracks from TM-Exchange' . $lf;
			$help .= ' Calling format is /tlist xxx [auth:yyy] [env:zzz] [' . $typelist . ']' . $lf;
			$help .= '  - xxx, where xxx is part of a track or author name (this is required)' . $lf;
			$help .= '  - author:yyy where yyy is part of the authors name' . $lf;
			$help .= '  - env:zzz where yyy is bay/coast/desert/island/rally/snow/stadium' . $lf;
			$help .= '  - ' . $typelist . ' indicates which part of TMX to search' . $lf;

			$this->sendPopupWindowToPlayer($login, $help, "/tlist Help");
			return;
			}
		else
			{
			foreach($params as $key=>$parm)
				{
				if ( strtolower(substr($parm, 0, 4)) == 'env:' )
					{
					$env = '&env=' . substr($parm, 4);
					unset($params[$key]);
					$pcount--;
					if ( $env == '&env=alpine' )
						{
						$env = '&env=snow';
						}
					if ( $env == '&env=speed' )
						{
						$env = '&env=desert';
						}
					}

				if ( strtolower(substr($parm, 0, 7)) == 'author:' )
					{
					$author = '&author=' . substr($parm, 7);
					unset($params[$key]);
					$pcount--;
					}
				}

			if ( $pcount > 1 )
				{
				$tracktype = strtoupper(array_pop($params));			// tmo/tms/tmn/tmu/tmnf will always be last param
				if ( !isset($tracktypes[$tracktype]) )
					{
					$message = formatText($this->messages['TMXBADTYPE'], $tracktype, $typelist);
					$message = $this->Aseco->formatColors($message);
					$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
					return;
					}
				}
			else
				{
				if ( $united_server )
					{
					$tracktype = 'TMU';
					}
				else
					{
					$tracktype = 'TMNF';
					}
				}

			$search = $params[0];

			for ($i = 0; $i < 5; $i++)
				{
				$s = $this->getTMXSearchResults($i, $tracktypes[$tracktype], $search, $env, $author );
				if ( strpos($s, 'Requested value') != false && strpos($s, 'was not found') != false)
					{
					$message = $this->messages['TMXLISTFAIL'];
					$message = $this->Aseco->formatColors($message);
					$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
					return;
					}

				if ( !$s )
					{
					break;
					}
				$results .= $s;
				}

			// we now have a result block, break it down into something usable

			$tracks = explode("<BR>\r\n", $results);			// array of entries, each line is one track w/tab delimited fields
			unset($tracks[count($tracks) - 1]);

			if ( count($tracks) == 0 )
				{
				$message = $this->messages['TMXLISTFAIL'];
				$message = $this->Aseco->formatColors($message);
				$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);
				return;
				}

//http://united.tm-exchange.com/apiget.aspx?action=apisearch&track=fof&env=alpine&author=spacejocky

			$width = 1;
			$isAdmin = $this->Aseco->isAdmin($login);
			if ( $isAdmin )
				{
				$width = 1.1;
				}

			$header = "<line><cell width='$width'><text halign='center'>- TMX Tracks -</text></cell></line>
						<line height='.05'>
				 		<cell width='0.12' bgcolor='888E'><text halign='right'>TMX Id</text></cell>
						<cell width='0.51' bgcolor='888E'><text>  Name</text></cell>
						<cell width='0.12' bgcolor='888E'><text>  Env</text></cell>
						<cell width='0.25' bgcolor='888E'><text>Author</text></cell>";
			if ( $isAdmin )
				{
				$header .= "<cell width='0.1' bgcolor='888E'><text halign='center'>Admin</text></cell>";
				}
			$header .= '</line>';

			$detail = "<line height='0.04'>
					<cell width='0.12'><text action='{name:tmxSearchAdd,param:{TRACKNUM}}' halign='right'>{TRACKID}</text></cell>
					<cell width='0.51'><text action='{name:tmxSearchAdd,param:{TRACKNUM}}'>  {TRACKNAME}</text></cell>
					<cell width='0.12'><text>{TRACKENV}</text></cell>
					<cell width='0.25'><text>{TRACKAUTHOR}</text></cell>";
			if ( $isAdmin )
				{
				$detail .= "<cell width='0.1'><text halign='center' action='{name:tmxSearchAddAdmin,param:{TRACKNUM}}'>Add</text></cell>";
				}
			$detail .= '</line>';

			$footer = "<line height='0.04'><cell width='$width'></cell></line><line height='0.05'><cell bgcolor='FFF5' width='$width'><text halign='center'>Click on track name to add to JukeBox</text></cell></line>" . CRLF;
			$newlist = array();
			$done = false;
			$i = 0;

			$player->data['tracklist'] = array();
			$player->data['tmxlist'] = array();
			$content = array();

			$s = '';
			$tid = 0;
			$ctr = 0;
			$msgs = 0;

			$mode = $this->Aseco->server->gameinfo->getMode();
			if ( $mode != 'Stunts' )
				{
				$mode = 'Race';
				}

			foreach ($tracks as $row)
				{
	// TrackID, TrackName, UserId, UserLogin, Race/Stunt/Puzzle, Env, Mood, Style, Routes, Length, Difficulty, Version, Awards, User Comments, World Rec Avail, Game (TM United), Version, ignore rest
	// 345445	Bay Rumble	1522	[ATP]scottmc	Race	Bay	Day	Race	Symmetric	1m	Intermediate	39203	15	12	True	TM United	354488	0	1		0		2007-06-13 17:29:26	2007-06-17 04:13:02
				$row = explode("\t", $row);

				if ( $row[4] != $mode )
					{
					continue;
					}

				$trackname = stripFormatting($row[1]);
				$trackauthor = $row[3];
				$trackenv = $row[5];

				// env is a additive compare always, so /list xxx env:bay will give all bay tracks with xxx in track or author

				$trkarr = array();
				$stgout = $detail;
				$stgout = str_replace('{TRACKNUM}', $tid, $stgout);
				$stgout = str_replace('{TRACKID}', $row[0], $stgout);
				$stgout = str_replace('{TRACKNAME}', htmlspecialchars($trackname) . '  ', $stgout);
				$stgout = str_replace('{TRACKAUTHOR}', htmlspecialchars($trackauthor), $stgout);
				$stgout = str_replace('{TRACKENV}', $trackenv, $stgout);

				$s .= $stgout;
				$tid++;
				$ctr++;
				if ( $ctr == 20 )
					{
					$ctr = 0;
					$s = $header . $s . $footer;
					$content[] = $s;
					$s = '';
					}
				$trkarr['tmxid'] = $row[0];
				$trkarr['tracktype'] = $tracktype;
				$trkarr['trackname'] = $trackname;
				$player->data['tmxlist'][] = $trkarr;
				}

			if ( $s != '' )		// add if last batch exists
				{
				$s = $header . $s . $footer;
				$content[] = $s;
				}

			if ( sizeof($content)>0 )
				{
    			$window = $this->createWindowToPlayerFramework('TMX Search', $width, 1.15, $player);
				$window->setContent($content);
//				$window->setWidth($width);
				$window->addCallback('tmxSearchAdd', 'tmxSearchAdd');
				$window->addCallback('tmxSearchAddAdmin', 'tmxSearchAddAdmin');
				}

			}  //  tmxSearch
		}

	function tmxSearchAdd($clickvalue, $window, $player)
		{
		$cmd['params'] = '' . $player->data['tmxlist'][$clickvalue]['tmxid'] . ' ' . $player->data['tmxlist'][$clickvalue]['tracktype'];
		$cmd['author'] = $player;
		$window->close();
		$this->addtrack($cmd);
		}  //  tmxSearchAdd

	function tmxSearchAddAdmin($clickvalue, $window, $player)
		{
		$cmd['params'] = '' . $player->data['tmxlist'][$clickvalue]['tmxid'] . ' ' . $player->data['tmxlist'][$clickvalue]['tracktype'];
		$cmd['author'] = $player;
		$window->close();
		$this->adminadd($cmd);
		}  //  tmxSearchAdd

	function replaytrackAP($clickvalue, $window, $player)
		{
		$cmd['author'] = $player;
		$this->replaytrack($cmd);
		}  //  replaytrackAP

	function replaytrack(&$command)
		{
		$admin = $command['author'];
		$uid = $this->Aseco->server->challenge->uid;

		// cancel possibly ongoing replay vote
		$this->Aseco->client->query('CancelVote');
		$junk = $this->Aseco->client->getResponse();

		// check if track already in jukebox
		if (array_key_exists($uid, $this->jukebox))                             // find by uid in jukebox
			{
			$message = $this->Aseco->formatColors($this->messages['TRACKINJB']);
			$this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $admin->login);
			return;
			}

		// prepend current challenge to start of jukebox
		$jb = array();
		$jb[$uid]['FileName'] = $this->Aseco->server->challenge->filename;
		$jb[$uid]['Name'] = $this->Aseco->server->challenge->name;
		$jb[$uid]['Login'] = $admin->login;
		$jb[$uid]['Nick'] = $admin->nickname;
		$jb[$uid]['tmx'] = false;
		$jb[$uid]['uid'] = $uid;
		$jb[$uid]['env'] = $this->Aseco->server->challenge->environment;
		// prepend this track to the jukebox
		$this->jukebox = $jb + $this->jukebox;

		// log console message
		$this->Aseco->console('[{1}] requeued challenge!', $admin->login);

		// show chat message
		$message = formatText('{#server}>> $z$s {#highlite}{1}$z$s queues challenge for replay!!', stripFormatting($admin->nickname));

		$this->Aseco->client->query('ChatSendServerMessage', $this->Aseco->formatColors($message));
		}

	function getNextJBEntry()
		{
		$rtn = array_shift($this->jukebox);
		$this->jukebox = array($rtn['uid']=>$rtn) + $this->jukebox;
		}  //  getNextJBEntry

	}

$_PLUGIN = new JukeBox();
$_PLUGIN->setVersion(1.5);
$_PLUGIN->addDependence('LocalDb', 1.0, null);
$_PLUGIN->addEvent('onEndRace', 'endrace');
$_PLUGIN->addEvent('onNewChallenge', 'newtrack');
$_PLUGIN->addChatCommand('admin writetracklist', 'writetracklist', 'Write current tracklist and settings (filename optional)', 'writetracklist');
$_PLUGIN->addChatCommand('admin clearjukebox', 'clearjukebox', 'Clear tracks in the JukeBox', 'jb_admin');
$_PLUGIN->addChatCommand('admin jbclear', 'clearjukebox', 'Clear tracks in the JukeBox', 'jb_admin');
$_PLUGIN->addChatCommand('admin jbskip', 'skipjukebox', 'Skip the next track in the JukeBox', 'jb_admin');
$_PLUGIN->addChatCommand('admin add', 'adminadd', 'Adds a track to the server no voting, directly from TMX', 'jb_add');
$_PLUGIN->addChatCommand('admin remove', 'adminremove', 'Remove a track from the tracklist', 'jb_admin');
$_PLUGIN->addChatCommand('admin erase', 'adminerase', 'Erase a track from the server', 'jb_admin');
$_PLUGIN->addChatCommand('admin replaymap', 'replaytrack', 'Stuff current track into the jukebox so it\'s up again next', 'jb_admin');
$_PLUGIN->addChatCommand('admin addcancel', 'chat_admin_cancel', 'Cancel a TMX vote for a track', 'jb_admin');
$_PLUGIN->addChatCommand('addcancel', 'chat_cancel', 'Cancel a TMX vote for a track that you started', 'jb_admin');
$_PLUGIN->addChatCommand('jukebox', 'jb', 'Sets a track to be played next');
$_PLUGIN->addChatCommand('list', 'listtracks', 'Lists tracks currently on the server');
$_PLUGIN->addChatCommand('add', 'addtrack', 'Adds a track to the server, directly from TMX');
$_PLUGIN->addChatCommand('y', 'chat_y', 'Votes for a TMX map');
$_PLUGIN->addChatCommand('tlist', 'tmxSearch', 'List tracks from TMX (/tlist help for more)');
$_PLUGIN->addChatCommand('nextmap', 'chat_nextmap', 'Show next map to be played');
$_PLUGIN->addEvent('onPlayerServerMessageAnswer', 'event_jukebox_message');
$_PLUGIN->addEvent('onStartup', 'preload_jb_buffer'); //Preload buffer with recently played tracks
$_PLUGIN->addEvent('onSync', 'onSync');
?>
