<?php
// Match Settings Loader
//
// Parse directory that contains matchsettings files, display them for user
// display includes mode & number of tracks


class RaspMatchSettings extends Plugin
	{
	private $settingsdir = '/MatchSettings';		// dynamically changed to be / or \ depending on OS
	private $sepchar = '';

	function onSync()
		{
		$this->sepchar = substr($this->Aseco->server->trackdir, -1, 1);

		if ( $this->sepchar == '\\' )
			{
			$this->settingsdir = str_replace('/', $this->sepchar, $this->settingsdir);
			}

		$this->settingsdir = $this->Aseco->server->trackdir . $this->settingsdir;

		if ( !file_exists($this->settingsdir) )
			{
			$this->Aseco->console_text('[RASP MS] Error - MatchSettings Directory (' . $this->settingsdir . ') does not exist');
			$this->feature_tmxadd = false;
			}

		if ( !is_writeable($this->settingsdir) )
			{
			$this->Aseco->console_text('[RASP MS] Error - MatchSettings Directory (' . $this->settingsdir . ') can not be written to');
			$this->feature_tmxadd = false;
			}
		}  //  startup

	function list_by_ext($extension, $path)
		{
		$extension = strtolower($extension);
    	$list = array(); //initialise a variable
    	$dir_handle = @opendir($path) or die("Unable to open $path"); //attempt to open path
    	while($file = readdir($dir_handle)) //loop through all the files in the path
			{
        	if($file == "." || $file == "..") { continue; } //ignore these
        	$filename = explode(".",$file); //seperate filename from extenstion
        	$cnt = count($filename);
			$cnt--;
			$ext = $filename[$cnt]; //as above
        	if(strtolower($ext) == $extension)	 //if the extension of the file matches the extension we are looking for...
				{
            	array_push($list, $file); //...then stick it onto the end of the list array
				}
        	}
    	if($list[0])		//...if matches were found...
			{
    		return $list;	// ...return the array
    		}
		else 				// otherwise...
			{
    		return false;
			}
		}

	function matchSettings(&$command)
		{
		$admin = $command['author'];
		$filelist = $this->list_by_ext('txt', $this->settingsdir);
		$filearray = array();
		if ( !$filelist )
			{
			$msg = 'No files found. I wonder where the op is hiding them......';
			$this->Aseco->client->query('ChatSendServerMessageToLogin', $msg, $admin->login);
			return;
			}

		$header = "<line>
	  <cell width='1.03'><text halign='center'>Load Match Settings</text></cell></line>
	<line height='.04'>
	<cell width='0.71' bgcolor='888E'><text halign='left'>Filename</text></cell>
	<cell width='0.18' bgcolor='888E'><text halign='center'>Mode</text></cell>
	<cell width='0.14' bgcolor='888E'><text halign='center'>#Tracks</text></cell></line>";

		$detail = "<line>
		<cell width='0.71'><text action='{name:loadSettings,param:{FILENUM}}' halign='left'>{FILENAME}</text></cell>
		<cell width='0.18'><text halign='center'>{MODE}</text></cell>
		<cell width='0.14'><text halign='center'>{TRACKCOUNT}</text></cell></line>\r\n";

		$footer = "<line height='0.02'><cell width='0.94'></cell></line><line height='0.05'><cell bgcolor='FFF5' width='0.94'><text halign='center'>Click on file name to load track settings</text></cell></line><line height='0.02'><cell width='0.94'></cell></line>";

		$filenum = 0;
		$content = array();
		$ctr = 0;
		$page = 0;
		$content[0] = $header;
		foreach($filelist as $filename)
			{
			$settings = $this->Aseco->xml_parser->parseXML($this->settingsdir . $this->sepchar . $filename);
			if ( !isset($settings['PLAYLIST']) )
				{
				echo "Bad data in file '" . $filename . "'" . CRLF;
				continue;
				}
//			print_r($settings); echo CRLF;
			$mode = $settings['PLAYLIST']['GAMEINFOS'][0]['GAME_MODE'][0];
			switch ($mode)
				{
				case 0:
					$mode = "Rounds";
					break;
				case 1:
					$mode = "TimeAttack";
					break;
				case 2:
					$mode = "Team";
					break;
				case 3:
					$mode = "Laps";
					break;
				case 4:
					$mode = "Stunts";
					break;

				case 5:
					$mode = "Cup";
					break;

				default:
					$mode = "Undefined";
					break;
				}

			$stgout = $detail;
			$stgout = str_replace('{FILENAME}', htmlspecialchars($filename), $stgout);
			$stgout = str_replace('{FILENUM}', $filenum, $stgout);
			$stgout = str_replace('{MODE}', $mode, $stgout);
			$trackcount = sizeof($settings['PLAYLIST']['CHALLENGE']);
			$stgout = str_replace('{TRACKCOUNT}', $trackcount, $stgout);

			$filearray[] = array('Filename'=>$filename, 'Mode'=>$mode, 'Count'=>$trackcount);

			if ( !isset($content[$page]) )
				{
				$content[$page] = $header;
				}
			$content[$page] .= $stgout;
			$ctr++;
			if ( $ctr >= 12 )
				{
				$content[$page] .= $footer;
				$ctr = 0;
				$page++;
				}
			$filenum++;
			}

		$admin->data['matchfiles'] = $filearray;

    	$window = $this->createWindowToPlayerFramework('MatchSettings', 1.05, 0.9, $admin);
		$window->setContent($content);
		$window->setWidth(1.05);
		$window->addCallback('loadSettings', 'loadSettings');
		}  //  matchSettings

	function loadSettings($clickvalue, $window, $player)
		{
		$window->close();

		if ( isset($this->Aseco->plugins['JukeBox']) )					// jukebox plugin running?
			{
			if ( $this->Aseco->plugins['JukeBox']->isTrackInJB() )			// only clear jukebox if anything is in it
				{
				$this->Aseco->plugins['JukeBox']->clearjukebox();		// sends msg to all
				}
			}
		// do something about loading the setting file here
		// if jukebox is running (the script), clear out the jukebox because chances are that the new list won't have the track in it

		$filename = $player->data['matchfiles'][$clickvalue]['Filename'];
		$trackcount = $player->data['matchfiles'][$clickvalue]['Count'];
		$mode = $player->data['matchfiles'][$clickvalue]['Mode'];

		$this->Aseco->console("Admin (" . $player->login . ") loaded match settings file $filename");

		$this->Aseco->client->query('LoadMatchSettings', 'matchsettings' . $this->sepchar . $filename);
		$response = $this->Aseco->client->getResponse();
		$msg = '{#server}>> New Match Settings. Mode: {#highlite}' . $mode . '  {#server}Tracks: {#highlite}' . $trackcount;
		$msg = $this->Aseco->formatColors($msg);
		$this->Aseco->client->query('ChatSendServerMessage', $msg);
		}  //  jbtrack

	}
$_PLUGIN = new RaspMatchSettings();
$_PLUGIN->setAuthor('Assembler Maniac');
$_PLUGIN->setVersion(1.0);
$_PLUGIN->setDescription('Match Settings Loader');
$_PLUGIN->addEvent('onSync', 'onSync');
$_PLUGIN->addChatCommand('admin loadms', 'matchSettings', 'Load new tracklist/matchsettings');
