<?php
// Display next track info during intermission
// shows num laps/points or time limit depending on game mode
// Rounds (0), TimeAttack (1), Team (2), Laps (3) and Stunts (4).

class RaspShowNext extends Plugin
	{
	function onEndRace(&$info)
		{
		// get next server mode
		$this->Aseco->client->query('GetNextGameInfo');
		$GameInfo = $this->Aseco->client->getResponse();
		$GameMode = $GameInfo['GameMode'];

		$failpoint = '';
		$failed = false;
		$this->Aseco->client->query("GetNextChallengeIndex");
		$next_index = $this->Aseco->client->getResponse();
		// do GetChallengeList and send it the next index, this way you avoid looping through data to find the right one later
		$this->Aseco->client->query("GetChallengeList", 1, $next_index);
		$challenge = $this->Aseco->client->getResponse();
		if ( $challenge['faultString'] == 'Start index out of bound.' )		// tried to get track xx, but only xx-1 tracks exist, get track 0 instead - using getnextchallengeindex this should NEVER occur
			{
			$this->Aseco->client->query("GetChallengeList", 1, 0);
			$challenge = $this->Aseco->client->getResponse();
			$this->Aseco->client->error->code = 0;
			$this->Aseco->client->error->message = '';
			}
		else
			{
			$filename = $challenge[0]['FileName'];
			}

		if ( !$failed )
			{
			$this->Aseco->client->query("GetChallengeInfo", $filename);
			$result = $this->Aseco->client->getResponse();
			if ( isset($result['faultCode'] ) )
				{
				$this->Aseco->console_text('[RaspShowNext] - \'' . $filename . '\' - unable to get track info');
				$this->Aseco->console_text('[RaspShowNext] - GetChallengeInfo failed - [' . $result['faultCode'] . '] - ' . $result['faultString']);
				$failed = true;
				$failpoint = 'GetChallengeInfo';
				}
			}

		if ( $failed )
			{
			$automsg = $failpoint . ' failed, no data shown';
			$trackinfo = array();		// clear it out so it doesn't crash
			}
		else
			{
			$trackinfo = $result;
			}

/*
Datablock in $result[0][0]

Array
(
    [UId] => nIGJHQkY_Wy5mt11GnWtcLo_zaf
    [Name] => $s$f80new mix on the block$ffc$sz
    [FileName] => new mix on the blockz.Challenge.gbx
    [Author] => z3383z
    [Environnement] => Stadium
    [Mood] => Sunset
    [BronzeTime] => 74000
    [SilverTime] => 59000
    [GoldTime] => 52000
    [AuthorTime] => 48790
    [CopperPrice] => 3779
    [LapRace] =>
)
 */

		$this->Aseco->client->query("GetTimeAttackLimit", $newtime);
		$result = $this->Aseco->client->getResponse();
		if (!$result)
			{
			trigger_error('[' . $this->Aseco->client->getErrorCode() . '] ' . $this->Aseco->client->getErrorMessage());
			}
		else
			{
			$nexttime = $result['NextValue'];
			if ( isset($this->Aseco->plugins['RaspAutotime']) )
				{
				$junk = "";		// calcautotime takes an arg for use in autotime script only
				$nexttime = $this->Aseco->plugins['RaspAutotime']->calcAutoTime($trackinfo['AuthorTime'], $junk);
				}
			$leftcell = "<cell width='0.125' bgcolor='000F'><text textcolor='37d' halign='right'>";
			$rightcell = "<cell width='0.275' bgcolor='000F'><text textcolor='8af' halign='left'>";
			$content = "<line><cell bgcolor='888F' width='.4'><text textcolor='0E0' halign='center'>Next Track Info</text></cell></line>
						<line height='.03'>
							" . $leftcell . "Name: </text></cell>"
							. $rightcell . htmlspecialchars(stripFormatting($trackinfo['Name'])) . "</text></cell>
						</line>
						<line height='.03'>
							" . $leftcell . "Env: </text></cell>"
							. $rightcell . $trackinfo['Environnement'] . "</text></cell>
						</line>
						<line height='.03'>
							" . $leftcell . "Mood: </text></cell>"
							. $rightcell . $trackinfo['Mood'] . "</text></cell>
						</line>
						<line height='.03'>
							" . $leftcell . "Author: </text></cell>"
							. $rightcell . $trackinfo['Author'] . "</text></cell>
						</line>";

    		switch ($GameMode)
				{
      			case 0:		// Rounds
					$c1l = 'Mode:';
					$c1r = 'Rounds';
					$c2l = 'Points:';
					$c2r = $GameInfo['RoundsPointsLimit'];
					break;

				case 1:		// TimeAttack
					$c1l = 'Author Time:';
					$c1r = formatTime($trackinfo['AuthorTime']);
					$c2l = 'Time Limit:';
					$c2r = formatTime($nexttime);
					break;

				case 2:		// Teams
					$c1l = 'Mode:';
					$c1r = 'Teams';
					$c2l = 'Points:';
					$c2r = 'Limit ' . $GameInfo['TeamsPointsLimit'] . ' / Max ' . $GameInfo['TeamMaxPoints'];
					break;

				case 3:		// Laps
					$c1l = 'Laps:';
					$c1r = $GameInfo['LapsNbLaps'];
					$c2l = 'Time Limit:';
					$c2r = formatTime($GameInfo['LapsTimeLimit']);
					break;

				case 4:		// Stunts
					$c1l = 'Mode:';
					$c1r = 'Stunts';
					$c2l = '';
					$c2r = '';
					break;
				}

			$content .= "<line height='.03'>
							" . $leftcell . $c1l . " </text></cell>"
							. $rightcell . $c1r . "</text></cell>
						</line>
						<line height='.03'>
							" . $leftcell . $c2l . " </text></cell>"
							. $rightcell . $c2r . "</text></cell>
						</line>";

			foreach($this->Aseco->server->players->player_list as $pl)
				{
				if ( !isset($pl->data['w_shownext'] ) )
					{
    				$window = $this->createWindowToPlayerFramework('', -.5, .745, $pl, true);
//					$window->makeStatic();
					$window->setContent($content);
					$window->setBackground('000F');
					$window->setBorderSize(0);
					$window->setTextsize(1);
					$pl->data['w_shownext'] = $window;
					}
				else
					{
					$window = $pl->data['w_shownext'];
					$window->setContent($content);
					}
				$window->show();
				}
			}
		}  //  onStatusChange

	function onStatusChange3()
		{
		foreach($this->Aseco->server->players->player_list as $pl)
			{
			if ( isset($pl->data['w_shownext'] ) )
				{
				$window = $pl->data['w_shownext'];
				$window->hide();
				}
			}
		}  //  onStatusChange4

	}

$_PLUGIN = new RaspShowNext();
$_PLUGIN->setAuthor('Assembler Maniac');
$_PLUGIN->setVersion(1.0);
$_PLUGIN->setDescription('Show Track Info During Intermission');
$_PLUGIN->addEvent('onEndRace', 'onEndRace');
$_PLUGIN->addEvent('onStatusChangeTo3', 'onStatusChange3');
