<?php
/**
 * Structure of an Player.
 * Can be instanciated with a rpc "GetDetailedPlayerInfo" response.
 *
 */
class Player {
  var $login;
  var $nickname;
  var $id;
  var $spectatorstatus;
  var $isofficial;
  var $ladderposition;
  var $ladderscore;
  var $ip;
  var $nation;
  var $hasvoted;
  var $paths;
  var $rights;
  var $IsReferee;
  var $idleCount;
  var $created;
  var $wins;
  var $data; // can be used to store additional information
  var $framework;
  
  

  function getWins() {
    return $this->wins;
  }

  function getTimeOnline() {
    return time() - $this->created;
  }

  // instanciates a player with an rpc response ...
  function Player($rpc_infos = null) {
    if ($rpc_infos) {
//print_r($rpc_infos); echo CRLF;
	  $this->login = $rpc_infos['Login'];
      $this->nickname = $rpc_infos['NickName'];
      $this->id = $rpc_infos['PlayerId'];
      $this->ip = $rpc_infos['IPAddress'];
      /* SpectatorStatus long gone from the game -cav */
      $this->spectatorstatus = $rpc_infos['IsSpectator'];
      $this->isofficial = $rpc_infos['IsInOfficialMode'];
      /* Handy to know who is behind the black referee screen -cav */
      $this->isreferee = $rpc_infos['IsReferee'];
      $this->teamid = $rpc_infos['TeamId'];			// only valid for team mode
      /*Rank and score are back, just buried deep in an array -cav */
      $this->ladderposition = $rpc_infos['LadderStats']['PlayerRankings'][0]['Ranking'];
      $this->ladderscore = $rpc_infos['LadderStats']['PlayerRankings'][0]['Score'];
      /* OnlineRights checks players ability to spend coppers. 0 = basic nations account with no rights, 1 = one month tmu upgrade, not likely to have many coppers, 2 = six months tmu, 3 = full version tmu -cav */
      $this->rights = $rpc_infos['OnlineRights'];
      /* Taken from Mistral mod 5, had to increase nation db table from 3 characters to 60 -cav */
      $this->paths = $rpc_infos['Path'];
      $this->nation = substr($this->paths, strpos($this->paths, '|')+1);
      $this->created = time();
    }
    $this->hasvoted = false;
    $this->wins = 0;
    $this->newwins = 0;
    $this->idleCount = 0;
    $this->framework = new ManialinkFramework();
    $this->framework->setOwner($this);
  }

  function storeData($key, $value) {
    $this->data[$key] = $value;
  }

  function removeData($key) {
    if (isset($this->data[$key]))
      unset($this->data[$key]);
  }

  function existsData($key) {
    return isset($this->data[$key]);
  }

  function getData($key) {
    return $this->data[$key];
  }

  function dispose() {

    // clear data ...
    if (count($this->data) > 0) {
      foreach($this->data as $value) {
        if (get_class($value) == 'ManialinkPage') {
          $value->close();
          $value->dispose();
          unset($value);
        }
      }
    }

    $this->framework->dispose();
    unset($this->framework);
  }

  function __destruct() {
    // echo "destroying player ...\n";
  }
}

/**
 * Structure of an RecPlayer.
 * Just for storing player information in a record object.
 *
 */
class RecPlayer {
  var $id;
  var $login;
  var $nickname;

  function __construct($in_player = null) {
    if ($in_player != null && get_class($in_player) == 'Player') {
      $this->id = $in_player->id;
      $this->login = $in_player->login;
      $this->nickname = $in_player->nickname;
    }
  }
}
?>
