<?php
/**
 * This is the structure for all plugins.
 * Extend this class to build your own one.
 *
 */
abstract class Plugin {
  private $_name;
  private $_author;
  private $_version;
  private $_dependencies;
  private $_events;
  private $_chat_commands;
  private $_description;
  private $_silent;
  private $_manialink_callbacks;
  private $_website;
  public $Aseco;

  function __construct() {
    $this->_name = get_class($this);
    $this->_dependencies = array();
    $this->_events = array();
    $this->_chat_commands = array();
    $this->_description = 'no Description'.LF;
    $this->_silent = false;
  }

  public function addDependence($in_plugin_name, $in_min_ver = null, $in_max_ver = null) {
    $this->_dependencies[] = new Dependence($in_plugin_name, $in_min_ver, $in_max_ver);
  }

  public function addEvent($in_type, $in_callback) {
    $this->_events[] = new Event($in_type, $in_callback, $this);
  }

  public function addChatCommand($in_name, $in_callback, $in_help, $in_adminitem = '') {
    $is_admin = false;
    if (substr($in_name, 0, 5) == 'admin')
		{
		$is_admin = true;
		if ( $in_adminitem == '' )
			{
			$in_adminitem = 'AdminSuper';
			}
		}
    $this->_chat_commands[] = new ChatCommand($in_name,
                                              $in_callback,
                                              $in_help,
                                              $is_admin,
                                              $this,
											  $in_adminitem);
  }

  public function clearChatCommands() {
    foreach($this->_chat_commands as $obj) {
      unset($this->Aseco->chat_commands[$obj->name]);
      unset($obj);
    }
    $this->_chat_commands = array();
  }

  /**
   * Enter description here...
   *
   * @param string $title
   * @param float $width
   * @param float $height
   * @param Player $player
   * @return ManialinkPage
   */
  public function createWindowToPlayerFramework($title, $width, $height, $player, $is_static = false) {
	return $player->framework->createWindow($title, $width, $height, $this, $is_static);
  }

  public function sendPopupWindowToPlayer($login, $message, $title = 'Popup', $timeout = 0) {
    $player = $this->Aseco->server->players->getPlayer($login);
    return $player->framework->createPopup($title, $message, $timeout);
  }

  public function sendServerMessageToLogin($login, $message) {
    if ($this->_silent | strlen($message) == 0 | !isset($message)) return;
    $this->Aseco->client->query('ChatSendServerMessageToLogin', $message, $login);		// 08-4-23 BTM - no more waiting, send when you get it
  }

  public function sendServerMessage($message) {
    if ($this->_silent | strlen($message) == 0 | !isset($message)) return;
    $this->Aseco->client->query('ChatSendServerMessage', $message);						// 08-4-23 BTM - no more waiting, send when you get it
  }

  public function console() {
    $args = func_get_args();
    $args[0] = '['.$this->getName().'] '.$args[0];
    call_user_func_array(array($this->Aseco, 'console'), $args);
  }

  public function console_text() {
    $args = func_get_args();
    $args[0] = '['.$this->getName().'] '.$args[0];
    call_user_func_array(array($this->Aseco, 'console_text'), $args);
  }

  public function console_debug() {
    $args = func_get_args();
    $args[0] = '['.$this->getName().'] '.$args[0];
    call_user_func_array(array($this->Aseco, 'console_debug'), $args);
  }

  public function trigger_error($message) {
    trigger_error('['.$this->getName().'] '.$message, E_USER_ERROR);
  }

  public function trigger_warning($message) {
    trigger_error('['.$this->getName().'] '.$message, E_USER_WARNING);
  }

  public function storePlayerData($player, $key, $data) {
    $player->storeData($this->getName().'_'.$key, $data);
  }

  public function getPlayerData($player, $key) {
    return $player->getData($this->getName().'_'.$key);
  }

  public function removePlayerData($player, $key) {
	if ( !isset($player) ) return;
    $player->removeData($this->getName().'_'.$key);
  }

  public function existsPlayerData($player, $key) {
    return $player->existsData($this->getName().'_'.$key);
  }

  public function getChatCommands() {
    return $this->_chat_commands;
  }

  public function setVersion($in_version) {
    $this->_version = $in_version;
  }

  public function getVersion() {
    return $this->_version;
  }

  public function setAuthor($in_author) {
    $this->_author = $in_author;
  }

  public function getAuthor() {
    return $this->_author;
  }

  public function getEvents() {
    return $this->_events;
  }

  public function getDependencies() {
    return $this->_dependencies;
  }

  public function getName() {
    return $this->_name;
  }

  public function getDescription() {
    return $this->_description;
  }

  public function setDescription($in_desc) {
    $this->_description = wordwrap($in_desc, 50, LF, true);
  }

  public function setSilent($in_state) {
    $this->_silent = (bool)$in_state;
  }

  public function setWebsite($url) {
    $this->_website = $url;
  }

  public function getWebsite() {
    return $this->_website;
  }
}
?>
