<?php
/**
 * Manages a list of records.
 * Add records to the list and remove them.
 */
class RecordList {
  private $record_list;
  private $max;
  private $mode;
  private $settings;

  function RecordList($limit, $gamesettings) {
    $this->record_list = array();
    $this->max = $limit;
    $this->settings = $gamesettings;
  }

  function setLimit($limit) {
    $this->max = $limit;
  }
  
  function getLimit() {
  	return $this->max;
  }

  function getRecord($rank) {
    return $this->record_list[$rank];
  }

  function setRecord($rank, $record) {
    if(isset($this->record_list[$rank])) {
      return $this->record_list[$rank] = $record;
    } else {
      return false;
    }
  }

  function moveRecord($from, $to) {
    moveArrayElement($this->record_list, $from, $to);
  }

  /**
   * Adds a new record to the list.
   *
   * Return values are:
   * first array element:
   * -2(error occured), -1(no record), 0(record updated), 1(record updated, rank improved), 2(new record)
   * second array element:
   * new rank, or -1 when first element is -1 or -2
   * 
   * @return mixed rank
   */
  function addRecord($record, $rank = -1) {
    
    $this->mode = $this->settings->getMode();
    
    // if no rank was set for this record, then calculate the right position ...
    if ($rank == -1) {
      for ($i = 0; $i < count($this->record_list); $i++) {
        if ((($this->mode == 'Stunts' && $record->score > $this->record_list[$i]->score) ||
              ($this->mode != 'Stunts' && $record->score < $this->record_list[$i]->score) || !isset($this->record_list[$i]->score)) &&
              $record->score > 0) {
          $rank = $i;
          break;
        }
      }
      if ($rank == -1) $rank = count($this->record_list);
    }
    // check if player has a record already ...
    for ($i = 0; $i < count($this->record_list); $i++) {
      
      // if record holder is the same player ...
      if ($record->player->login == $this->record_list[$i]->player->login) {
        
        // if the new record is better than the old one of this player ...
        if (($this->mode != 'Stunts' && $record->score < $this->record_list[$i]->score) ||
            ($this->mode == 'Stunts' && $record->score > $this->record_list[$i]->score)) {

          // then replace it and move it ...
          if ($rank == $i) {
            $this->record_list[$i] = $record;
            return array(0, $rank); // record updated!
          } else {
            $this->record_list[$i] = $record;
            $this->moveRecord($i, $rank);
            return array(1, $rank); // record updated, rank improved!
          }
          
        } else {
          
          // if the old one is better, then do nothing ...
          return array(-1, -1); // no record!
        }
      }
    }

    // do not insert a record behind the border of the list ...
    if ($rank >= $this->max) return array(-1, -1); // no record!

    // do not insert a record with no score ...
    if ($record->score <= 0) return array(-2, -1); // error!

    // if the given object is a record ...
    if (get_class($record) == "Record") {

      // if records are getting too much, drop the last from the list ...
      if (count($this->record_list) >= $this->max) {
        array_pop($this->record_list);
      }

      // insert the record on the specific position ...
      if (insertArrayElement($this->record_list, $record, $rank)) {
        return array(2, $rank); // new record!
      } else {
        return array(-2, -1); // error!
      }
    } else {
      return array(-2, -1); // error!
    }
  }

  function count() {
    return count($this->record_list);
  }

  function clear() {
    unset($this->record_list);
    $this->record_list = array();
  }
}
?>
